<?php
// Copyright (C) 2010-2012 Combodo SARL
//
//   This file is part of TeemIp.
//
//   TeemIp is free software; you can redistribute it and/or modify	
//   it under the terms of the GNU Affero General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   TeemIp is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Affero General Public License for more details.
//
//   You should have received a copy of the GNU Affero General Public License
//   along with TeemIp. If not, see <http://www.gnu.org/licenses/>

/**
 * @copyright   Copyright (C) 2010-2012 Combodo SARL
 * @license     http://opensource.org/licenses/AGPL-3.0
 */

class _IPv4Address extends IPv4Object
{
	/**
	 * Get the subnet mask of the subnet that the IP belongs to, if any.
	 */
	function GetSubnetMaskFromIp()
	{
		$sIp = $this->Get('ip');
		$sOrgId = $this->Get('org_id');
		$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE INET_ATON(s.ip) <= INET_ATON('$sIp') AND INET_ATON('$sIp') <= INET_ATON(s.broadcastip) AND s.org_id = $sOrgId"));
		if ($oSubnetSet->Count() != 0)
		{ 
			$oSubnet = $oSubnetSet->Fetch();
			return ($oSubnet->Get('mask'));
		}
		return '255.255.255.255';
	}
	
	/**
	 * Get the gateway of the subnet that the IP belongs to, if any.
	 */
	function GetSubnetGatewayFromIp()
	{
		$sIp = $this->Get('ip');
		$sOrgId = $this->Get('org_id');
		$sSubnetGateway = GetFromGlobalIPConfig('subnet_gateway', $sOrgId);
		$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE INET_ATON(s.ip) <= INET_ATON('$sIp') AND INET_ATON('$sIp') <= INET_ATON(s.broadcastip) AND s.org_id = $sOrgId"));
		if ($oSubnetSet->Count() != 0)
		{ 
			$oSubnet = $oSubnetSet->Fetch();
			switch ($sSubnetGateway)
			{
				case 'subnet_ip_plus_one':
					$iGwIp = myip2long($oSubnet->Get('ip')) + 1;
					return (mylong2ip($iGwIp));
					
				case 'subnet_broadcast_minus_one':
					$iGwIp = myip2long($oSubnet->Get('broadcastip')) - 1;
					return (mylong2ip($iGwIp));

				case 'undefined':
				default:
					return null;
			}
		}
		return null;
	}
	
	/**
	 * Update attributes of IP Interface when Ip is created or updated.
	 */
	function UpdateIPInterface($iIPInterfaceId)
	{
		$oIPInterface = MetaModel::GetObject('IPInterface', $iIPInterfaceId, false /* MustBeFound */);
		if (!is_null($oIPInterface)) 
		{
			// Prepare history records
			$oMyChange = MetaModel::NewObject("CMDBChange");
			$oMyChange->Set("date", time());
			$sUserString = CMDBChange::GetCurrentUserName();
			$oMyChange->Set("userinfo", $sUserString);
			$iChangeId = $oMyChange->DBInsert();
			
			if (MetaModel::IsValidAttCode('IPInterface', 'ipaddress_id'))
			{
				$oAttDef = MetaModel::GetAttributeDef('IPInterface', 'ipaddress_id');
				if ($oAttDef->IsExternalKey() && ($oIPInterface->Get('ipaddress_id') != $this->GetKey()))
				{
					$oIPInterface->Set('ipaddress_id', $this->GetKey());
					$oIPInterface->DBUpdateTracked($oMyChange);
				}
			}
		}
	}
	
	/**
	 * Displays the tab listing the Global Parameters
	 */
	function DisplayBareRelations(WebPage $oPage, $bEditMode = false)
	{
		$sOrgId = $this->Get('org_id');
		if ($sOrgId != null)
		{
			// Tab for Global Parameters at creation time only
			if ($bEditMode)
			{
				// Execute parent function first 
				parent::DisplayBareRelations($oPage, $bEditMode);
			
				// Then global parameters
				$oPage->SetCurrentTab(Dict::Format('Class:IPv4Address/Tab:globalparam'));
				$oPage->p(Dict::Format('UI:IPManagement:Action:Modify:IPv4Address:GlobalConfig'));
				$oPage->add('<table style="vertical-align:top"><tr>');
				$oPage->add('<td style="vertical-align:top">');
				
				if ($this->IsNew())
				{
					$sParameter = array ('ip_allow_duplicate_name', 'ping_before_assign', null);
				}
				else
				{
					$sParameter = array ('ip_allow_duplicate_name', null);
				}
				DisplayGlobalParametersInLocalModifyForm($oPage, $sParameter, $sOrgId);      
				
				$oPage->add('</td>');
				$oPage->add('</tr></table>');
			}
			else
			{
				// Tab for parent objects
				$sIp = $this->Get('ip');
				$aDetails = array();
				
				// Look for parent subnets (should exist)
				$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE INET_ATON(s.ip) <= INET_ATON('$sIp') AND INET_ATON('$sIp') <= INET_ATON(s.broadcastip) AND s.org_id = $sOrgId"));
				while ($oSubnet = $oSubnetSet->Fetch())
				{
					// Display Block first
					$sDisplayValue = $oSubnet->GetAsHTML('block_id');	
					$aDetails[] = array('label' => '<span title="'.MetaModel::GetDescription('IPv4Subnet', 'block_id').'">'.MetaModel::GetLabel('IPv4Subnet', 'block_id').'</span>', 'value' => $sDisplayValue);
					
					// Display Subnet then          
					$sDisplayValue = $oSubnet->GetHyperlink();	
					$aDetails[] = array('label' => '<span title="'.MetaModel::GetName('IPv4Subnet').'">'.MetaModel::GetName('IPv4Subnet').'</span>', 'value' => $sDisplayValue);
				}
				
				// Look for parent range
				$oIpRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Range AS r WHERE INET_ATON(r.firstip) <= INET_ATON('$sIp') AND INET_ATON('$sIp') <= INET_ATON(r.lastip) AND r.org_id = $sOrgId"));
				while ($oIpRange = $oIpRangeSet->Fetch())
				{
					$sDisplayValue = $oIpRange->GetHyperlink();	
					$aDetails[] = array('label' => '<span title="'.MetaModel::GetName('IPv4Range').'">'.MetaModel::GetName('IPv4Range').'</span>', 'value' => $sDisplayValue); 		      
				}
				
				$oPage->SetCurrentTab(Dict::Format('Class:IPv4Address/Tab:parents'));
				$oPage->add('<table style="vertical-align:top"><tr>');
				$oPage->add('<td style="vertical-align:top">');
				$oPage->Details($aDetails);
				$oPage->add('</td>');
				$oPage->add('</tr></table>');
				
				// Tab for NAT IPs
				$iKey = $this->GetKey();
				$oNatIpSearch = DBObjectSearch::FromOQL("SELECT lnkIPv4AdressToIPv4Address AS ln WHERE (ln.ip1_id = $iKey OR ln.ip2_id = $iKey)");
				$oNatIpSet = new CMDBObjectSet($oNatIpSearch);
				$oIpSet = array();
				while ($oNatIp = $oNatIpSet->fetch())
				{
					if ($oNatIp->Get('ip1_id') == $iKey)
					{
						$iIpKey = $oNatIp->Get('ip2_id');
					}
					else
					{
					$iIpKey = $oNatIp->Get('ip1_id');
					}
					$oIpSet[] = MetaModel::GetObject('IPv4Address', $iIpKey, false);
				}
				$oSet = CMDBObjectSet::FromArray('IPv4Address', $oIpSet);
				$oPage->SetCurrentTab(Dict::Format('Class:IPv4Address/Tab:ip_list', $oNatIpSet->Count()));
				$oPage->p(MetaModel::GetClassIcon('IPv4Address').'&nbsp;'.Dict::Format('Class:IPv4Address/Tab:ip_list+'));
				if ($oNatIpSet->Count() != 0)
				{
					$oBlock = DisplayBlock::FromObjectSet($oSet, 'list');
					$oBlock->Display($oPage, 'nat_ips', array('menu' => false));
				}
				
				// Tab for related IP Requests, if any
				if  (MetaModel::IsValidClass('IPv4AddressRequest'))
				{
					$oIpRequestSearch = DBObjectSearch::FromOQL("SELECT IPv4AddressRequest AS r WHERE r.ip_id = $iKey");
					$oIpRequestSet = new CMDBObjectSet($oIpRequestSearch);
					$sCount = $oIpRequestSet->Count();
					if ($sCount > 0)
					{
					$oPage->SetCurrentTab(Dict::Format('Class:IPv4Address/Tab:requests', $sCount));
					$oPage->p(MetaModel::GetClassIcon('IPObjRequest').'&nbsp;'.Dict::Format('Class:IPv4Address/Tab:requests+'));
					$oBlock = new DisplayBlock($oIpRequestSearch, 'list');
					$oBlock->Display($oPage, 'ip_requests', array('menu' => false));
					}
				}
				
				// Tab for related IP Changes, if any
				if  (MetaModel::IsValidClass('IPv4AddressChange'))
				{
					$oIpChangeSearch = DBObjectSearch::FromOQL("SELECT IPv4AddressChange AS c WHERE c.ip_id = $iKey");
					$oIpChangeSet = new CMDBObjectSet($oIpChangeSearch);
					$sCount = $oIpChangeSet->Count();
					if ($sCount > 0)
					{
						$oPage->SetCurrentTab(Dict::Format('Class:IPv4Address/Tab:changes', $sCount));
						$oPage->p(MetaModel::GetClassIcon('IPObjChange').'&nbsp;'.Dict::Format('Class:IPv4Address/Tab:changes+'));
						$oBlock = new DisplayBlock($oIpChangeSearch, 'list');
						$oBlock->Display($oPage, 'ip_changes', array('menu' => false));
					}
				}
			}
		}
	}
	
	/**
	 * Check validity of new IP attributes before creation
	 */
	public function DoCheckToWrite()
	{
		// Run standard iTop checks first
		parent::DoCheckToWrite();
		
		$sOrgId = $this->Get('org_id');
		$iKey = $this->GetKey();
		$sIp = $this->Get('ip');
		
		// Make sure IP doesn't already exist for creation
		$oIpSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE i.ip = '$sIp' AND i.org_id = $sOrgId"));
		while ($oIp = $oIpSet->Fetch())
		{
			// Check if it's a creation or a modification
			if ($iKey != $oIp->GetKey())
			{
				// It's a creation -> deny it
				$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Address:IPCollision');
				return;
			}
		}
		
		// Make sure name doesn't already exist (matches)
		if (! IsFqdnUnique($sOrgId, $iKey, $this->Get('short_name'), $this->Get('domain_name')))
		{
			$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Address:IPNameCollision');
			return;
		}
		
		// For new IPs only: if required by global parameters, ping IP before assigning it 
		//    See comment on organisation above
		if ($this->IsNew())
		{
			$sPingBeforeAssign = utils::ReadPostedParam('attr_ping_before_assign', '');
			if (empty($sPingBeforeAssign))
			{
				$sPingBeforeAssign = GetFromGlobalIPConfig('ping_before_assign', $sOrgId);
			}
			if ($sPingBeforeAssign =='ping_yes')
			{
				$aOutput = IpPings($sIp, TIME_TO_WAIT_FOR_PING_LONG);
				if (!empty($aOutput))
				{
					$sOutput = '';
					foreach ($aOutput as $line)
					{
						$sOutput .= $line."    ";
					}
					$this->m_aCheckIssues[] = Dict::S('UI:IPManagement:Action:New:IPv4Address:IPPings').$sOutput;
					return;
				}
			}
		}
	}
	
	/*
	 * Perform actions when new object inserted in DB 
	 */
	protected function OnInsert()
	{
		// Set raw IP attribute
		$this->Set('rip',$this->Get('ip'));
	}
	
	/*
	 * Perform actions after new object inserted in DB 
	 */
	protected function AfterInsert()
	{
		$sOrgId = $this->Get('org_id');
		$sIp = $this->Get('ip');
		
		// Check if link has been made to a IP Interface
		// If yes, update that object
		$iIpInterfaceId = $this->Get('ipinterface_id');
		if ($iIpInterfaceId != null)
		{
			$this->UpdateIPInterface($iIpInterfaceId);
		}
		
		// If Triggers are used
		if  (MetaModel::IsValidClass('IPTriggerOnWaterMark'))
		{
			// Look for subnet where IP belongs to (should be only one)
			//  Activate trigger if High Water Mark is crossed 
			//  Note: triggers don't apply for subnet which size is lower than /29 - 8 IPs
			$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE INET_ATON(s.ip) <= INET_ATON('$sIp') AND INET_ATON('$sIp') <= INET_ATON(s.broadcastip) AND s.org_id = $sOrgId"));
			$sSubnetLowWaterMark = GetFromGlobalIPConfig('subnet_low_watermark', $sOrgId);
			$sSubnetHighWaterMark = GetFromGlobalIPConfig('subnet_high_watermark', $sOrgId);
			while ($oSubnet = $oSubnetSet->Fetch())
			{
				if (MaskToSize($oSubnet->Get('mask')) > 4)
				{
					$oSubnetOccupancy = $oSubnet->GetOccupancy('IPv4Address');
					$oSubnetAlarm = $oSubnet->Get('alarm_water_mark');
					if ($oSubnetOccupancy >= $sSubnetHighWaterMark)
					{
						// If no HWM alarm sent yet, generate event: objet = $oSubnet, reason = HWM crossed
						if ($oSubnetAlarm != 'high_sent')
						{
							$oSubnet->Set('alarm_water_mark', 'high_sent');
							$oSubnet->DBUpdate();   
							$oTriggerSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPTriggerOnWaterMark AS t WHERE t.target_class = 'IPv4Subnet' AND t.event = 'cross_high' AND t.org_id = $sOrgId"));
							while ($oTrigger = $oTriggerSet->Fetch())
							{
								$oTrigger->DoActivate($oSubnet->ToArgs('this'));
							}
						}
					}
					else if ($oSubnetOccupancy >= $sSubnetLowWaterMark)
					{
						// If no LWM alarm sent yet, generate event: objet = $oSubnet, reason = LWM crossed
						if ($oSubnetAlarm != 'no_alarm')
						{
							$oSubnet->Set('alarm_water_mark', 'low_sent');
							$oSubnet->DBUpdate();   
							$oTriggerSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPTriggerOnWaterMark AS t WHERE t.target_class = 'IPv4Subnet' AND t.event = 'cross_low' AND t.org_id = $sOrgId"));
							while ($oTrigger = $oTriggerSet->Fetch())
							{
								$oTrigger->DoActivate($oSubnet->ToArgs('this'));
							}
						}
					}
				}
			}
			
			// Look for ranges where IP belongs to (should be only one)
			//  Generate event if High Water Mark is crossed 
			$oIpRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Range AS r WHERE INET_ATON(r.firstip) <= INET_ATON('$sIp') AND INET_ATON('$sIp') <= INET_ATON(r.lastip) AND r.org_id = $sOrgId"));
			$sIpRangeLowWaterMark = GetFromGlobalIPConfig('iprange_low_watermark', $sOrgId);
			$sIpRangeHighWaterMark = GetFromGlobalIPConfig('iprange_high_watermark', $sOrgId);
			while ($oIpRange = $oIpRangeSet->Fetch())
			{
				$oIpRangeOccupancy = $oIpRange->GetOccupancy('IPv4Address');
				$oIpRangeAlarm = $oIpRange->Get('alarm_water_mark');
				if ($oIpRangeOccupancy >= $sIpRangeHighWaterMark)
				{
					// If no HWM alarm sent yet, generate event: objet = $oSubnet, reason = HWM crossed
					if ($oIpRangeAlarm != 'high_sent')
					{
						$oIpRange->Set('alarm_water_mark', 'high_sent');
						$oIpRange->DBUpdate();   
						$oTriggerSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPTriggerOnWaterMark AS t WHERE t.target_class = 'IPv4Range' AND t.event = 'cross_high' AND t.org_id = $sOrgId"));
						while ($oTrigger = $oTriggerSet->Fetch())
						{
							$oTrigger->DoActivate($oIpRange->ToArgs('this'));
						}
					}
				}
				else if ($oIpRangeOccupancy >= $sIpRangeLowWaterMark)
				{
					// If no LWM alarm sent yet, generate event: objet = $oSubnet, reason = LWM crossed
					if ($oIpRangeAlarm != 'no_alarm')
					{
						$oIpRange->Set('alarm_water_mark', 'low_sent');
						$oIpRange->DBUpdate();   
						$oTriggerSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPTriggerOnWaterMark AS t WHERE t.target_class = 'IPv4Range' AND t.event = 'cross_low' AND t.org_id = $sOrgId"));
						while ($oTrigger = $oTriggerSet->Fetch())
						{
							$oTrigger->DoActivate($oIpRange->ToArgs('this'));
						}
					}
				}
			}
		}
	}
	
	/*
	 * Perform actions after existing object is updated in DB 
	 */     
	protected function AfterUpdate()
	{
		// Check if link has been made to a IP Interface
		// If yes, update that object
		$iIpInterfaceId = $this->Get('ipinterface_id');
		if ($iIpInterfaceId != null)
		{
			$this->UpdateIPInterface($iIpInterfaceId);
		}
	}
	
	/*
	 * Perform actions after new object removed from DB 
	 */     
	protected function AfterDelete()
	{
		$sOrgId = $this->Get('org_id');
		$sIp = $this->Get('ip');
		
		// Look for subnet where IP belongs to (should be only one)
		//  Generate event if Low Water Mark is crossed 
		$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE INET_ATON(s.ip) <= INET_ATON('$sIp') AND INET_ATON('$sIp') <= INET_ATON(s.broadcastip) AND s.org_id = $sOrgId"));
		$sSubnetLowWaterMark = GetFromGlobalIPConfig('subnet_low_watermark', $sOrgId);
		$sSubnetHighWaterMark = GetFromGlobalIPConfig('subnet_high_watermark', $sOrgId);
		while ($oSubnet = $oSubnetSet->Fetch())
		{
			$oSubnetOccupancy = $oSubnet->GetOccupancy('IPv4Address');
			$oSubnetAlarm = $oSubnet->Get('alarm_water_mark');
			if ($oSubnetOccupancy < $sSubnetLowWaterMark)
			{
				// Reset water mark alarm if needed
				if ($oSubnetAlarm != 'no_alarm')
				{
					$oSubnet->Set('alarm_water_mark', 'no_alarm');
					$oSubnet->DBUpdate();
				}
			}
			else if ($oSubnetOccupancy < $sSubnetHighWaterMark)
			{
				// Reset water mark alarm if needed
				if ($oSubnetAlarm != 'low_sent')
				{
					$oSubnet->Set('alarm_water_mark', 'low_sent');
					$oSubnet->DBUpdate();
				}
			}
		}
		
		// Look for ranges where IP belongs to (should be only one)
		//  Generate event if Low Water Mark is crossed 
		$oIpRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Range AS r WHERE INET_ATON(r.firstip) <= INET_ATON('$sIp') AND INET_ATON('$sIp') <= INET_ATON(r.lastip) AND r.org_id = $sOrgId"));
		$sIpRangeLowWaterMark = GetFromGlobalIPConfig('iprange_low_watermark', $sOrgId);
		$sIpRangeHighWaterMark = GetFromGlobalIPConfig('iprange_high_watermark', $sOrgId);
		while ($oIpRange = $oIpRangeSet->Fetch())
		{
			$oIpRangeOccupancy = $oIpRange->GetOccupancy('IPv4Address');
			$oIpRangeAlarm = $oIpRange->Get('alarm_water_mark');
			if ($oIpRangeOccupancy < $sIpRangeLowWaterMark)
			{
				// Reset water mark alarm if needed
				if ($oIpRangeAlarm != 'no_alarm')
				{
					$oIpRange->Set('alarm_water_mark', 'no_alarm');
					$oIpRange->DBUpdate();
				}
			}
			else if ($oIpRangeOccupancy < $sIpRangeHighWaterMark)
			{
				// Reset water mark alarm if needed
				if ($oIpRangeAlarm != 'low_sent')
				{
					$oIpRange->Set('alarm_water_mark', 'low_sent');
					$oIpRange->DBUpdate();
				}
			}
		}
	}
	
	/**
	 * Change flag of attributes that shouldn't be modified beside creation.
	 */
	public function GetAttributeFlags($sAttCode, &$aReasons = array(), $sTargetState = '')
	{
		if ((!$this->IsNew()) && (($sAttCode == 'org_id') || ($sAttCode == 'ip')))
		{
			return OPT_ATT_READONLY;
		}
		return parent::GetAttributeFlags($sAttCode, $aReasons, $sTargetState);
	}
	
	/**
	 * Provides attributes' parameters
	 */     
	public function GetAttributeParams($sAttCode)
	{
		if ($sAttCode == 'mac_address')
		{
			$sOrgId = $this->Get('org_id');
			if ($sOrgId != null)
			{
				return(GetFromGlobalIPConfig('mac_address_format', $sOrgId));
			}
			else
			{
				return('colons');
			}
		}
	}

}
?>