<?php
// Copyright (C) 2010-2012 Combodo SARL
//
//   This file is part of TeemIp.
//
//   TeemIp is free software; you can redistribute it and/or modify	
//   it under the terms of the GNU Affero General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   TeemIp is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Affero General Public License for more details.
//
//   You should have received a copy of the GNU Affero General Public License
//   along with TeemIp. If not, see <http://www.gnu.org/licenses/>

/**
 * @copyright   Copyright (C) 2010-2012 Combodo SARL
 * @license     http://opensource.org/licenses/AGPL-3.0
 */

class _IPv4Block extends IPv4Object
{
	/**
	 * Handles internal variables
	 */
	public function SetWriteReason($sReason)
	{
		$this->m_WriteReason = $sReason; 
	}
	
	public function GetBlockSize()
	{
		// Just return size of block
		return(myip2long($this->Get('lastip')) - myip2long($this->Get('firstip')) + 1);
	}
	
	function GetMinBlockSize()
	{
		// Return minimum block size required.
		// Default value may be overwritten but not under absolute minimum value.
		
		$iBlockMinSize = utils::ReadPostedParam('attr_block_min_size', BLOCK_MIN_SIZE);
		if ($iBlockMinSize < BLOCK_MIN_SIZE)
		{
			$iBlockMinSize = BLOCK_MIN_SIZE;
		}
		return $iBlockMinSize;
	}
	
	public function GetIcon($bImgTag = true, $bXsIcon = false)
	{
		// Return standard icon or extra small one
		if ($bXsIcon)
		{
			$sIcon = utils::GetAbsoluteUrlModulesRoot().'teemip-ip-mgmt/images/ipblock-xs.png';
		}
		else
		{
			$sIcon = utils::GetAbsoluteUrlModulesRoot().'teemip-ip-mgmt/images/ipblock.png';
		}
		return ("<img src=\"$sIcon\" style=\"vertical-align:middle;\"/>");
	}
	
	public function GetNewFormId($sPrefix)
	{
		self::$iGlobalFormId++;
		$this->m_iFormId = $sPrefix.self::$iGlobalFormId;
		return ($this->m_iFormId);
	}
	
	/**
	 * Return % of occupancy of objects linked to $this
	 */
	public function GetOccupancy($sObject)
	{
		$sOrgId = $this->Get('org_id');
		$iKey = $this->GetKey();
		$iBlockSize = $this->GetBlockSize();
		
		switch ($sObject)
		{
			case 'IPv4Block':
				// Look for all child blocks
				$iChildBlockSize = 0;
				$oSRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Block AS b WHERE b.parent_id = '$iKey' AND b.org_id = $sOrgId"));
				while ($oSRange = $oSRangeSet->Fetch())
				{
					$iChildBlockSize += $oSRange->GetBlockSize();
				}
				return ($iChildBlockSize / $iBlockSize)*100;
			
			case 'IPv4Subnet':
				// Look for all child subnets
				$iSubnetSize = 0;
				$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE s.block_id = '$iKey' AND s.org_id = $sOrgId"));
				while ($oSubnet = $oSubnetSet->Fetch())
				{
					$iSubnetSize += MaskToSize($oSubnet->Get('mask'));
				}
				return ($iSubnetSize / $iBlockSize)*100;
				
			default:
				return 0;
		}
	}
	
	/**
	* Find space within the block to create child block or subnet
	 */
	public function GetFreeSpace($iSize, $iMaxOffer)
	{
		$bitMask = SizeToMask($iSize);
		$sOrgId = $this->Get('org_id');
		$iKey = $this->GetKey();
		$aFreeSpace = array();
		
		// Get list of registered blocks and subnets in subnet range
		$sFirstIp = $this->Get('firstip');
		$iObjFirstIp = myip2long($sFirstIp);
		$sLastIp = $this->Get('lastip');
		$iObjLastIp = myip2long($sLastIp);
		$iBlockSize = $this->GetBlockSize();
		$oChildBlockSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Block AS b WHERE b.parent_id = $iKey AND b.org_id = $sOrgId"));
		$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE s.block_id = $iKey AND s.org_id = $sOrgId"));
		
		$aList = array();
		$i = 0;
		while ($oChildBlock = $oChildBlockSet->Fetch())
		{
			$aList[$i] = array();
			$aList[$i]['firstip'] = myip2long($oChildBlock->Get('firstip'));
			$aList[$i]['lastip'] = myip2long($oChildBlock->Get('lastip'));
			$i++;
		}
		while ($oSubnet = $oSubnetSet->Fetch())
		{
			$aList[$i] = array();
			$aList[$i]['firstip'] = myip2long($oSubnet->Get('ip'));
			$aList[$i]['lastip'] = myip2long($oSubnet->Get('broadcastip'));
			$i++;
		}
		// Sort $aList by 'firstip' and create array of free ranges
		$aFreeList = array();
		if (!empty($aList))
		{
			foreach ($aList as $key => $row)
			{
				$aFirstIp[$key] = $row['firstip'];
			}
			array_multisort($aFirstIp, SORT_ASC, $aList);
			
			$iSizeArray = $i;
			$iAnIp = $iObjFirstIp;
			$i = 0;
			$j = 0;
			while ($i < $iSizeArray)
			{
				while (($i < $iSizeArray) && ($iAnIp == $aList[$i]['firstip']))
				{
					$iAnIp = $aList[$i]['lastip'] + 1;
					$i++; 
				}
				if ($iAnIp < $iObjLastIp)
				{
					$aFreeList[$j] = array();
					$aFreeList[$j]['firstip'] = $iAnIp;
					if ($i < $iSizeArray)
					{
						$aFreeList[$j]['lastip'] = $aList[$i]['firstip'] - 1;
						$iAnIp = $aList[$i]['firstip'];
					}
					else
					{
						$aFreeList[$j]['lastip'] = $iObjLastIp;
					}
					$j++;
				}
			}
			$iSizeFreeArray = $j;
		}
		else
		{
			$iSizeFreeArray = 1;
			$aFreeList[0] = array();
			$aFreeList[0]['firstip'] = $iObjFirstIp;
			$aFreeList[0]['lastip'] = $iObjLastIp;
		}
		
		$oAppContext = new ApplicationContext();
		$sParams = $oAppContext->GetForLink();
	
		// Store possible choices in array
		if ($iSizeFreeArray != 0)
		{
			$j = 0;
			$n = 0;
			do
			{
				$iAnIp = $aFreeList[$j]['firstip'];
				// Align $iAnIp to mask 
				if ((ip2long(long2ip($iAnIp)) & ip2long(long2ip($bitMask))) != ip2long(long2ip($iAnIp)))
				{
					$iAnIp = (ip2long(long2ip($iAnIp)) & ip2long(long2ip($bitMask))) + $iSize; 
				}
				$iLastFreeIp = $aFreeList[$j]['lastip'];
				$iLastIp = $iAnIp + $iSize - 1;
				while (($iLastIp <= $iLastFreeIp) && ($n < $iMaxOffer))
				{
					$aFreeSpace[$n] = array();
					$aFreeSpace[$n]['firstip'] = mylong2ip($iAnIp);
					$aFreeSpace[$n]['lastip'] = mylong2ip($iLastIp); 
					$aFreeSpace[$n]['mask'] = mylong2ip($bitMask);
					$n++;
					$iAnIp = $iLastIp + 1;
					$iLastIp = $iAnIp + $iSize - 1;
				}
			}
			while ((++$j < $iSizeFreeArray) && ($n < $iMaxOffer));
		}
		
		// Return result
		return $aFreeSpace;
	}
	
	/**
	* Displays the tabs listing the child blocks and the subnets belonging to a block
	 */
	public function DisplayBareRelations(WebPage $oPage, $bEditMode = false)
	{
		// Execute parent function first 
		parent::DisplayBareRelations($oPage, $bEditMode);
		
		$sOrgId = $this->Get('org_id');
		if ($this->IsNew())
		{
			// Tab for Global Parameters at creation time only
			if ($sOrgId != null)
			{
				$oPage->SetCurrentTab(Dict::Format('Class:IPv4Block/Tab:globalparam'));
				$oPage->p(Dict::Format('UI:IPManagement:Action:Change:GlobalConfig'));
				$oPage->add('<table style="vertical-align:top"><tr>');
				$oPage->add('<td style="vertical-align:top">');
				
				$sParameter = array ('block_min_size', 'block_cidr_aligned', null);
				DisplayGlobalParametersInLocalModifyForm($oPage, $sParameter, $sOrgId);			
				
				$oPage->add('</td>');
				$oPage->add('</tr></table>');
			}
		}
		else
		{
			$sBlockId = $this->GetKey();
			$iSizeBlock = $this->GetBlockSize();
			
			$aExtraParams = array();
			$aExtraParams['menu'] = false;
			
			// Tab for child blocks
			$oChildBlockSearch = DBObjectSearch::FromOQL("SELECT IPv4Block AS block WHERE block.parent_id = $sBlockId AND block.org_id = $sOrgId");
			$oChildBlockSet = new CMDBObjectSet($oChildBlockSearch);
			$oPage->SetCurrentTab(Dict::Format('Class:IPv4Block/Tab:childblock', $oChildBlockSet->Count()));
			$oPage->p(MetaModel::GetClassIcon('IPv4Block').'&nbsp;'.Dict::Format('Class:IPv4Block/Tab:childblock+'));
			$oPage->p($this->GetAsHTML('children_occupancy').Dict::Format('Class:IPv4Block/Tab:childblock-count-percent'));
			$oBlock = new DisplayBlock($oChildBlockSearch, 'list');
			$oBlock->Display($oPage, 'child_blocks', $aExtraParams);
			
			// Tab for subnets
			$oSubnetSearch = DBObjectSearch::FromOQL("SELECT IPv4Subnet AS subnet WHERE subnet.block_id = $sBlockId AND subnet.org_id = $sOrgId");
			$oSubnetSet = new CMDBObjectSet($oSubnetSearch);
			$oPage->SetCurrentTab(Dict::Format('Class:IPv4Block/Tab:subnet', $oSubnetSet->Count()));
			$oPage->p(MetaModel::GetClassIcon('IPv4Subnet').'&nbsp;'.Dict::Format('Class:IPv4Block/Tab:subnet+'));
			$oPage->p($this->GetAsHTML('subnet_occupancy').Dict::Format('Class:IPv4Block/Tab:subnet-count-percent'));
			$oBlock = new DisplayBlock($oSubnetSearch, 'list');
			$oBlock->Display($oPage, 'child_subnets', $aExtraParams);
		}
	}
	
	/**
	* Check validity of new block attributes before creation
	 */
	public function DoCheckToWrite()
	{
		// Run standard iTop checks first
		parent::DoCheckToWrite();
		
		$sOrgId = $this->Get('org_id');
		$iKey = $this->GetKey();
		$sName = $this->Get('name');
		$iFirstIp = myip2long($this->Get('firstip'));
		$iLastIp = myip2long($this->Get('lastip'));	 
		$Size = $iLastIp - $iFirstIp + 1;
		
		// Check name doesn't already exist
		$oSRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Block AS b WHERE b.name = '$sName' AND b.org_id = $sOrgId"));
		while ($oSRange = $oSRangeSet->Fetch())
		{
			// Skip check with self (DB copy) if necessary
			if ($oSRange->GetKey() != $iKey)
			{
				$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Block:NameExist');
				return;
			}
		}
		
		// All modifications related to first and last IPs are done through special actions (shrink, split, expand)
		// As a consequence:
		//	Code of shrink, split, expand functions must check coherency of these IPs
		//	DoCheckToWrite only checks their coherency at creation.
		
		// If check is performed because of split, skip checks
		//		if ($this->m_WriteReason == ACTION_SPLIT)
		if ($this->Get('write_reason') == 'split')
		{
			return;
		}
		
		// In case of modification, no specific check is done as changes do concern minor points and not first or last IP of block.
		if ($this->IsNew())
		{
			// Check that 1st IP is smaller than last one
			if ($iFirstIp >= $iLastIp)
			{
				$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Block:Reverted');
				return;
			}
			
			// Make sure size of block is bigger than absolute minimum size allowed (constant)
			// Default value may be overwritten but not under absolute minimum value.
			$iBlockMinSize = utils::ReadPostedParam('attr_block_min_size', BLOCK_MIN_SIZE);
			if ($iBlockMinSize < BLOCK_MIN_SIZE)
			{
				$iBlockMinSize = BLOCK_MIN_SIZE;
			}
			if ($Size < $iBlockMinSize)
			{
				$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Block:SmallerThanMinSize', $iBlockMinSize);
				return;
			}
			
			// If required by global parameters, check if block needs to be CIDR aligned and check last IP if needed.
			if (!IsBlockCIDRAligned($sOrgId, $iFirstIp, $iLastIp))
			{
				$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Block:NotCIDRAligned');
				return;
			} 
			
			// Make sure range is fully and strictly contained in parent range requested, if any
			//		If no parent is specified, then parent is entire IPv4 space by default and tested condition is true.
			$iParentId = $this->Get('parent_id');
			if ($iParentId != 0)
			{
				$oParent = MetaModel::GetObject('IPv4Block', $iParentId, false /* MustBeFound */);
				if (!is_null($oParent))
				{
					$iParentFirstIp = myip2long($oParent->Get('firstip'));
					$iParentLastIp = myip2long($oParent->Get('lastip'));
					if (($iFirstIp < $iParentFirstIp) || ($iParentLastIp < $iLastIp) || (($iFirstIp == $iParentFirstIp) && ($iParentLastIp == $iLastIp)))
					{
						$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Block:NotInParent');
						return;
					}
				}
			}
			
			// Make sure range doesn't collide with another range attached to the same parent.
			//		If no parent is specified (null), then check is done with all such blocks with null parent specified.
			//		It is done on blocks belonging to the same parent otherwise
			$oSRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Block AS b WHERE b.parent_id = '$iParentId' AND b.org_id = $sOrgId"));
			while ($oSRange = $oSRangeSet->Fetch())
			{
				// Skip check with self (DB copy) if necessary
				if ($oSRange->GetKey() != $iKey)
				{
					$iCurrentFirstIp = myip2long($oSRange->Get('firstip'));
					$iCurrentLastIp = myip2long($oSRange->Get('lastip'));
					
					// Does the range already exist?
					if (($iCurrentFirstIp == $iFirstIp) && ($iCurrentLastIp == $iLastIp))
					{
						$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Block:Collision0');
						return;
					}
					// Is new first Ip part of an existing range?
					if (($iCurrentFirstIp < $iFirstIp) && ($iFirstIp <= $iCurrentLastIp))
					{
						$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Block:Collision1');
						return;
					}
					// Is new last Ip part of an existing range?
					if (($iCurrentFirstIp <= $iLastIp) && ($iLastIp < $iCurrentLastIp))
					{
						$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Block:Collision2');
						return;
					}
					// If new subnet range is including existing ones, the included ranges will automatically be attached
					//	 to the one newly created because of hierarchical structure of blocks (see AfterInsert).
				}
			}
		}
	}
	
	/**
	 * Perform specific tasks related to block creation
	 **/
	public function AfterInsert()
	{
		parent::AfterInsert();
		
		$sOrgId = $this->Get('org_id');
		$iKey = $this->GetKey();
		$iParentId = $this->Get('parent_id');
		$sFirstIp = $this->Get('firstip');
		$sLastIp = $this->Get('lastip');
		
		// Look for all blocks attached to parent of block being created and contained within new block
		// Attach them to new block
		// Note that $this will be part of returned $oSRangeSet
		$HasChildren = false;
		$oSRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Block AS b WHERE b.parent_id = '$iParentId' AND INET_ATON('$sFirstIp') <= INET_ATON(b.firstip) AND INET_ATON(b.lastip) <= INET_ATON('$sLastIp') AND b.org_id = $sOrgId"));
		if ($oSRangeSet->Count() > 1)
		{
			while ($oSRange = $oSRangeSet->Fetch())
			{
				// Skip self
				if ($oSRange->GetKey() != $iKey)
				{
					$oSRange->Set('parent_id', $iKey);
					$oSRange->DBUpdateTracked();	 
					$HasChildren = true;						
				}
			}
		}
		
		// If block is not at the top (all subnets are attached to a block), 
		//	Look for all subnets attached to parent block contained within new block
		//	Attach them to new block
		$HasParent = false;
		if ($iParentId != 0)
		{
			$HasParent = true;
			$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE s.block_id = '$iParentId' AND INET_ATON('$sFirstIp') <= INET_ATON(s.ip) AND INET_ATON(s.broadcastip)<= INET_ATON('$sLastIp') AND s.org_id = $sOrgId"));
			if ($oSubnetSet->Count() != 0)
			{
				$iSubnetSize = 0;
				while ($oSubnet = $oSubnetSet->Fetch())
				{
					$oSubnet->Set('block_id', $iKey);
					$oSubnet->DBUpdate();
				}
			}
		}
	}
				
	/**
	 * Perform specific tasks related to block modification
	 */
	public function AfterUpdate()
	{
//		if ($this->m_WriteReason != ACTION_SPLIT)
		if ($this->Get('write_reason') != 'split')
		{
			$sOrgId = $this->Get('org_id');
			$iKey = $this->GetKey();
			$iParentId = $this->Get('parent_id');
			$iFirstIp = ip2long($this->Get('firstip'));
			$iLastIp = myip2long($this->Get('lastip'));
						
			// Look for all subnets attached to block that may have fallen out of block
			//	Attach them to parent block 
			//	Note: previous check have made sure a parent block exists
			$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE s.block_id = '$iKey' AND s.org_id = $sOrgId"));
			$iSubnetSize = 0;
			while ($oSubnet = $oSubnetSet->Fetch())
			{
				$iCurrentFirstIp = ip2long($oSubnet->Get('ip'));
				$iCurrentLastIp = myip2long($oSubnet->Get('broadcastip'));
				if (($iCurrentLastIp < $iFirstIp) || ($iLastIp < $iCurrentFirstIp))
				{
					if ($iParentId == 0)
					{
						throw new ApplicationException(Dict::Format('UI:IPManagement:Action:Modify:IPv4Block:ParentIdNull', $iKey));
					}
					$oSubnet->Set('block_id', $iParentId);
					$oSubnet->DBUpdate();
				}
			}
		}
//		$this->m_WriteReason = ACTION_NONE;
		$this->Set('write_reason', 'none');
		
		parent::AfterUpdate();		
	}
		
	/**
	 * Change flag of attribute that shouldn't be modified beside creation.
	 */
	public function GetAttributeFlags($sAttCode, &$aReasons = array(), $sTargetState = '')
	{
		if ((!$this->IsNew()) && (($sAttCode == 'org_id') || ($sAttCode == 'parent_id') || ($sAttCode == 'firstip') || ($sAttCode == 'lastip') || ($sAttCode == 'occupancy') || ($sAttCode == 'children_occupancy')	|| ($sAttCode == 'subnet_occupancy')))
		{
			return OPT_ATT_READONLY;
		}
		return parent::GetAttributeFlags($sAttCode, $aReasons, $sTargetState);
	}
	
	/**
	 * Provides attributes' parameters
	 */		 
	public function GetAttributeParams($sAttCode)
	{
		$aParams = array();
		if (($sAttCode == 'occupancy') || ($sAttCode == 'children_occupancy') || ($sAttCode == 'subnet_occupancy')) 
		{
			if ($sAttCode == 'children_occupancy')
			{
				$Occupancy = $this->GetOccupancy('IPv4Block');
			}
			else if ($sAttCode == 'subnet_occupancy')
			{
				$Occupancy = $this->GetOccupancy('IPv4Subnet');
			}
			else
			{
				$Occupancy = $this->GetOccupancy('IPv4Block') + $this->GetOccupancy('IPv4Subnet');
			}
			// Note: water marks for blocks are not global parameters that can be modified
			$sLowWaterMark = DEFAULT_BLOCK_LOW_WATER_MARK;
			$sHighWaterMark = DEFAULT_BLOCK_HIGH_WATER_MARK;
			if ($Occupancy >= $sHighWaterMark)
			{
				$sColor = RED;
			}
			else if ($Occupancy >= $sLowWaterMark)
			{
				$sColor = YELLOW;
			}
			else
			{
				$sColor = GREEN;
			}
			$aParams ['value'] = round ($Occupancy, 0);
			$aParams ['color'] = $sColor;
		}
		else
		{
			$aParams ['value'] = 0;
			$aParams ['color'] = GREEN;
		}
		return ($aParams);
	}

}

?>