<?php
// Copyright (C) 2010-2012 Combodo SARL
//
//   This file is part of TeemIp.
//
//   TeemIp is free software; you can redistribute it and/or modify	
//   it under the terms of the GNU Affero General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   TeemIp is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Affero General Public License for more details.
//
//   You should have received a copy of the GNU Affero General Public License
//   along with TeemIp. If not, see <http://www.gnu.org/licenses/>

/**
 * @copyright   Copyright (C) 2010-2012 Combodo SARL
 * @license     http://opensource.org/licenses/AGPL-3.0
 */

class _IPv4Range extends IPv4Object
{
 	/**
	 * Handles internal variables
	 */
	public function SetWriteReason($sReason)
	{
		$this->m_WriteReason = $sReason; 
	}
	
	/**
	 * Provide right icon
	 */		 
	public function GetIcon($bImgTag = true, $bXsIcon = false)
	{
		// Return standard icon or extra small one
		if ($bXsIcon)
		{
			$sIcon = utils::GetAbsoluteUrlModulesRoot().'teemip-ip-mgmt/images/iprange-xs.png';
		}
		else
		{
			$sIcon = utils::GetAbsoluteUrlModulesRoot().'teemip-ip-mgmt/images/iprange.png';
		}
		return ("<img src=\"$sIcon\" style=\"vertical-align:middle;\"/>");
	}
	
	public function GetOccupancy($sObject)
	{
		// Compute number of IPs in IP range (in %)
		$sOrgId = $this->Get('org_id');
		$sFirstIp = $this->Get('firstip');
		$sLastIp = $this->Get('lastip');
		$iSize = myip2long ($sLastIp) - myip2long($sFirstIp) + 1;
		$oIpRegisteredSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE INET_ATON('$sFirstIp') <= INET_ATON(i.ip) AND INET_ATON(i.ip) <= INET_ATON('$sLastIp') AND i.org_id = $sOrgId"));
		return ($oIpRegisteredSet->Count()/$iSize)*100;
	}
	
	/**
	 * Displays the tabs listing the registered IPs belonging to an IP range
	 */
	function DisplayBareRelations(WebPage $oPage, $bEditMode = false)
	{
		// Execute parent function first 
		parent::DisplayBareRelations($oPage, $bEditMode);
		
		if (!$this->IsNew())
		{
			$sOrgId = $this->Get('org_id');
			$sFirstIp = $this->Get('firstip');
			$sLastIp = $this->Get('lastip');
			$iFirstIp = myip2long($sFirstIp);
			$iLastIp = myip2long ($sLastIp);
			
			$iSize = $iLastIp - $iFirstIp + 1;
			
			$aExtraParams = array();
			$aExtraParams['menu'] = false;			
			
			// Tab for Registered IPs
			$oIpAssignedSearch = DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE INET_ATON(i.ip) >= INET_ATON('$sFirstIp') AND INET_ATON(i.ip) <= INET_ATON('$sLastIp')	AND i.org_id = $sOrgId");
			$oIpAssignedSet = new CMDBObjectSet($oIpAssignedSearch);
			$iCountAssigned = $oIpAssignedSet->Count();
			$iCountAllocated = 0;
			while ($oIpAssigned = $oIpAssignedSet->Fetch())
			{
				if ($oIpAssigned->Get('status') == 'allocated')
				{
					$iCountAllocated++;
				}
			}
			$iCountReserved = $iCountAssigned - $iCountAllocated;
			$oPage->SetCurrentTab(Dict::Format('Class:IPv4Range/Tab:ipregistered', $iCountAssigned));
			$oPage->p(MetaModel::GetClassIcon('IPv4Address').'&nbsp;'.Dict::Format('Class:IPv4Range/Tab:ipregistered+'));
			$oPage->p($this->GetAsHTML('occupancy').Dict::Format('Class:IPv4Range/Tab:ipregistered-count', $iCountReserved, $iCountAllocated, $iSize));
			$oBlock = new DisplayBlock($oIpAssignedSearch, 'list');
			$oBlock->Display($oPage, 'ip_addresses', $aExtraParams);
			
			// Tab for related notifications, if any, in non edit mode
			if (!$bEditMode)
			{
				// Look for triggers referring to IPv4Range class
				// If any trigger has been found, look for notifications related to object and display them if any.
				if	(MetaModel::IsValidClass('IPTriggerOnWaterMark'))
				{
					$oTriggerSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPTriggerOnWaterMark AS T WHERE T.target_class = 'IPv4Range'"));
					if ($oTriggerSet->Count() > 0)
					{
						$iId = $this->GetKey();
						$oNotifSearch = DBObjectSearch::FromOQL("SELECT EventNotificationEmail AS Ev JOIN IPTriggerOnWaterMark AS T ON Ev.trigger_id = T.id WHERE T.target_class = 'IPv4Range' AND Ev.object_id = $iId");
						$oNotifSet = new DBObjectSet($oNotifSearch);
						$sCount = $oNotifSet->Count();
						if ($sCount > 0)
						{
							$oPage->SetCurrentTab(Dict::Format('Class:IPv4Range/Tab:notifications', $sCount));
							$oPage->p(MetaModel::GetClassIcon('IPTriggerOnWaterMark').'&nbsp;'.Dict::Format('Class:IPv4Range/Tab:notifications+'));
							$oBlock = new DisplayBlock($oNotifSearch, 'list', false);
							$oBlock->Display($oPage, 'notifications', array('menu' => false));
						}
					}
				}
			}
		}
	}
	
	/**
	 * Check validity of new IP attributes before creation
	.*/
	function DoCheckToWrite()
	{
		// Run standard iTop checks first
		parent::DoCheckToWrite();
		
		$sOrgId = $this->Get('org_id');
		$iKey = $this->GetKey();
		$sRange = $this->Get('range');
		$iFirstIp = myip2long($this->Get('firstip'));
		$iLastIp = myip2long($this->Get('lastip'));
		$iSubnetId = $this->Get('subnet_id');	 
		
		// If check is done during subnet expand, skip checks
//		if ($this->m_WriteReason == ACTION_EXPAND)
		if ($this->Get('write_reason') == 'expand')
		{
			// Reset reason for action
//			$this->m_WriteReason = ACTION_NONE;
			$this->Set('write_reason', 'none');
		}
		else
		{
			// Check that 1st Ip is smaller than last one
			if ($iFirstIp >= $iLastIp)
			{
				$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Range:Reverted');
				return;
			}
			
			// Make sure range is fully contained in subnet
			$oSubnet = MetaModel::GetObject('IPv4Subnet', $this->Get('subnet_id'), true /* MustBeFound */);
			$iSubnetBroadcastIp = myip2long($oSubnet->Get('broadcastip'));
			if (($iFirstIp < myip2long($oSubnet->Get('ip'))) || ($iSubnetBroadcastIp < $iLastIp))
			{
				$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Range:NotInSubnet');
				return;
			}
			
			// Make sure range doesn't collide with another range within subnet
			$oRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Range AS r WHERE r.subnet_id = '$iSubnetId' AND r.org_id = $sOrgId"));
			while ($oRange = $oRangeSet->Fetch())
			{
				// If it's a modification (keys are the same) further checks are not relevant
				if ($oRange->GetKey() != $iKey)
				{
					$iCurrentFirstIp = myip2long($oRange->Get('firstip'));
					$iCurrentLastIp = myip2long($oRange->Get('lastip'));
					
					// Check that name doesn't already exist in same subnet
					if ($oRange->Get('range') == $sRange)
					{
						$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Range:NameExist');
						return;
					}
					// Does the range already exist?
					if (($iCurrentFirstIp == $iFirstIp) && ($iCurrentLastIp == $iLastIp))
					{
						$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Range:Collision0');
						return;
					}
					// Is new first Ip part of an existing range?
					if (($iCurrentFirstIp <= $iFirstIp) && ($iFirstIp <= $iCurrentLastIp))
					{
						$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Range:Collision1');
						return;
					}
					// Is new last Ip part of an existing range?
					if (($iCurrentFirstIp <= $iLastIp) && ($iLastIp <= $iCurrentLastIp))
					{
						$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Range:Collision2');
						return;
					}
					// Is new range including an existing one?
	 				if (($iFirstIp < $iCurrentFirstIp) && ($iCurrentLastIp < $iLastIp))
					{
						$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Range:Collision3');
						return;
					}
				}
			}
		}
	}
	
	/**
	 * Change flag of attributes that shouldn't be modified beside creation.
	 */
	public function GetAttributeFlags($sAttCode, &$aReasons = array(), $sTargetState = '')
	{
		if ((!$this->IsNew()) && (($sAttCode == 'org_id') || ($sAttCode == 'subnet_id') || ($sAttCode == 'occupancy')))
		{
			return OPT_ATT_READONLY;
		}
		return parent::GetAttributeFlags($sAttCode, $aReasons, $sTargetState);
	}		
		
	/**
	 * Provides attributes' parameters
	 */		 
	public function GetAttributeParams($sAttCode)
	{
		$aParams = array();
		if ($sAttCode == 'occupancy')
		{
			$Occupancy = $this->GetOccupancy('IPv4Address');
			$sOrgId = $this->Get('org_id');
			if ($sOrgId != null)
			{
				$sLowWaterMark = GetFromGlobalIPConfig('iprange_low_watermark', $sOrgId);
				$sHighWaterMark = GetFromGlobalIPConfig('iprange_high_watermark', $sOrgId);
				if ($Occupancy >= $sHighWaterMark)
				{
					$sColor = RED;
				}
				else if ($Occupancy >= $sLowWaterMark)
				{
					$sColor = YELLOW;
				}
				else
				{
					$sColor = GREEN;
				}
				$aParams ['value'] = round ($Occupancy, 0);
				$aParams ['color'] = $sColor;
			}
			else
			{
				$aParams ['value'] = 0;
				$aParams ['color'] = GREEN;
			}
		}
		else
		{
			$aParams ['value'] = 0;
			$aParams ['color'] = GREEN;
		}
		return ($aParams);
	}

}
?>