<?php
// Copyright (C) 2010-2012 Combodo SARL
//
//   This file is part of TeemIp.
//
//   TeemIp is free software; you can redistribute it and/or modify	
//   it under the terms of the GNU Affero General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   TeemIp is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Affero General Public License for more details.
//
//   You should have received a copy of the GNU Affero General Public License
//   along with TeemIp. If not, see <http://www.gnu.org/licenses/>

/**
 * @copyright   Copyright (C) 2010-2012 Combodo SARL
 * @license     http://opensource.org/licenses/AGPL-3.0
 */

class _IPv4Subnet extends IPv4Object
{
	/**
	 * Handles internal variables
	 */
	public function SetWriteReason($sReason)
	{
		$this->m_WriteReason = $sReason; 
	}
	
	public function GetIcon($bImgTag = true, $bXsIcon = false)
	{
		// Return standard icon or extra small one
		if ($bXsIcon)
		{
			$sIcon = utils::GetAbsoluteUrlModulesRoot().'teemip-ip-mgmt/images/ipsubnet-xs.png';
		}
		else
		{
			$sIcon = utils::GetAbsoluteUrlModulesRoot().'teemip-ip-mgmt/images/ipsubnet.png';
		}
		return ("<img src=\"$sIcon\" style=\"vertical-align:middle;\"/>");
	}
	
	public function GetNewFormId($sPrefix)
	{
		self::$iGlobalFormId++;
		$this->m_iFormId = $sPrefix.self::$iGlobalFormId;
		return ($this->m_iFormId);
	}
	
	/**
	 * Compute number of IPs in IP subnet (in %)
	 */	 
	public function GetOccupancy($sObject)
	{
		$sOrgId = $this->Get('org_id');
		
		switch ($sObject)
		{
			case 'IPv4Address': 
				// Look for all IPs within subnets
				//	Note that these IPs can belong to an IP range
				$sIp = $this->Get('ip');
				$sIpBroadcast = $this->Get('broadcastip');
				$oIpRegisteredSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE INET_ATON('$sIp') <= INET_ATON(i.ip) AND INET_ATON(i.ip) <= INET_ATON('$sIpBroadcast') AND i.org_id = $sOrgId"));
				return ($oIpRegisteredSet->Count() / MaskToSize($this->Get('mask')))*100;
			
			case 'IPv4Range':
				// Look for all child IP ranges
				$sSubnet = $this->GetKey();
				$oIpRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Range AS r WHERE r.subnet_id = '$sSubnet' AND r.org_id = $sOrgId"));
				$iSizeRanges = 0;
				while ($oIpRange = $oIpRangeSet->Fetch())
				{
					$iSizeRanges += myip2long($oIpRange->Get('lastip')) - myip2long($oIpRange->Get('firstip')) + 1; 
				}
				return ($iSizeRanges / MaskToSize($this->Get('mask')))*100;
				
			case 'IPv4Address_out_IPv4Range':
				// Look for all IPs within subnets
				$sIp = $this->Get('ip');
				$sIpBroadcast = $this->Get('broadcastip');
				$oIpRegisteredSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE INET_ATON('$sIp') <= INET_ATON(i.ip) AND INET_ATON(i.ip) <= INET_ATON('$sIpBroadcast') AND i.org_id = $sOrgId"));
				// Look for all child IP ranges
				$sSubnet = $this->GetKey();
				$oIpRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Range AS r WHERE r.subnet_id = '$sSubnet' AND r.org_id = $sOrgId"));
				$iIpInRanges = 0;
				$iSizeRanges = 0;
				while ($oIpRange = $oIpRangeSet->Fetch())
				{
					$sIpRangeFirstIp = $oIpRange->Get('firstip');
					$sIpRangeLastIp = $oIpRange->Get('lastip');
					$oIpRegisteredInRange = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE INET_ATON('$sIpRangeFirstIp') <= INET_ATON(i.ip) AND INET_ATON(i.ip) <= INET_ATON('$sIpRangeLastIp') AND i.org_id = $sOrgId"));
					$iIpInRanges += $oIpRegisteredInRange->Count();
					$iSizeRanges += myip2long($oIpRange->Get('lastip')) - myip2long($oIpRange->Get('firstip')) + 1;
				}
				return (($oIpRegisteredSet->Count() - $iIpInRanges) / MaskToSize($this->Get('mask')))*100;
					
			default:
				return 0;
		}
	}
	
	/**
	 * Find space within the subnet to create range
	 */
	public function GetFreeSpace($iRangeSize, $iMaxOffer)
	{
		$sOrgId = $this->Get('org_id');
		$iKey = $this->GetKey();
		$aFreeSpace = array();
		
		// Get list of registered IPs & ranges in subnet
		$sFirstIp = $this->Get('ip');
		$bitFirstIp = myip2long($sFirstIp);
		$sLastIp = $this->Get('broadcastip');
		$bitLastIp = myip2long($sLastIp);
		$iSubnetSize = MaskToSize($this->Get('mask'));
		if ($iRangeSize >= $iSubnetSize)
		{
			// Required range size is to big, exit
			return $aFreeSpace;
		}
		else
		{
			$oIpRegisteredSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE INET_ATON('$sFirstIp') <= INET_ATON(i.ip) AND INET_ATON(i.ip) <= INET_ATON('$sLastIp') AND i.org_id = $sOrgId"));
			$aRegisteredIPs = $oIpRegisteredSet->GetColumnAsArray('ip', false);
			$oIpRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Range AS r WHERE r.subnet_id = $iKey AND r.org_id = $sOrgId"));
			$aRangeIPs = $oIpRangeSet->GetColumnAsArray('firstip', false);
			
			$iAnIp = $bitFirstIp + 1;
			$sAnIp = mylong2ip($iAnIp);
			$n = 0;
			do
			{
				// Find next free IP
				while (in_array($sAnIp, $aRegisteredIPs))
				{
					$iAnIp++;
					$sAnIp = mylong2ip($iAnIp);
				}
				if ($iAnIp < $bitLastIp)
				{
					// If free IP belongs to an IP range, skip range
					$oIpRangeSet->Rewind();
					$bContinue = true;
					while ($bContinue && ($oIpRange = $oIpRangeSet->Fetch()))
					{
						if ((myip2long($oIpRange->Get('firstip')) <= $iAnIp) && ($iAnIp <= myip2long($oIpRange->Get('lastip'))))
						{
							$iAnIp = myip2long($oIpRange->Get('lastip')) + 1;
							$sAnIp = mylong2ip($iAnIp);
							$bContinue = false;
						}
					}
					if ($iAnIp < $bitLastIp)
					{
						// Make sure we don't have any IP or range until last IP
						$iRangeFirstIp = $iAnIp;
						$i = 0; 
						$bContinue = true;
						while ($bContinue && (!in_array($sAnIp, $aRegisteredIPs)) && ($iAnIp < $bitLastIp) && ($i < $iRangeSize))
						{
							if (in_array($sAnIp, $aRangeIPs))
							{
								$bContinue = false;
							}
							else
							{
								$iAnIp++;
								$sAnIp = mylong2ip($iAnIp);
								$i++;
							}
						}
						if ($i == $iRangeSize)
						{
							$aFreeSpace[$n] = array();
							$iRangeLastIp = $iAnIp - 1;
							$aFreeSpace[$n]['firstip'] = mylong2ip($iRangeFirstIp);
							$aFreeSpace[$n]['lastip'] = mylong2ip($iRangeLastIp);
							$n++;
						}
					}
				}
			} while (($iAnIp < $bitLastIp) && ($n < $iMaxOffer));
		}
		
		// Return result
		return $aFreeSpace;
	}
	
	/**
	 * Displays the tabs listing the registered IPs, the IP Ranges belonging to a subnet and its related notifications
	 */
	function DisplayBareRelations(WebPage $oPage, $bEditMode = false)
	{
		// Execute parent function first 
		parent::DisplayBareRelations($oPage, $bEditMode);
		
		$sOrgId = $this->Get('org_id');
		if ($this->IsNew())
		{
			// Tab for Global Parameters at creation time only
			if ($sOrgId != null)
			{
				$oPage->SetCurrentTab(Dict::Format('Class:IPv4Subnet/Tab:globalparam'));
				$oPage->p(Dict::Format('UI:IPManagement:Action:Change:GlobalConfig'));
				$oPage->add('<table style="vertical-align:top"><tr>');
				$oPage->add('<td style="vertical-align:top">');
				
				$sParameter = array ('reserve_subnet_IPs', null);
				DisplayGlobalParametersInLocalModifyForm($oPage, $sParameter, $sOrgId);			
				
				$oPage->add('</td>');
				$oPage->add('</tr></table>');
			}
		}
		else
		{
			$iKey = $this->GetKey();
			$sIp = $this->Get('ip');
			$sMask = $this->Get('mask');
			$sIpBroadcast = $this->Get('broadcastip');
			$iIp = myip2long($sIp);
			$iIpBroadcast = myip2long($sIpBroadcast);
			$iSize = MaskToSize($sMask);
			
			$aExtraParams = array();
			$aExtraParams['menu'] = false;
			
			// Tab for Registered IPs
			$oIpRegisteredSearch = DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE INET_ATON('$sIp') <= INET_ATON(i.ip) AND INET_ATON(i.ip) <= INET_ATON('$sIpBroadcast') AND i.org_id = $sOrgId");
			$oIpRegisteredSet = new CMDBObjectSet($oIpRegisteredSearch);
			$iCountRegistered = $oIpRegisteredSet->Count();
			$aStatusRegisteredIPs = $oIpRegisteredSet->GetColumnAsArray('status', false);
			$iCountAllocated = 0;
			$i = 0;
			while ($i < $iCountRegistered)
			{
				if ($aStatusRegisteredIPs[$i++] == 'allocated')
				{
					$iCountAllocated++;
				}
			} 
			$iCountReserved = $iCountRegistered - $iCountAllocated;
			$oPage->SetCurrentTab(Dict::Format('Class:IPv4Subnet/Tab:ipregistered', $iCountRegistered));
			$oPage->p(MetaModel::GetClassIcon('IPv4Address').'&nbsp;'.Dict::Format('Class:IPv4Subnet/Tab:ipregistered+'));
			$oPage->p($this->GetAsHTML('ip_occupancy').Dict::Format('Class:IPv4Subnet/Tab:ipregistered-count', $iCountReserved, $iCountAllocated, $iSize));
			$oBlock = new DisplayBlock($oIpRegisteredSearch, 'list');
			$oBlock->Display($oPage, 'ip_addresses', $aExtraParams);
			
			// Tab for IP Ranges
			$oIpRangeSearch = DBObjectSearch::FromOQL("SELECT IPv4Range AS r WHERE r.subnet_id = '$iKey' AND r.org_id = $sOrgId");
			$oIpRangeSet = new CMDBObjectSet($oIpRangeSearch);
			$iCountRange = 0;
			while ($oIpRange = $oIpRangeSet->Fetch())
			{
				$iCountRange += myip2long($oIpRange->Get('lastip')) - myip2long($oIpRange->Get('firstip')) + 1; 
			}
			$oPage->SetCurrentTab(Dict::Format('Class:IPv4Subnet/Tab:iprange', $oIpRangeSet->Count()));
			$oPage->p(MetaModel::GetClassIcon('IPv4Range').'&nbsp;'.Dict::Format('Class:IPv4Subnet/Tab:iprange+'));
			$oPage->p($this->GetAsHTML('range_occupancy').Dict::Format('Class:IPv4Subnet/Tab:iprange-count-percent'));
			$oBlock = new DisplayBlock($oIpRangeSearch, 'list');
			$oBlock->Display($oPage, 'ip_ranges', $aExtraParams);
			
			if (!$bEditMode)
			{
				// Tab for related notifications, if any, in non edit mode
				// Look for triggers referring to IPv4Subnet class
				// If any trigger has been found, look for notifications related to object and display them if any.
				if	(MetaModel::IsValidClass('IPTriggerOnWaterMark'))
				{
					$oTriggerSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPTriggerOnWaterMark AS t WHERE t.target_class = 'IPv4Subnet'"));
					if ($oTriggerSet->Count() > 0)
					{
						$iId = $this->GetKey();
						$oNotifSearch = DBObjectSearch::FromOQL("SELECT EventNotificationEmail AS Ev JOIN IPTriggerOnWaterMark AS T ON Ev.trigger_id = T.id WHERE T.target_class = 'IPv4Subnet' AND Ev.object_id = $iId");
						$oNotifSet = new DBObjectSet($oNotifSearch);
						$sCount = $oNotifSet->Count();
						if ($sCount > 0)
						{
							$oPage->SetCurrentTab(Dict::Format('Class:IPv4Subnet/Tab:notifications', $sCount));
							$oPage->p(MetaModel::GetClassIcon('IPTriggerOnWaterMark').'&nbsp;'.Dict::Format('Class:IPv4Subnet/Tab:notifications+'));
							$oBlock = new DisplayBlock($oNotifSearch, 'list', false);
							$oBlock->Display($oPage, 'notifications', $aExtraParams);
						}
					}
				}
				
				// Tab for related subnet requests, if any, in non edit mode
				if	(MetaModel::IsValidClass('IPv4SubnetRequest'))
				{
					$oSubnetRequestSearch = DBObjectSearch::FromOQL("SELECT IPv4SubnetRequest AS r WHERE r.subnet_id = $iKey");
					$oSubnetRequestSet = new CMDBObjectSet($oSubnetRequestSearch);
					$sCount = $oSubnetRequestSet->Count();
					if ($sCount > 0)
					{
						$oPage->SetCurrentTab(Dict::Format('Class:IPv4Subnet/Tab:requests', $sCount));
						$oPage->p(MetaModel::GetClassIcon('IPObjRequest').'&nbsp;'.Dict::Format('Class:IPv4Subnet/Tab:requests+'));
						$oBlock = new DisplayBlock($oSubnetRequestSearch, 'list');
						$oBlock->Display($oPage, 'subnet_requests', $aExtraParams);
					}
				}
				
				// Tab for related subnet changes, if any, in non edit mode
				if	(MetaModel::IsValidClass('IPv4SubnetChange'))
				{
					$oSubnetChangeSearch = DBObjectSearch::FromOQL("SELECT IPv4SubnetChange AS c WHERE c.subnet_id = $iKey");
					$oSubnetChangeSet = new CMDBObjectSet($oSubnetChangeSearch);
					$sCount = $oSubnetChangeSet->Count();
					if ($sCount > 0)
					{
						$oPage->SetCurrentTab(Dict::Format('Class:IPv4Subnet/Tab:changes', $sCount));
						$oPage->p(MetaModel::GetClassIcon('IPObjChange').'&nbsp;'.Dict::Format('Class:IPv4Subnet/Tab:changes+'));
						$oBlock = new DisplayBlock($oSubnetChangeSearch, 'list');
						$oBlock->Display($oPage, 'subnet_changes', $aExtraParams);
					}
				}
			}
		}
	}
	
	/**
	 * Check validity of new subnet attributes before creation
	 */
	function DoCheckToWrite()
	{
		// Run standard iTop checks first
		parent::DoCheckToWrite();
		
		$sOrgId = $this->Get('org_id');
		$iKey = $this->GetKey();
		$sIp = $this->Get('ip');
		$sMask = $this->Get('mask');
		$iIp = myip2long($sIp);
		$iMask = myip2long($sMask);	 
		$Size = MaskToSize($sMask);
		$iIpBroadcast = $iIp + $Size - 1;
		$sIpBroadcast = mylong2ip($iIpBroadcast);
		$iBlockId = $this->Get('block_id');
		
		// Forbid change of subnet IP but for subnet expansion
		//		As we look for subnet by its key, we cannot have an org mismatch
		$oSubnet = MetaModel::GetObject('IPv4Subnet', $iKey, false /* MustBeFound */);
		if (!is_null($oSubnet))
		{
//			if (($sIp != $oSubnet->Get('ip')) && ($this->m_WriteReason != ACTION_EXPAND))
			if (($sIp != $oSubnet->Get('ip')) && ($this->Get('write_reason') != 'expand'))
			{
				$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Subnet:IpCannotChange');
				return;
			}
		}
		
		// Forbid change of subnet mask unless required by programmatic functions
		//		As we look for subnet by its key, we cannot have an org mismatch
//		if (($this->m_WriteReason != ACTION_SHRINK) && ($this->m_WriteReason != ACTION_SPLIT) && ($this->m_WriteReason != ACTION_EXPAND))
	$sWriteReason = $this->Get('write_reason');
		if (($sWriteReason != 'shrink') && ($sWriteReason != 'split') && ($sWriteReason != 'expand'))
		{
			$oSubnet = MetaModel::GetObject('IPv4Subnet', $iKey, false /* MustBeFound */);
			if (!is_null($oSubnet) && ($sMask != $oSubnet->Get('mask')))
			{
			 	$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Subnet:MaskCannotChange');
			 	return;
			}
		}
		
		// Check consitency between subnet IP and mask. IP must be aligned with block defined by mask.
		if (!IsSubnetCIDRAligned($iIp, $iMask))
		{
			$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Subnet:IpIncorrect');
			return;
		}	 
		
		// Make sure subnet is fully contained in range
		$oBlock = MetaModel::GetObject('IPv4Block', $this->Get('block_id'), true /* MustBeFound */);
		$iBlockLastIp = myip2long($oBlock->Get('lastip'));
		if (($iIp < myip2long($oBlock->Get('firstip'))) || ($iBlockLastIp < $iIpBroadcast))
		{
			$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Subnet:NotInBlock');
			return;
		}
		
		// Make sure subnet doesn't collide with another subnet
		$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE s.block_id = $iBlockId AND s.org_id = $sOrgId"));
		while ($oSubnet = $oSubnetSet->Fetch())
		{
			// If it's a modification (keys are the same) further checks are not relevant
			if ($oSubnet->GetKey() != $iKey)
			{
				$iCurrentIp = myip2long($oSubnet->Get('ip'));
				$iCurrentIpBroadcast = myip2long($oSubnet->Get('broadcastip'));
				
				// Does the subnet already exist?
				if (($iCurrentIp == $iIp) && ($iCurrentIpBroadcast == $iIpBroadcast))
				{
					$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Subnet:Collision0');
					return;
				}
				// Is the subnet IP part of an existing subnet?
				if (($iCurrentIp <= $iIp) && ($iIp <= $iCurrentIpBroadcast))
				{
					$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Subnet:Collision1');
					return;
				}
				// Is the broadcast IPs part of an existing subnet?
				if (($iCurrentIp <= $iIpBroadcast) && ($iIpBroadcast <= $iCurrentIpBroadcast))
				{
					$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Subnet:Collision2');
					return;
				}
				// Is new subnet including an existing one?
				if (($iIp < $iCurrentIp) && ($iCurrentIpBroadcast < $iIpBroadcast))
				{
					$this->m_aCheckIssues[] = Dict::Format('UI:IPManagement:Action:New:IPv4Subnet:Collision3');
					return;
				}
			}
		}
		
		// Reset reason for action
//		$this->m_WriteReason = ACTION_NONE;
	$this->Set('write_reason', 'none');
	}
	
	/*
	 * Perform actions when new object inserted in DB 
	 */		 
	protected function OnInsert()
	{
		$sIp = $this->Get('ip');
		$iIp = myip2long($sIp);
		$sMask = $this->Get('mask');
		$iMask = myip2long($sMask);	 
		
		// Set raw IP attribute
		$this->Set('rip',$sIp);
		
		// Compute and register broadcast IP
		$iIpBroadcast = $iIp + MaskToSize($sMask) - 1;
		$this->Set('broadcastip', mylong2ip($iIpBroadcast));
	}
	
	/**
	 * Perform specific tasks related to subnet creation:
	 */	 
	protected function AfterInsert()
	{
		parent::AfterInsert();
		
		$sOrgId = $this->Get('org_id');
		
		// Check if subnet and broadcast IPs need to be created / updated or not
		$sReserveSubnetIPs = utils::ReadPostedParam('attr_reserve_subnet_IPs', '');
		if (empty($sReserveSubnetIPs))
		{
			$sReserveSubnetIPs = GetFromGlobalIPConfig('reserve_subnet_IPs', $sOrgId);
		}
		if ($sReserveSubnetIPs == 'reserve_yes')
		{
			$sIp = $this->Get('ip');
			$sIpBroadcast = $this->Get('broadcastip');
			
			// Create or update subnet IP
			$sUsageNetworkIpId = GetIpUsageId($sOrgId, NETWORK_IP_CODE);
			$oIp = MetaModel::GetObjectFromOQL("SELECT IPv4Address AS i WHERE i.ip = '$sIp' AND i.org_id = $sOrgId", null, false);
			if (is_null($oIp))
			{
				$oIp = MetaModel::NewObject('IPv4Address');
				$oIp->Set('ip', $sIp);
				$oIp->Set('org_id', $sOrgId);
				$oIp->Set('status', 'reserved');
				$oIp->Set('usage_id', $sUsageNetworkIpId);
				$oIp->DBInsert();
			}
			else
			{
				if (($oIp->Get('status') != 'reserved') || ($oIp->Get('usage_id') != $sUsageNetworkIpId))
				{
					$oIp->Set('status', 'reserved');
					$oIp->Set('usage_id', $sUsageNetworkIpId);
					$oIp->DBUpdate();
				}
			}
			
			// Create or update broadcast IP
			$sUsageBroadcastIpId = GetIpUsageId($sOrgId, BROADCAST_IP_CODE);
			$oIp = MetaModel::GetObjectFromOQL("SELECT IPv4Address AS i WHERE i.ip = '$sIpBroadcast' AND i.org_id = $sOrgId", null, false);
			if (is_null($oIp))
			{
				$oIp = MetaModel::NewObject('IPv4Address');
				$oIp->Set('ip', $sIpBroadcast);
				$oIp->Set('org_id', $sOrgId);
				$oIp->Set('status', 'reserved');
				$oIp->Set('usage_id', $sUsageBroadcastIpId);
				$oIp->DBInsert();
			}
			else
			{
				if (($oIp->Get('status') != 'reserved') || ($oIp->Get('usage_id') != $sUsageBroadcastIpId)) 
				{
					$oIp->Set('status', 'reserved');
					$oIp->Set('usage_id', $sUsageBroadcastIpId);
					$oIp->DBUpdate();
				}
			}
		}
	}
	
	/**
	 * Perform specific tasks related to subnet update:
	 */	 
	protected function AfterUpdate()
	{
		parent::AfterUpdate();
		
		$sOrgId = $this->Get('org_id');
		$sReserveSubnetIPs = GetFromGlobalIPConfig('reserve_subnet_IPs', $sOrgId);
		
		if ($sReserveSubnetIPs == 'reserve_yes')
		{
			$sIp = $this->Get('ip');
			$sIpBroadcast = $this->Get('broadcastip');
			
			// Create or update subnet IP
			$sUsageNetworkIpId = GetIpUsageId($sOrgId, NETWORK_IP_CODE);
			$oIp = MetaModel::GetObjectFromOQL("SELECT IPv4Address AS i WHERE i.ip = '$sIp' AND i.org_id = $sOrgId", null, false);
			if (is_null($oIp))
			{
				$oIp = MetaModel::NewObject('IPv4Address');
				$oIp->Set('ip', $sIp);
				$oIp->Set('org_id', $sOrgId);
				$oIp->Set('status', 'reserved');
				$oIp->Set('usage_id', $sUsageNetworkIpId);
				$oIp->DBInsert();
			}
			else
			{
				if (($oIp->Get('status') != 'reserved') || ($oIp->Get('usage_id') != $sUsageNetworkIpId))
				{
					$oIp->Set('status', 'reserved');
					$oIp->Set('usage_id', $sUsageNetworkIpId);
					$oIp->DBUpdate();
				}
			}
			
			// Create or update broadcast IP
			$sUsageBroadcastIpId = GetIpUsageId($sOrgId, BROADCAST_IP_CODE);
			$oIp = MetaModel::GetObjectFromOQL("SELECT IPv4Address AS i WHERE i.ip = '$sIpBroadcast' AND i.org_id = $sOrgId", null, false);
			if (is_null($oIp))
			{
				$oIp = MetaModel::NewObject('IPv4Address');
				$oIp->Set('ip', $sIpBroadcast);
				$oIp->Set('org_id', $sOrgId);
				$oIp->Set('status', 'reserved');
				$oIp->Set('usage_id', $sUsageBroadcastIpId);
				$oIp->DBInsert();
			}
			else
			{
				if (($oIp->Get('status') != 'reserved') || ($oIp->Get('usage_id') != $sUsageBroadcastIpId)) 
				{
					$oIp->Set('status', 'reserved');
					$oIp->Set('usage_id', $sUsageBroadcastIpId);
					$oIp->DBUpdate();
				}
			}
		}
	}
	
	/**
	 * Check validity of deletion request
	 */
	protected function DoCheckToDelete(&$oDeletionPlan)
	{
		$sOrgId = $this->Get('org_id');
		$sIp = $this->Get('ip');
		$sIpBroadcast = $this->Get('broadcastip');
		
		parent::DoCheckToDelete($oDeletionPlan);
		
		// Add subnet and broadcast IP to deletion plan if they exist
		$oIp = MetaModel::GetObjectFromOQL("SELECT IPv4Address AS i WHERE i.ip = '$sIp' AND i.org_id = $sOrgId", null, false);
		if (!is_null($oIp))
		{
			$oDeletionPlan->AddToDelete($oIp, DEL_AUTO);
		}
		
		$oIp = MetaModel::GetObjectFromOQL("SELECT IPv4Address AS i WHERE i.ip = '$sIpBroadcast' AND i.org_id = $sOrgId", null, false);
		if (!is_null($oIp))
		{
			$oDeletionPlan->AddToDelete($oIp, DEL_AUTO);
		}
	}
	
	/**
	 * Change flag of attributes that shouldn't be modified beside creation.
	 */
	public function GetAttributeFlags($sAttCode, &$aReasons = array(), $sTargetState = '')
	{
		if ((!$this->IsNew()) && (($sAttCode == 'org_id') || ($sAttCode == 'block_id') || ($sAttCode == 'ip') || ($sAttCode == 'mask') || ($sAttCode == 'ip_occupancy') || ($sAttCode == 'range_occupancy')))
		{
			return OPT_ATT_READONLY;
		}
		return parent::GetAttributeFlags($sAttCode, $aReasons, $sTargetState);
	}
	
	/**
	 * Provides attributes' parameters
	 */		 
	public function GetAttributeParams($sAttCode)
	{
		$aParams = array();
		if (($sAttCode == 'ip_occupancy') || ($sAttCode == 'range_occupancy'))
		{
			if ($sAttCode == 'ip_occupancy')
			{
				$Occupancy = $this->GetOccupancy('IPv4Address');
			}
			else
			{
				$Occupancy = $this->GetOccupancy('IPv4Range');
			}
			$sOrgId = $this->Get('org_id');
			if ($sOrgId != null)
			{
				$sLowWaterMark = GetFromGlobalIPConfig('subnet_low_watermark', $sOrgId);
				$sHighWaterMark = GetFromGlobalIPConfig('subnet_high_watermark', $sOrgId);
				if ($Occupancy >= $sHighWaterMark)
				{
					$sColor = RED;
				}
				else if ($Occupancy >= $sLowWaterMark)
				{
					$sColor = YELLOW;
				}
				else
				{
					$sColor = GREEN;
				}
				$aParams ['value'] = round ($Occupancy, 0);
				$aParams ['color'] = $sColor;
			}
			else
			{
				$aParams ['value'] = 0;
				$aParams ['color'] = GREEN;
			}
		}
		else
		{
			$aParams ['value'] = 0;
			$aParams ['color'] = GREEN;
		}
		return ($aParams);
	}

}

?>