<?php
// Copyright (C) 2010-2012 Combodo SARL
//
//   This file is part of TeemIp.
//
//   TeemIp is free software; you can redistribute it and/or modify	
//   it under the terms of the GNU Affero General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   TeemIp is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Affero General Public License for more details.
//
//   You should have received a copy of the GNU Affero General Public License
//   along with TeemIp. If not, see <http://www.gnu.org/licenses/>

/**
 * @copyright   Copyright (C) 2010-2012 Combodo SARL
 * @license     http://opensource.org/licenses/AGPL-3.0
 */

function GetToolkitMenu(WebPage $oPage, $oSet, $aExtraParams)
	{
		$aActions = array();
		$sHtml = '<div class="itop_popup toolkit_menu" id="tk_'.$this->iListId.'"><ul><li><img src="../images/toolkit_menu.png"><ul>';
		utils::GetPopupMenuItems($oPage, iPopupMenuExtension::MENU_OBJLIST_TOOLKIT, $oSet, $aActions);
		$sHtml = $oPage->RenderPopupMenuItems($aActions);
		return $sHtml;
	}
	

/**********************************************		
 * Displays Subnet Block tree for a given Org.
 */
function DisplayOrgBlockTree(WebPage $oP, $sOrgId, $bWithSubnet)
{
	$oBlockSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Block AS b WHERE b.org_id = $sOrgId"));
	$aTree = array();
	$aNodes = array();
	while($oBlock = $oBlockSet->Fetch())
	{
		$iParentId = $oBlock->Get('parent_id');
		$iKey = $oBlock->GetKey();
		if (!isset($aTree[$iParentId]))
		{
			$aTree[$iParentId] = array();
		}
		//$aTree[$iParentId][$iKey] = $oBlock->GetName();
		$aTree[$iParentId][$iKey] = myip2long($oBlock->Get('firstip'));
		$aNodes[$iKey] = $oBlock;
	}
		
	$aParents = array_keys($aTree);
	$aRoots = array();
	foreach($aParents as $id)
	{
		if (!array_key_exists($id, $aNodes))
		{
			$aRoots[] = $id;
		}
	}
	foreach($aRoots as $iRootId)
	{
			DumpBlockNodes($oP, $iRootId, $aTree, $aNodes, '', $bWithSubnet);
	}
}

/*****************************************
 * Displays nodes of a Subnet Block tree.
 */
function DumpBlockNodes($oP, $iRootId, $aTree, $aNodes, $currValue, $bWithSubnet)
{
	if (array_key_exists($iRootId, $aTree))
	{
		$bMultiple = false;
		
		$aSortedRoots = $aTree[$iRootId];
		asort($aSortedRoots);
		$oP->add("<ul>\n");
		foreach($aSortedRoots as $id => $sName)
		{
			if	($bWithSubnet)
			{
				$sNode = "&nbsp;&nbsp;".$aNodes[$id]->GetName();
				$sIcon = $aNodes[$id]->GetIcon(true, true);
			}
			else
			{
				$sNode = $aNodes[$id]->GetHyperlink();
				$sIcon = '';
			}
			$oP->add("<li>".$sIcon.$sNode);
			$oP->add("&nbsp;&nbsp;&nbsp;[".$aNodes[$id]->Get('firstip')." - ".$aNodes[$id]->Get('lastip')."]");
			$oP->add("&nbsp;&nbsp;&nbsp;".$aNodes[$id]->Get('type'));
			// Expand subnet list if required
			if ($bWithSubnet)
			{
				$BlockId = $aNodes[$id]->GetKey();
				$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE s.block_id = '$BlockId'"));
				if ($oSubnetSet->Count() != 0)
				{
					$oP->add("<ul>\n");
					while ($oSubnet = $oSubnetSet->Fetch())
					{
						$oP->add("<li>".$oSubnet->GetHyperlink());
						$oP->add("&nbsp;".Dict::S('Class:IPv4Subnet/Attribute:mask/Value_cidr:'.$oSubnet->Get('mask')));
						$oP->add("</li>\n");
					}
					$oP->add("</ul>\n");
				}
			}
			DumpBlockNodes($oP, $id, $aTree, $aNodes, $currValue, $bWithSubnet);
			
			$oP->add("</li>\n");
		}
		$oP->add("</ul>\n");
	}
}

/************************************	 
 * List space within a Subnet Range.
 */
function ListSpace(WebPage $oP, $sClass, $oObj)
{
	if ($sClass != 'IPv4Block')
	{
	 return;
	}
	
	// Get list of registered blocks and subnets in subnet range
	$iId = $oObj->GetKey();
	$sOrgId = $oObj->Get('org_id');
	$iObjFirstIp = myip2long($oObj->Get('firstip'));
	$iObjLastIp = myip2long($oObj->Get('lastip'));
	$iBlockSize = $oObj->GetBlockSize();
	$oChildBlockSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Block AS b WHERE b.parent_id = $iId AND b.org_id = $sOrgId"));
	$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE s.block_id = $iId AND s.org_id = $sOrgId"));
	
	$aList = array();
	$i = 0;
	while ($oChildBlock = $oChildBlockSet->Fetch())
	{
		$aList[$i] = array();
		$aList[$i]['type'] = 'IPv4Block';
		$aList[$i]['firstip'] = myip2long($oChildBlock->Get('firstip'));
		$aList[$i]['lastip'] = myip2long($oChildBlock->Get('lastip'));
		$aList[$i]['obj'] = $oChildBlock;
		$i++;
	}
	while ($oSubnet = $oSubnetSet->Fetch())
	{
		$aList[$i] = array();
		$aList[$i]['type'] = 'IPv4Subnet';
		$aList[$i]['firstip'] = myip2long($oSubnet->Get('ip'));
		$aList[$i]['lastip'] = myip2long($oSubnet->Get('broadcastip'));
		$aList[$i]['obj'] = $oSubnet;
		$i++;
	}
	// Sort $aList by 'firstip'
	if (!empty($aList))
	{
		foreach ($aList as $key => $row)
		{
			$aFirstIp[$key]	= $row['firstip'];
		}
		array_multisort($aFirstIp, SORT_ASC, $aList);
	}
	
	// Preset display of name and subnet attributes
	$sHtml = "&nbsp;[".$oObj->GetAsHTML('firstip')." - ".$oObj->GetAsHTML('lastip')."]";
	
	$oAppContext = new ApplicationContext();
	$sParams = $oAppContext->GetForLink();
	
	// Display Block ref
	$oP->add("<ul>\n");
	$oP->add("<li>".$oObj->GetHyperlink().$sHtml."<ul>\n");
	
	// Display sub ranges as list
	$iSizeArray = $i;
	$i = 0;
	$iAnIp = $iObjFirstIp;
	while ($iAnIp <= $iObjLastIp)
	{
		$sAnIp = mylong2ip($iAnIp);
		if ($i < $iSizeArray)
		{
			if ($iAnIp < $aList[$i]['firstip'])
			{
				// Display free space
				$iLastIp = $aList[$i]['firstip'] - 1;
				$iNbIps = $iLastIp - $iAnIp + 1;
				$iFormatNbIps = number_format($iNbIps, 0, ',', ' ');
				$oP->add("<li>".Dict::Format('UI:IPManagement:Action:ListSpace:IPv4Block:FreeSpace',$sAnIp, mylong2ip($iLastIp), $iFormatNbIps, ($iNbIps / $iBlockSize) * 100));	
				$iAnIp = $aList[$i]['firstip'];
			}
			else if ($iAnIp == $aList[$i]['firstip'])
			{
				// Display object attributes
				$sIcon = $aList[$i]['obj']->GetIcon(true, true);
				$oP->add("<li>".$sIcon.$aList[$i]['obj']->GetHyperlink());
				if ($aList[$i]['type'] == 'IPv4Subnet')
				{
					$oP->add("&nbsp;".Dict::S('Class:IPv4Subnet/Attribute:mask/Value_cidr:'.$aList[$i]['obj']->Get('mask')));
				}
				else
				{
					$oP->add("&nbsp;[".mylong2ip($aList[$i]['firstip'])." - ".mylong2ip($aList[$i]['lastip'])."]");
				}
				$iAnIp = $aList[$i]['lastip'] + 1;
				$i++;
			}
		}
		else
		{
			$iNbIps = $iObjLastIp - $iAnIp + 1;
			$iFormatNbIps = number_format($iNbIps, 0, ',', ' ');
			$oP->add("<li>".Dict::Format('UI:IPManagement:Action:ListSpace:IPv4Block:FreeSpace',$sAnIp, mylong2ip($iObjLastIp), $iFormatNbIps, ($iNbIps / $iBlockSize) * 100));
			$iAnIp = $iObjLastIp + 1;
		}
		$oP->add("</li>\n");
	}
	$oP->add("</ul></li></ul>\n");
}

/************************************************
 * Find space within a Subnet Range or a Subnet.
 */
function DoFindSpace(WebPage $oP, $sClass, $oObj, $iChangeId, $sParameter)
{
	if (($sClass != 'IPv4Block') && ($sClass != 'IPv4Subnet'))
	{
		return;
	}
	
	$iId = $oObj->GetKey();
	$sOrgId = $oObj->Get('org_id');
	if ($sClass == 'IPv4Block')
	{
		$iSize = $sParameter['spacesize'];
		$bitMask = SizeToMask($iSize);
		$iMaxOffer = $sParameter['maxoffer'];
		$sStatusSubnet = $sParameter['status_subnet'];
		$sType = $sParameter['type'];
		$iLocationId = $sParameter['location_id'];
		$iRequestorId = $sParameter['requestor_id'];
		$bOfferBlock = ($iChangeId == 0) ? true : false;
		$bOfferSubnet = ($iSize <= SUBNET_MAX_SIZE) ? true : false;
		
		// Get list of free space in subnet range
		$oBlock = MetaModel::GetObject('IPv4Block', $iId, true /* MustBeFound */);
		$aFreeSpace = $oBlock->GetFreeSpace($iSize, $iMaxOffer);
		
		$oAppContext = new ApplicationContext();
		$sParams = $oAppContext->GetForLink();
		
		// Display Summary of parameters
		$oP->add("<ul>\n");
		$oP->add("<li>"."&nbsp;".Dict::Format('UI:IPManagement:Action:DoFindSpace:IPv4Block:Summary', $iMaxOffer, SizeToBit($iSize))."<ul>\n");
		
		// Display possible choices as list
		$iSizeFreeArray = sizeof ($aFreeSpace);
		if ($iSizeFreeArray != 0)
		{
			$i = 0;
			$iVIdCounter = 1;
			do
			{
				$sAnIp = $aFreeSpace[$i]['firstip'];
				$sLastIp = $aFreeSpace[$i]['lastip'];
				$sMask = $aFreeSpace[$i]['mask'];
				$oP->add("<li>".$sAnIp." - ".$sLastIp."\n"."<ul>");
				
				// Create as a block
				if ($bOfferBlock)
				{
					$iVId = $iVIdCounter++;
					$sHTMLValue = "<li><div><span id=\"v_{$iVId}\">";
					$sHTMLValue .= "<img style=\"border:0;vertical-align:middle;cursor:pointer;\" src=\"".utils::GetAbsoluteUrlModulesRoot()."/teemip-ip-mgmt/images/ipmini-add-xs.png\" onClick=\"oIpWidget_{$iVId}.DisplayCreationForm();\"/>&nbsp;";
					$sHTMLValue .= "&nbsp;".Dict::Format('UI:IPManagement:Action:DoFindSpace:IPv4Block:CreateAsBlock')."&nbsp;&nbsp;";
					$sHTMLValue .= "</span></div></li>\n";
					$oP->add($sHTMLValue);	
					$oP->add_ready_script(
<<<EOF
					oIpWidget_{$iVId} = new IpWidget($iVId, 'IPv4Block', $iChangeId, {'org_id': '$sOrgId', 'parent_id': '$iId', 'firstip': '$sAnIp', 'lastip': '$sLastIp'});
EOF
					);
				}
				
				// Create as a subnet
				if ($bOfferSubnet)
				{
					$iVId = $iVIdCounter++;
					$sHTMLValue = "<li><div><span id=\"v_{$iVId}\">";
					$sHTMLValue .= "<img style=\"border:0;vertical-align:middle;cursor:pointer;\" src=\"".utils::GetAbsoluteUrlModulesRoot()."/teemip-ip-mgmt/images/ipmini-add-xs.png\" onClick=\"oIpWidget_{$iVId}.DisplayCreationForm();\"/>&nbsp;";
					$sHTMLValue .= "&nbsp;".Dict::Format('UI:IPManagement:Action:DoFindSpace:IPv4Block:CreateAsSubnet')."&nbsp;&nbsp;";
					$sHTMLValue .= "</span></div></li>\n";
					$oP->add($sHTMLValue);	
					$oP->add_ready_script(
<<<EOF
					oIpWidget_{$iVId} = new IpWidget($iVId, 'IPv4Subnet', $iChangeId, {'org_id': '$sOrgId', 'block_id': '$iId', 'ip': '$sAnIp', 'mask': '$sMask', 'status': '$sStatusSubnet', 'type': '$sType', 'location_id': '$iLocationId', 'requestor_id': '$iRequestorId'});
EOF
					);
				}
					
				$oP->add("</ul></li>\n"); 
			}
			while (++$i < $iSizeFreeArray);
		}
		else
		{
			$sHTMLValue = "";
		}
		$oP->add("</ul></li></ul>\n");
	}
	else // IPv4Subnet
	{
		$iRangeSize = utils::ReadPostedParam('rangesize', '', 'raw_data');
		$iMaxOffer = utils::ReadPostedParam('maxoffer', '', 'raw_data');
		
		// Get list of registered IPs & ranges in subnet
		$iSubnetSize = MaskToSize($oObj->Get('mask'));
		if ($iRangeSize >= $iSubnetSize)
		{
			// Required range size is to big, exit
			$oP->add(Dict::Format('UI:IPManagement:Action:DoFindSpace:IPv4Subnet:RangeTooBig')."<br><br>");
		}
		else
		{
			// Get list of free space in subnet
			$oSubnet = MetaModel::GetObject('IPv4Subnet', $iId, true /* MustBeFound */);
			$aFreeSpace = $oSubnet->GetFreeSpace($iRangeSize, $iMaxOffer);
			
			$oAppContext = new ApplicationContext();
			$sParams = $oAppContext->GetForLink();
			
			// Display Summary of parameters
			$oP->add("<ul>\n");
			$oP->add("<li>"."&nbsp;".Dict::Format('UI:IPManagement:Action:DoFindSpace:IPv4Subnet:Summary', $iMaxOffer, $iRangeSize)."<ul>\n");
			
			// Display possible choices as list
			$iSizeFreeArray = sizeof ($aFreeSpace);
			if ($iSizeFreeArray != 0)
			{
				$i = 0;
				$iVIdCounter = 1;
				do
				{
					$sRangeFirstIp = $aFreeSpace[$i]['firstip'];
					$sRangeLastIp = $aFreeSpace[$i]['lastip'];
					$oP->add("<li>".$sRangeFirstIp." - ".$sRangeLastIp."\n");
					
					// Create as a range
					$iVId = $iVIdCounter++;
					$sHTMLValue = "<ul><li><div><span id=\"v_{$iVId}\">";
					$sHTMLValue .= "<img style=\"border:0;vertical-align:middle;cursor:pointer;\" src=\"".utils::GetAbsoluteUrlModulesRoot()."/teemip-ip-mgmt/images/ipmini-add-xs.png\" onClick=\"oIpWidget_{$iVId}.DisplayCreationForm();\"/>&nbsp;";
					$sHTMLValue .= "&nbsp;".Dict::Format('UI:IPManagement:Action:DoFindSpace:IPv4Subnet:CreateAsRange')."&nbsp;&nbsp;";
					$sHTMLValue .= "</span></div></li>\n";
					$oP->add($sHTMLValue);
					$oP->add_ready_script(
<<<EOF
					oIpWidget_{$iVId} = new IpWidget($iVId, 'IPv4Range', $iChangeId, {'org_id': '$sOrgId', 'subnet_id': '$iId', 'firstip': '$sRangeFirstIp', 'lastip': '$sRangeLastIp'});
EOF
					);
					$oP->add("</ul></li>\n"); 
				}
			while (++$i < $iSizeFreeArray);
		}
		else
		{
			$sHTMLValue = "";
		}
		$oP->add("</ul></li></ul>\n");
		}
	} 
}

/***************************************
 * List IPs for a given Subnet or Range
 */
function ListIps(WebPage $oP, $sClass, $oObj, $iChangeId, $sParameter)
{
	switch ($sClass)
	{
		case 'IPv4Range':
			// Get list of registered IPs in range
			$iId = $oObj->GetKey();
			$sOrgId = $oObj->Get('org_id');
			$sFirstIp = $oObj->Get('firstip');
			$sLastIp = $oObj->Get('lastip');
			$bitFirstIp = myip2long($sFirstIp);
			$bitLastIp = myip2long($sLastIp);
			$oIpRegisteredSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS ipv4 WHERE INET_ATON('$sFirstIp') <= INET_ATON(ipv4.ip) AND INET_ATON(ipv4.ip) <= INET_ATON('$sLastIp') AND ipv4.org_id = $sOrgId"));
			$aRegisteredIPs = $oIpRegisteredSet->GetColumnAsArray('ip', false);
			$bWithRanges = false;
			
			// Preset display of name and subnet attributes
			$sHtml = "&nbsp;[".$oObj->GetAsHTML('firstip')." - ".$oObj->GetAsHTML('lastip')."]";
		break;
		
		case 'IPv4Subnet':
		default:
			// Get list of registered IPs & Ranges in subnet
			$iId = $oObj->GetKey();
			$sOrgId = $oObj->Get('org_id');
			$sIp = $oObj->Get('ip');
			$sMask = $oObj->Get('mask');
			$sIpBroadcast = $oObj->Get('broadcastip');
			$bitFirstIp = myip2long($sIp);
			$bitLastIp = myip2long($sIpBroadcast);
			$oIpRegisteredSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS ipv4 WHERE INET_ATON('$sIp') <= INET_ATON(ipv4.ip) AND INET_ATON(ipv4.ip) <= INET_ATON('$sIpBroadcast') AND ipv4.org_id = $sOrgId"));
			$aRegisteredIPs = $oIpRegisteredSet->GetColumnAsArray('ip', false);
			$oIpRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Range AS rangev4 WHERE rangev4.subnet_id = $iId"));
			$aRangeIPs = $oIpRangeSet->GetColumnAsArray('firstip', false);
			$oIpRangeSet->Rewind();
			$bWithRanges = true;
			
			// Preset display of name and subnet attributes
			$sHtml = "&nbsp;".Dict::S('Class:IPv4Subnet/Attribute:mask/Value_cidr:'.$oObj->Get('mask'))."	 - ".$oObj->GetLabel('type').': '.$oObj->GetAsHTML('type');
		break;
	}
	$sStatusIp = $sParameter['status_ip'];
	$sShortName = $sParameter['short_name'];
	$sDomainName = $sParameter['domain_name'];
	$iUsageId = $sParameter['usage_id'];
	$sMacAddress = $sParameter['mac_address'];
	$iRequestorId = $sParameter['requestor_id'];
	
	$iAnIp = $bWithRanges ? ($bitFirstIp + 1) : $bitFirstIp;
	$iLastRangeIp = $bitFirstIp;
	$i = 0;
	$iVIdCounter = 1;
	$oAppContext = new ApplicationContext();
	$sParams = $oAppContext->GetForLink();
	
	// Check user rights
	$UserHasRightsToCreate = (UserRights::IsActionAllowed($sClass, UR_ACTION_MODIFY) == UR_ALLOWED_YES) ? true : false;
	
	// Display first IP
	$oP->add("<ul>\n");
	$oP->add("<li>".$oObj->GetIcon(true,true).$oObj->GetHyperlink().$sHtml."<ul>\n");
	
	// Display other IPs as list
	while ($iAnIp <= $bitLastIp)
	{
		$sAnIp = mylong2ip($iAnIp);
		if ($bWithRanges)
		{
			if (in_array($sAnIp, $aRangeIPs))
			{ 
				// Found a range within list of IPs
				$oIpRangeSet->Rewind();
				$oIpRange = $oIpRangeSet->Fetch();
				while ($oIpRange->Get('firstip') != $sAnIp)
				{
					$oIpRange = $oIpRangeSet->Fetch();
				}
				
				// Display name and range attributes
				$sIcon = $oIpRange->GetIcon(true, true);
				$oP->add("<li>".$sIcon.$oIpRange->GetHyperlink()."&nbsp;&nbsp;&nbsp;[".$oIpRange->Get('firstip')." - ".$oIpRange->Get('lastip')."]");
				$oP->add("&nbsp;&nbsp; - ".$oIpRange->GetLabel('usage').': '.$oIpRange->GetAsHTML('usage')."<ul>\n");
				$iLastRangeIp = myip2long($oIpRange->Get('lastip'));
			}
		}
		if (in_array($sAnIp, $aRegisteredIPs))
		{
			// Found registered IP
			$oIpRegisteredSet->Rewind();
			$oIpRegistered = $oIpRegisteredSet->Fetch();
			while ($oIpRegistered->Get('ip') != $sAnIp)
			{
				$oIpRegistered = $oIpRegisteredSet->Fetch();
			}
			$sDomainName = $oIpRegistered->Get('domain_name');
			if ($sDomainName != '')
			{
				$sDotDomainName = ".".$sDomainName;
			}
			else
			{
				$sDotDomainName = ""; 
			}
			$oP->add("<li>".$oIpRegistered->GetHyperlink()."&nbsp;&nbsp; - ".$oIpRegistered->GetAsHTML('status')."&nbsp;&nbsp; - ".$oIpRegistered->Get('short_name').$sDotDomainName);
		}
		else
		{
			// If user has rights to create IPs
			// Display unregistered IP with icon to create it
			if  ($UserHasRightsToCreate)
			{
				$iVId = $iVIdCounter++;
				$sHTMLValue = "<li><div><span id=\"v_{$iVId}\">";
				$sHTMLValue .= "<img style=\"border:0;vertical-align:middle;cursor:pointer;\" src=\"".utils::GetAbsoluteUrlModulesRoot()."/teemip-ip-mgmt/images/ipmini-add-xs.png\" onClick=\"oIpWidget_{$iVId}.DisplayCreationForm();\"/>&nbsp;";
				$sHTMLValue .= "&nbsp;".$sAnIp."&nbsp;&nbsp;";
				$sHTMLValue .= "</span></div>";
				$oP->add($sHTMLValue);	
				$oP->add_ready_script(
<<<EOF
				oIpWidget_{$iVId} = new IpWidget($iVId, 'IPv4Address', $iChangeId, {'org_id': '$sOrgId', 'ip': '$sAnIp', 'status': '$sStatusIp', 'short_name': '$sShortName', 'domain_name': '$sDomainName', 'usage_id': '$iUsageId', 'mac_address': '$sMacAddress', 'requestor_id': '$iRequestorId'});
EOF
				);
			}
			else
			{
				$oP->add("<li>".$sAnIp);
			}
		}
		if ($bWithRanges)
		{
			if ($iAnIp == $iLastRangeIp)
			{
				$oP->add("</ul></li>\n");
			}
		}
		else
		{
			$oP->add("</li>\n");
		}
		$iAnIp++;
	}
	$oP->add("</ul></li></ul>\n");
}

/*******************
 * Set page titles.
 */
function SetPageTitles(WebPage $oP, $sUIPath, $oObj, $sClassLabel, $bIcon = true)
{
	$oP->set_title(Dict::Format($sUIPath.'PageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
	$oP->add("<div class=\"page_header\">\n");
	if ($bIcon)
	{
		$oP->add("<h1>".$oObj->GetIcon()."&nbsp;".Dict::Format($sUIPath.'Title_Class_Object', $sClassLabel, $oObj->GetName())."</h1>\n");
	}
	else
	{
		$oP->add("<h1>".Dict::Format($sUIPath.'Title_Class_Object', $sClassLabel, $oObj->GetName())."</h1>\n");
	}
	$oP->add("</div>\n");
}

/**************************************
 * Displays choices related to action.
 */			
function DisplayActionForm(WebPage $oP, $oAppContext, $sOperation, $sClass, $id, $oObj, $aDefault = array())
{
	$sClassLabel = MetaModel::GetName($sClass);
	switch ($sClass)
	{
		case 'IPv4Block':
			switch ($sOperation)
			{
				case 'findspace':
					$sUIPath = 'UI:IPManagement:Action:FindSpace:IPv4Block:';
					$sNextOperation = 'dofindspace';
				break;
				
				case 'shrinkblock':
					$sUIPath = 'UI:IPManagement:Action:Shrink:IPv4Block:';
					$sNextOperation = 'applyshrinkblock';
				break;
				
				case 'splitblock':
					$sUIPath = 'UI:IPManagement:Action:Split:IPv4Block:';
					$sNextOperation = 'applysplitblock';
				break;
				
				case 'expandblock':
				default:
					$sUIPath = 'UI:IPManagement:Action:Expand:IPv4Block:';
					$sNextOperation = 'applyexpandblock';
				break;
			}
			
			// Set page titles
			SetPageTitles($oP, $sUIPath, $oObj, $sClassLabel);
			
			// Set blue modification frame 
			$oP->add("<div class=\"wizContainer\">\n");
			
			// Preparation to allow new values to be posted
			$aFieldsMap = array();
			$sPrefix = '';
			$m_iFormId = $oObj->GetNewFormId($sPrefix);
			$iTransactionId = utils::GetNewTransactionId();
			$oP->SetTransactionId($iTransactionId);
			$sFormAction= utils::GetAbsoluteUrlModulesRoot()."/teemip-ip-mgmt/ui.teemip-ip-mgmt.php";
			$oP->add("<form action=\"$sFormAction\" id=\"form_{$m_iFormId}\" enctype=\"multipart/form-data\" method=\"post\" onSubmit=\"return OnSubmit('form_{$m_iFormId}');\">\n");
			$oP->add_ready_script("$(window).unload(function() { OnUnload('$iTransactionId') } );\n");
			
			// Display tabs and actions
			if ($sOperation != 'findspace')
			{
				DisplayMainAttributes($oP, $sOperation, $oObj, $m_iFormId, $sPrefix, $aFieldsMap, $aDefault);
				DisplayGlobalConfig($oP, $sOperation, $oObj->Get('org_id'), $aDefault);
				$oP->SetCurrentTab('');
			}
			
			$iScaleOfAction = 0;
		break; // IPv4Block
		
		case 'IPv4Subnet':
			switch ($sOperation)
			{
				case 'findspace':
					$sUIPath = 'UI:IPManagement:Action:FindSpace:IPv4Subnet:';
					$sNextOperation = 'dofindspace';
				break;
				
				case 'shrinksubnet':
					$sUIPath = 'UI:IPManagement:Action:Shrink:IPv4Subnet:';
					$sNextOperation = 'applyshrinksubnet';
				break;
				
				case 'splitsubnet':
					$sUIPath = 'UI:IPManagement:Action:Split:IPv4Subnet:';
					$sNextOperation = 'applysplitsubnet';
				break;
				
				case 'expandsubnet':
				default:
					$sUIPath = 'UI:IPManagement:Action:Expand:IPv4Subnet:';
					$sNextOperation = 'applyexpandsubnet';
				break;
			}
			
			// Set page titles
			SetPageTitles($oP, $sUIPath, $oObj, $sClassLabel);
			
			// Make sure action can be performed
			$sMask = $oObj->Get('mask');
			$sContinue = true;
			switch ($sOperation)
			{
				case 'findspace':
					if (MaskToBit($sMask) > 28)
					{
						// No point to look for space in less than /28
						$oP->add(Dict::Format($sUIPath.'SizeTooSmall')."<br><br>");
						$oP->add("<input type=\"button\" value=\"".Dict::S('UI:Button:Cancel')."\" onClick=\"window.history.back()\">&nbsp;&nbsp;");
						$sContinue = false;
					}
				break;
				
				case 'shrinksubnet':
				case 'splitsubnet':
					if (MaskToBit($sMask) > 30)
					{
						// To small to be shrunk or split. Minimum size is /30
						$oP->add(Dict::Format($sUIPath.'SizeTooSmall')."<br><br>");
						$oP->add("<input type=\"button\" value=\"".Dict::S('UI:Button:Cancel')."\" onClick=\"window.history.back()\">&nbsp;&nbsp;");
						$sContinue = false;
					}
				break;
				
				case 'expandsubnet':
				default:
					if (MaskToBit($sMask) < 17)
					{
						// To big to be expanded. Maximum size is /17 (by choice - bigger doesn't make sense))
						$oP->add(Dict::Format($sUIPath.'SizeTooBig')."<br><br>");
						$oP->add("<input type=\"button\" value=\"".Dict::S('UI:Button:Cancel')."\" onClick=\"window.history.back()\">&nbsp;&nbsp;");
						$sContinue = false;
					}
				break;
			}
			
			if ($sContinue)
			{
				// Set blue modification frame 
				$oP->add("<div class=\"wizContainer\">\n");
				
				// Preparation to allow new values to be posted
				$aFieldsMap = array();
				$sPrefix = '';
				$m_iFormId = $oObj->GetNewFormId($sPrefix);
				$iTransactionId = utils::GetNewTransactionId();
				$oP->SetTransactionId($iTransactionId);
				$sFormAction= utils::GetAbsoluteUrlModulesRoot()."/teemip-ip-mgmt/ui.teemip-ip-mgmt.php";
				$oP->add("<form action=\"$sFormAction\" id=\"form_{$m_iFormId}\" enctype=\"multipart/form-data\" method=\"post\" onSubmit=\"return OnSubmit('form_{$m_iFormId}');\">\n");
				$oP->add_ready_script("$(window).unload(function() { OnUnload('$iTransactionId') } );\n");
				
				// Display summary of parameters for subnet to be shrunk
				if ($sOperation != 'findspace')
				{
					DisplayMainAttributes($oP, $sOperation, $oObj, $m_iFormId, $sPrefix, $aFieldsMap, $aDefault);
					$oP->SetCurrentTab('');	
				}
				
				switch ($sOperation)
				{
					case 'shrinksubnet':
					case 'splitsubnet':
						// Display possible actions
						switch ($sMask)
						{
							case '255.255.255.252':
								// A /30 can only be shrunk or split by 2
								$iScaleOfAction = 1;
							break;
							
							case '255.255.255.248':
								// A /29 can be shrunk or split by 2 or 4
								$iScaleOfAction = 2;
							break;
							
							case '255.255.255.240':
								// A /28 can be shrunk or split by 2, 4 or 8
								$iScaleOfAction = 3;
							break;
							
							default:
								// All other subnets can be shrunk or split by 2, 4, 8 or 16
								$iScaleOfAction = 4;
							break;		
						}
					break;
					
					case 'expandsubnet':
					default:
						switch ($sMask)
						{
							case '255.255.128.0.':
								// A /128 can only be expanded by 2
								$iScaleOfAction = 1;
							break;
							
							case '255.255.192.0':
								// A /192 can be expanded by 2 or 4
								$iScaleOfAction = 2;
							break;
							
							case '255.255.224.0':
								// A /192 can be expanded by 2, 4 or 8
								$iScaleOfAction = 3;
							break;
							
							default:
								// All other subnets can be expanded by 2, 4, 8 or 16
								$iScaleOfAction = 4;
							break; 
						}
					break;
				}
			}
			else
			{
				return;
			}
		break; // Ipv4Subnet
							
		case 'IPv4AddressChangeCreate':
		case 'IPv4SubnetChangeCreate':
			$sUIPath = 'UI:IPManagement:Action:Implement:IPObjChange:';
			$sNextOperation = 'apply_stimulus';
				
			// Set page titles
			SetPageTitles($oP, $sUIPath, $oObj, '', false);
			
			// Set blue modification frame 
			$oP->add("<div class=\"wizContainer\">\n");
								
			// Preparation to allow new values to be posted
			$aFieldsMap = array();
			$sPrefix = '';
			$m_iFormId = $oObj->GetNewFormId($sPrefix);
			$iTransactionId = utils::GetNewTransactionId();
			$oP->SetTransactionId($iTransactionId);
			$sFormAction= utils::GetAbsoluteUrlModulesRoot()."/teemip-ip-mgmt/ui.teemip-ip-mgmt.php";
			$oP->add("<form action=\"$sFormAction\" id=\"form_{$m_iFormId}\" enctype=\"multipart/form-data\" method=\"post\" onSubmit=\"return OnSubmit('form_{$m_iFormId}');\">\n");
			$oP->add_ready_script("$(window).unload(function() { OnUnload('$iTransactionId') } );\n");
			$iScaleOfAction = 0;
		break;
		
		default:
		break;
	}
	
	// Display available actions
	DisplayActionFields($oP, $sOperation, $sClass, $oObj, $m_iFormId, $iScaleOfAction, $aDefault);
	
	// Load other parameters to post
	$oP->add($oAppContext->GetForForm());
	$oP->add("<input type=\"hidden\" name=\"operation\" value=\"$sNextOperation\">\n");	
	$oP->add("<input type=\"hidden\" name=\"class\" value=\"$sClass\">\n");
	$oP->add("<input type=\"hidden\" name=\"transaction_id\" value=\"$iTransactionId\">\n");
	$oP->add("<input type=\"hidden\" name=\"id\" value=\"$id\">\n");
	
	$oP->add('</form>');
	$oP->add("</div>\n");
	
	$iFieldsCount = count($aFieldsMap);
	$sJsonFieldsMap = json_encode($aFieldsMap);
	$sState = $oObj->GetState();
	$oP->add_script(
<<<EOF
	// Create the object once at the beginning of the page...
	var oWizardHelper$sPrefix = new WizardHelper('$sClass', '$sPrefix', '$sState');
	oWizardHelper$sPrefix.SetFieldsMap($sJsonFieldsMap);
	oWizardHelper$sPrefix.SetFieldsCount($iFieldsCount);
EOF
);

}

/********************************************************************************
 * Displays main attibutes of block or subnet on which action will be performed
 */
function DisplayMainAttributes(WebPage $oP, $sOperation, $oObj, $m_iFormId, $sPrefix, $aFieldsMap, $aDefault = array())
{
	// Display main frame and title
	$oP->AddTabContainer(OBJECT_PROPERTIES_TAB);
	$oP->SetCurrentTabContainer(OBJECT_PROPERTIES_TAB);
	switch ($sOperation)
	{
		case 'shrinkblock':
			$sLabelOfAction = Dict::S('UI:IPManagement:Action:Shrink:IPv4Block:Summary');
		break;
		
		case 'splitblock':
			$sLabelOfAction = Dict::S('UI:IPManagement:Action:Split:IPv4Block:Summary');
		break;
					
		case 'expandblock':
			$sLabelOfAction = Dict::S('UI:IPManagement:Action:Expand:IPv4Block:Summary');
		break;
					
		case 'shrinksubnet':
			$sLabelOfAction = Dict::S('UI:IPManagement:Action:Shrink:IPv4Subnet:Summary');
		break;
			
		case 'splitsubnet':
			$sLabelOfAction = Dict::S('UI:IPManagement:Action:Split:IPv4Subnet:Summary');
		break;
			
		case 'expandsubnet':
		default:
			$sLabelOfAction = Dict::S('UI:IPManagement:Action:Expand:IPv4Subnet:Summary');
		break;	
	}
	$oP->SetCurrentTab($sLabelOfAction);
	$oP->add('<table style="vertical-align:top"><tr>');
	$oP->add('<td style="vertical-align:top">');
	
	$aDetails = array();
	
	switch ($sOperation)
	{
		case 'shrinkblock':
		case 'splitblock':
		case 'expandblock':
			// Parent ID
			$sDisplayValue = $oObj->GetAsHTML('parent_id');	
			$aDetails[] = array('label' => '<span title="'.MetaModel::GetDescription('IPv4Block', 'parent_id').'">'.MetaModel::GetLabel('IPv4Block', 'parent_id').'</span>', 'value' => $sDisplayValue); 					
			
			// First IP
			$sDisplayValue = $oObj->GetAsHTML('firstip');	
			$aDetails[] = array('label' => '<span title="'.MetaModel::GetDescription('IPv4Block', 'firstip').'">'.MetaModel::GetLabel('IPv4Block', 'firstip').'</span>', 'value' => $sDisplayValue);					
			
			// Last IP
			$sDisplayValue = $oObj->GetAsHTML('lastip');	
			$aDetails[] = array('label' => '<span title="'.MetaModel::GetDescription('IPv4Block', 'lastip').'">'.MetaModel::GetLabel('IPv4Block', 'lastip').'</span>', 'value' => $sDisplayValue);
		break;
		
		case 'shrinksubnet':
		case 'splitsubnet':
		case 'expandsubnet':
		default:
			// Subnet Range
			$sDisplayValue = $oObj->GetAsHTML('block_id');	
			$aDetails[] = array('label' => '<span title="'.MetaModel::GetDescription('IPv4Subnet', 'block_id').'">'.MetaModel::GetLabel('IPv4Subnet', 'block_id').'</span>', 'value' => $sDisplayValue);
			
			// Subnet IP
			$sDisplayValue = $oObj->GetAsHTML('ip');	
			$aDetails[] = array('label' => '<span title="'.MetaModel::GetDescription('IPv4Subnet', 'ip').'">'.MetaModel::GetLabel('IPv4Subnet', 'ip').'</span>', 'value' => $sDisplayValue);
			
			// Mask
			$sDisplayValue = $oObj->GetAsHTML('mask');	
			$aDetails[] = array('label' => '<span title="'.MetaModel::GetDescription('IPv4Subnet', 'mask').'">'.MetaModel::GetLabel('IPv4Subnet', 'mask').'</span>', 'value' => $sDisplayValue);
		break;
	};
								 
	// Requestor ID - Can be modified
	$sInputId = $m_iFormId.'_'.'requestor_id';
	$oAttDef = MetaModel::GetAttributeDef('IPv4Subnet', 'requestor_id');
	$sValue = (array_key_exists('requestor_id', $aDefault)) ? $aDefault['requestor_id'] : $oObj->Get('requestor_id');
	$iFlags = $oObj->GetAttributeFlags('requestor_id');
	$aArgs = array('this' => $oObj, 'formPrefix' => $sPrefix);
	$sHTMLValue = "<span id=\"field_{$sInputId}\">".$oObj->GetFormElementForField($oP, 'IPv4Subnet', 'requestor_id', $oAttDef, $sValue, '', $sInputId, '', $iFlags, $aArgs).'</span>';
	$aFieldsMap['requestor_id'] = $sInputId;
	$aDetails[] = array('label' => '<span title="'.$oAttDef->GetDescription().'">'.$oAttDef->GetLabel().'</span>', 'value' => $sHTMLValue);
	
	$oP->Details($aDetails);
	$oP->add('</td>');
	$oP->add('</tr></table>');
}

/**************************************
 * Displays choices related to action. 
 */ 
function DisplayActionFields(WebPage $oP, $sOperation, $sClass, $oObj, $m_iFormId, $iScaleMax, $aDefault = array())
{
	switch ($sOperation)
	{
		case 'findspace':
			if ($sClass == 'IPv4Block')
			{
				$sLabelOfAction1 = Dict::S('UI:IPManagement:Action:FindSpace:IPv4Block:SizeOfSpace');
				$sLabelOfAction2 = Dict::S('UI:IPManagement:Action:FindSpace:IPv4Block:MaxNumberOfOffer');
			}
			else
			{
				$sLabelOfAction1 = Dict::S('UI:IPManagement:Action:FindSpace:IPv4Subnet:SizeOfRange');
				$sLabelOfAction2 = Dict::S('UI:IPManagement:Action:FindSpace:IPv4Subnet:MaxNumberOfOffer');
			}
		break;
		
		case 'shrinkblock':
			$sLabelOfAction1 = Dict::S('UI:IPManagement:Action:Shrink:IPv4Block:NewFirstIP');
			$sLabelOfAction2 = Dict::S('UI:IPManagement:Action:Shrink:IPv4Block:NewLastIP');
		break;
		
		case 'splitblock':
			$sLabelOfAction1 = Dict::S('UI:IPManagement:Action:Split:IPv4Block:At');
			$sLabelOfAction2 = Dict::S('UI:IPManagement:Action:Split:IPv4Block:NameNewBlock');
		break;
					
		case 'expandblock':
			$sLabelOfAction1 = Dict::S('UI:IPManagement:Action:Expand:IPv4Block:NewFirstIP');
			$sLabelOfAction2 = Dict::S('UI:IPManagement:Action:Expand:IPv4Block:NewLastIP');
		break;
					
		case 'shrinksubnet':
			$sLabelOfAction1 = Dict::S('UI:IPManagement:Action:Shrink:IPv4Subnet:By');
		break; 
		
		case 'splitsubnet':
			$sLabelOfAction1 = Dict::S('UI:IPManagement:Action:Split:IPv4Subnet:In');
		break;
		
		case 'expandsubnet':
			$sLabelOfAction1 = Dict::S('UI:IPManagement:Action:Expand:IPv4Subnet:By');
		break;	
		
		case 'stimulus':
			if ($sClass == 'IPv4AddressChangeCreate')
			{
				// Check if IP has already been manually allocated
				if ($oObj->Get('ip_id') <= 0)
				{
					$sLabelOfAction1 = Dict::S('UI:IPManagement:Action:Implement:IPv4AddressChangeCreate:PickAnIp');
				}
				else
				{
					$sLabelOfAction1 = Dict::Format('UI:IPManagement:Action:Implement:IPv4AddressChangeCreate:ConfirmSelectedIP', $oObj->GetAsHTML('ip_id'));
				}
			}
			elseif ($sClass == 'IPv4SubnetChangeCreate')
			{
				// Check if Subnet has already been manually allocated
				if ($oObj->Get('subnet_id') <= 0)
				{
					$sLabelOfAction1 = Dict::S('UI:IPManagement:Action:Implement:IPv4SubnetChangeCreate:PickASubnet');
				}
				else
				{
					$sLabelOfAction1 = Dict::Format('UI:IPManagement:Action:Implement:IPv4SubnetChangeCreate:ConfirmSelectedSubnet', $oObj->GetAsHTML('subnet_id'));
				}
			}
		break;
		
		default:
		return;
	}
	$oP->add("<table>");
	
	switch ($sOperation)
	{
		case 'findspace':
			if ($sClass == 'IPv4Block')
			{
				$oP->add('<tr><td style="vertical-align:top">');
				
				// Size of space
				// Compute max possible 'CIDR aligned' space to look for, 
				//	1. Search space that can fit in block only
				$iBlockSize = $oObj->GetBlockSize();
				$iMaxSize = MaskToSize("192.0.0.0");
				while($iBlockSize <= $iMaxSize)
				{
					$iMaxSize /= 2;
				}
				$bitMask = SizeToMask($iMaxSize);
				//	2. Make sure block holds space of $iMaxSize CIDR aligned
				$iFirstIp = myip2long($oObj->Get('firstip'));
				$iLastIp = myip2long($oObj->Get('lastip'));
				if (($iFirstIp & $bitMask) != $iFirstIp)
				{
					if (($iLastIp - (($iFirstIp & $bitMask) + $bitMask + 1)) > $iMaxSize)
					{
						$iMaxSize /= 2;
						$bitMask = $bitMask >> 1;
					}
				}
				$i = SizeToBit($iMaxSize);
				if ($i < 16)
				{
					$iDefaultMask = 16;
				}
				else if ($i < 24)
				{
					$iDefaultMask = 24;
				}
				else
				{
					$iDefaultMask = 31;
				}
				// Display list of choices now
				$sAttCode = 'spacesize';
				$sInputId = $m_iFormId.'_'.'spacesize';
				$sHTMLValue = "<select id=\"$sInputId\" name=\"spacesize\">\n";
				$InputSize = MaskToSize(mylong2ip($bitMask));
				while($i <= 31)
				{
					if ($i == $iDefaultMask)
					{
						$sHTMLValue .= "<option selected='' value=\"$InputSize\">".mylong2ip($bitMask)." /$i</option>\n";
					}
					else
					{
						$sHTMLValue .= "<option value=\"$InputSize\">".mylong2ip($bitMask)." /$i</option>\n";
					}
					$InputSize /= 2;
					$bitMask = SizeToMask($InputSize);
					$i++;
				}
				$sHTMLValue .= "</select>";	
				$aDetails[] = array('label' => '<span title="">'.$sLabelOfAction1.'</span>', 'value' => $sHTMLValue);
				
				// Max number of offers
				$sInputId = $m_iFormId.'_'.'maxoffer';
				$jDefault = (array_key_exists('maxoffer', $aDefault)) ? $aDefault['maxoffer'] : DEFAULT_MAX_FREE_SPACE_OFFERS;
				$sHTMLValue = "<input id=\"$sInputId\" type=\"text\" value=\"$jDefault\" name=\"maxoffer\" maxlength=\"2\" size=\"2\">\n";
				$aDetails[] = array('label' => '<span title="">'.$sLabelOfAction2.'</span>', 'value' => $sHTMLValue);
				
				$oP->Details($aDetails);
				$oP->add('</td></tr>');
				
				// Cancell button
				$iObjId = $oObj->GetKey();
				$oP->add("<tr><td><button type=\"button\" class=\"action\" onClick=\"BackToDetails('$sClass', $iObjId)\"><span>".Dict::S('UI:Button:Cancel')."</span></button>&nbsp;&nbsp;");
				
				// Apply button
				$oP->add("&nbsp;&nbsp<button type=\"submit\" class=\"action\"><span>".Dict::S('UI:Button:Apply')."</span></button></td></tr>");
			}
			else
			{
				$oP->add('<tr><td style="vertical-align:top">');
				
				// Size of range
				$sInputId = $m_iFormId.'_'.'rangesize';
				$sHTMLValue = "<input id=\"$sInputId\" type=\"text\" name=\"rangesize\" maxlength=\"4\" size=\"4\">\n";
				$aDetails[] = array('label' => '<span title="">'.$sLabelOfAction1.'</span>', 'value' => $sHTMLValue);
				
				// Max number of offers
				$sInputId = $m_iFormId.'_'.'maxoffer';
				$jDefault = (array_key_exists('maxoffer', $aDefault)) ? $aDefault['maxoffer'] : DEFAULT_MAX_FREE_SPACE_OFFERS;
				$sHTMLValue = "<input id=\"$sInputId\" type=\"text\" value=\"$jDefault\" name=\"maxoffer\" maxlength=\"2\" size=\"2\">\n";
				$aDetails[] = array('label' => '<span title="">'.$sLabelOfAction2.'</span>', 'value' => $sHTMLValue);
				
				$oP->Details($aDetails);
				$oP->add('</td></tr>');
				
				// Cancell button
				$iObjId = $oObj->GetKey();
				$oP->add("<tr><td><button type=\"button\" class=\"action\" onClick=\"BackToDetails('$sClass', $iObjId)\"><span>".Dict::S('UI:Button:Cancel')."</span></button>&nbsp;&nbsp;");
				
				// Apply button
				$oP->add("&nbsp;&nbsp<button type=\"submit\" class=\"action\"><span>".Dict::S('UI:Button:Apply')."</span></button></td></tr>");
			}
		break;
		
		case 'shrinkblock':
		case 'expandblock':
			$oP->add('<tr><td style="vertical-align:top">');
			
			// New first IP
			$sAttCode = 'firstip';
			$sInputId = $m_iFormId.'_'.'firstip';
			$oAttDef = MetaModel::GetAttributeDef('IPv4Block', 'firstip');
			$sDefault = (array_key_exists('firstip', $aDefault)) ? $aDefault['firstip'] : '';
			$sHTMLValue = cmdbAbstractObject::GetFormElementForField($oP, 'IPv4Block', $sAttCode, $oAttDef, $sDefault, '', $sInputId, '', '', '');
			$aDetails[] = array('label' => '<span title="">'.$sLabelOfAction1.'</span>', 'value' => $sHTMLValue);
			
			// New last IP
			$sAttCode = 'lastip';
			$sInputId = $m_iFormId.'_'.'lastip';
			$oAttDef = MetaModel::GetAttributeDef('IPv4Block', 'lastip');
			$sDefault = (array_key_exists('lastip', $aDefault)) ? $aDefault['lastip'] : '';
			$sHTMLValue = cmdbAbstractObject::GetFormElementForField($oP, 'IPv4Block', $sAttCode, $oAttDef, $sDefault, '', $sInputId, '', '', '');
			$aDetails[] = array('label' => '<span title="">'.$sLabelOfAction2.'</span>', 'value' => $sHTMLValue);
			
			$oP->Details($aDetails);
			$oP->add('</td></tr>');
			
			// Cancell button
			$iObjId = $oObj->GetKey();
			$oP->add("<tr><td><button type=\"button\" class=\"action\" onClick=\"BackToDetails('$sClass', $iObjId)\"><span>".Dict::S('UI:Button:Cancel')."</span></button>&nbsp;&nbsp;");
			
			// Apply button
			$oP->add("&nbsp;&nbsp<button type=\"submit\" class=\"action\"><span>".Dict::S('UI:Button:Apply')."</span></button></td></tr>");
		break;
		
		case 'splitblock':
			$oP->add('<tr><td style="vertical-align:top">');
			// Split IP
			$sAttCode = 'ip';
			$sInputId = $m_iFormId.'_'.'ip';
			$oAttDef = MetaModel::GetAttributeDef('IPv4Address', 'ip');
			$sDefault = (array_key_exists('ip', $aDefault)) ? $aDefault['ip'] : '';
			$sHTMLValue = cmdbAbstractObject::GetFormElementForField($oP, 'IPv4Address', $sAttCode, $oAttDef, $sDefault, '', $sInputId, '', '', '');
			$aDetails[] = array('label' => '<span title="">'.$sLabelOfAction1.'</span>', 'value' => $sHTMLValue);
			
			// Name of new block
			$sInputId = $m_iFormId.'_'.'newname';
			$sDefault = (array_key_exists('newname', $aDefault)) ? $aDefault['newname'] : '';
			$sHTMLValue = "<input id=\"$sInputId\" value=\"$sDefault\" name=\"newname\">";
			$aDetails[] = array('label' => '<span title="">'.$sLabelOfAction2.'</span>', 'value' => $sHTMLValue);
			
			$oP->Details($aDetails);
			$oP->add('</td></tr>');
			
			// Cancell button
			$iObjId = $oObj->GetKey();
			$oP->add("<tr><td><button type=\"button\" class=\"action\" onClick=\"BackToDetails('$sClass', $iObjId)\"><span>".Dict::S('UI:Button:Cancel')."</span></button>&nbsp;&nbsp;");
			
			// Apply button
			$oP->add("&nbsp;&nbsp<button type=\"submit\" class=\"action\"><span>".Dict::S('UI:Button:Apply')."</span></button></td></tr>");
		break;
		
		case 'shrinksubnet':
		case 'splitsubnet':
		case 'expandsubnet':
			// Cancell button
			$iObjId = $oObj->GetKey();
			$oP->add("<tr><td><button type=\"button\" class=\"action\" onClick=\"BackToDetails('$sClass', $iObjId)\"><span>".Dict::S('UI:Button:Cancel')."</span></button>&nbsp;&nbsp;</td>");
			
			// Name of action
			$oP->add("<td class=\"label\"><span title=\"\">".$sLabelOfAction1."&nbsp;</span></td>");
			
			// Scale of action
			$sInputId = $m_iFormId.'_'.'scale_id';
			$sHTMLValue = "<td><select id=\"$sInputId\" name=\"scale_id\">\n";
			$jDefault = (array_key_exists('scale_id', $aDefault)) ? $aDefault['scale_id'] : 1;
			$j = 1;
			for($i = 1; $i <= $iScaleMax; $i++)
			{
				$j = $j * 2;
				if ($j == $jDefault)
				{
					$sHTMLValue .= "<option selected='' value=\"$j\">$j</option>\n";
				}
				else
				{
					$sHTMLValue .= "<option value=\"$j\">$j</option>\n";
				}
			}
			$sHTMLValue .= "</select></td>";	
			$oP->add($sHTMLValue);
			
			// Apply button
			$oP->add("<td>&nbsp;&nbsp<button type=\"submit\" class=\"action\"><span>".Dict::S('UI:Button:Apply')."</span></button></td></tr>");
		break;
		
		case 'stimulus':
			$sStimulus = $aDefault['stimulus'];
			$oP->add("<input type=\"hidden\" name=\"stimulus\" value=\"$sStimulus\">\n");
			$oP->add('<tr><td style="vertical-align:top">');
			$sOrgId = $oObj->Get('org_id');
			if ($sClass == 'IPv4AddressChangeCreate')
			{
				if ($oObj->Get('ip_id') <= 0)
				{
					// No IP has already been manually allocated, offer some
					// Create array of free IPs
					$oIpContainer = MetaModel::GetObject('IPv4Range', $oObj->Get('range_id'), false /* MustBeFound */);
					if (is_null($oIpContainer))
					{
						$bWithRanges = true;
						$iSubnetId = $oObj->Get('subnet_id');
						$oIpContainer = MetaModel::GetObject('IPv4Subnet', $iSubnetId, true /* MustBeFound */);
						$sIpContainerClass = 'IPv4Subnet';
						$sFirstIp = $oIpContainer->Get('ip');
						$sLastIp = $oIpContainer->Get('broadcastip');
						$oIpRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Range AS r WHERE r.subnet_id = $iSubnetId"));
						$aRangeIPs = $oIpRangeSet->GetColumnAsArray('firstip', false);
						$oIpRangeSet->Rewind();
					}
					else
					{
						$bWithRanges = false;
						$sFirstIp = $oIpContainer->Get('firstip');
						$sLastIp = $oIpContainer->Get('lastip');
					}
					$oIpRegisteredSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE INET_ATON('$sFirstIp') <= INET_ATON(i.ip) AND INET_ATON(i.ip) <= INET_ATON('$sLastIp') AND i.org_id = $sOrgId"));
					$aRegisteredIPs = $oIpRegisteredSet->GetColumnAsArray('ip', false);
					$iFirstIp = myip2long($sFirstIp);
					$iLastIp = myip2long($sLastIp);
					$sPingBeforeAssign = GetFromGlobalIPConfig('ping_before_assign', $sOrgId);
					
					$iAnIp = $bWithRanges ? ($iFirstIp + 1) : $iFirstIp;
					$i = 0;
					$iNumberOfFreeIps = 0;
					$iMaxFreeOffers = ($sPingBeforeAssign == 'ping_yes') ? DEFAULT_MAX_FREE_IP_OFFERS_WITH_PING : DEFAULT_MAX_FREE_IP_OFFERS;
					while (($iAnIp <= $iLastIp) && ($i < $iMaxFreeOffers))
					{
						$sAnIp = mylong2ip($iAnIp);
						if ($bWithRanges)
						{
							// If IP belongs to a range, skip range
							if (in_array($sAnIp, $aRangeIPs))
							{
								$oIpRangeSet->Rewind();
								$oIpRange = $oIpRangeSet->Fetch();
								while ($oIpRange->Get('firstip') != $sAnIp)
								{
									$oIpRange = $oIpRangeSet->Fetch();
								}
								$iAnIp = myip2long($oIpRange->Get('lastip'));
							}
						}
						if (!in_array($sAnIp, $aRegisteredIPs))
						{								
							// Found free IP. If required, make sure it doesn't ping (well... locally)
							if ($sPingBeforeAssign == 'ping_yes')
							{
								if (IpPings($sAnIp, TIME_TO_WAIT_FOR_PING_SHORT) == '')
								{
									// IP doesn't ping
									$aFreeIPs [$i++] = $sAnIp;
								}
							}
							else
							{
								$aFreeIPs [$i++] = $sAnIp;
							}
						}
						$iAnIp++;
					}
					$iNumberOfFreeIps = $i;
					
					// Translate it into select box
					$sInputId = $m_iFormId.'_'.'ip';
					$sHTMLValue = "<select id=\"$sInputId\" name=\"ip\">\n";
					// There is at least an IP free. Check has been done before.
					$sHTMLValue .= "<option selected='' value=\"$aFreeIPs[0]\">$aFreeIPs[0]</option>\n";
					for($i = 1; $i < $iNumberOfFreeIps; $i++)
					{
						$sHTMLValue .= "<option value=\"$aFreeIPs[$i]\">$aFreeIPs[$i]</option>\n";
					}
					$sHTMLValue .= "</select>";	
				}
				else
				{
					$sHTMLValue = "";
				}
			}
			elseif ($sClass == 'IPv4SubnetChangeCreate')
			{
				if ($oObj->Get('subnet_id') <= 0)
				{
					// No subnet has already been manually allocated, offer some
					$oBlock = MetaModel::GetObject('IPv4Block', $oObj->Get('block_id'), true /* MustBeFound */);
					$iSize = MaskToSize($oObj->Get('mask'));
					$aFreeSpace = $oBlock->GetFreeSpace($iSize, DEFAULT_MAX_FREE_SPACE_OFFERS);
					if (!empty($aFreeSpace))
					{
						// Translate list of spaces into select box
						$sInputId = $m_iFormId.'_'.'ip';
						$sHTMLValue = "<select id=\"$sInputId\" name=\"ip\">\n";
						$sFirstIp = $aFreeSpace[0]['firstip'];			
						$sHTMLValue .= "<option selected='' value=\"$sFirstIp\">$sFirstIp</option>\n";
						for ($i = 1; $i < sizeof ($aFreeSpace); $i++)
						{
							$sFirstIp = $aFreeSpace[$i]['firstip'];
							$sHTMLValue .= "<option value=\"$sFirstIp\">$sFirstIp</option>\n";
						}
						$sHTMLValue .= "</select>";	
					}
					else
					{
						$sHTMLValue = "";
					}
				}
				else
				{
					$sHTMLValue = "";
				}
			}
			$aDetails[] = array('label' => '<span title="">'.$sLabelOfAction1.'</span>', 'value' => $sHTMLValue);
			$oP->Details($aDetails);
			$oP->add('</td></tr>');
				
			// Cancel button
			$iObjId = $oObj->GetKey();
			$oP->add("<tr><td><button type=\"button\" class=\"action\" onClick=\"BackToDetails('$sClass', $iObjId)\"><span>".Dict::S('UI:Button:Cancel')."</span></button>&nbsp;&nbsp;");
			
			// Implement button
			$oP->add("&nbsp;&nbsp<button type=\"submit\" class=\"action\"><span>".Dict::S('UI:IPManagement:Action:Implement:IPObjChange:Button')."</span></button></td></tr>");
		break;	
		
		default:
		break;
	};
	
	$oP->add("</table>");
}

/*****************************************************************
 * 
 * Main user interface pages for IP Management module starts here
 *
 * ***************************************************************/
try
{
	require_once('../../approot.inc.php');
	require_once(APPROOT.'/application/application.inc.php');
	require_once(APPROOT.'/application/displayblock.class.inc.php');
	require_once(APPROOT.'/application/itopwebpage.class.inc.php');
	require_once(APPROOT.'/application/loginwebpage.class.inc.php');
	require_once(APPROOT.'/application/startup.inc.php');
	require_once(APPROOT.'/application/wizardhelper.class.inc.php');
	
	$sLoginMessage = LoginWebPage::DoLogin(); // Check user rights and prompt if needed
	$oAppContext = new ApplicationContext();
	
	// Start construction of page
	$oP = new iTopWebPage('pages/');
	$oP->set_base(utils::GetAbsoluteUrlAppRoot().'pages/');
	
	// All the following actions use advanced forms that require more javascript to be loaded
	$oP->add_linked_script("../js/json.js");
	$oP->add_linked_script("../js/forms-json-utils.js");
	$oP->add_linked_script("../js/wizardhelper.js");
	$oP->add_linked_script("../js/wizard.utils.js");
	
	$oP->add_linked_script("../js/linkswidget.js");
	$oP->add_linked_script("../js/extkeywidget.js");
	
	$oP->add_linked_script(utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/teemip-ip-mgmt.js");
	
	$operation = utils::ReadParam('operation', '');
	switch($operation)
	{
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'displaytree': // Display hierarchical tree for blocks or subnets
			$sClass = utils::ReadParam('class', '');
			$sOrgId = utils::ReadParam('org_id', '');
			// Check if right parameters have been given
			if ( empty($sClass) || empty($sOrgId))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			// Process request now
			$sClassLabel = MetaModel::GetName($sClass);
			$bWithSubnet = ($sClass == 'IPv4Subnet') ? true : false;
			
			// Set browser title
			$oP->set_title(Dict::Format('UI:IPManagement:Action:DisplayTree:'.$sClass.':PageTitle_Class'));
			
			// Set page titles
			$oP->add("<p class=\"page-header\">\n");
			$oP->add(MetaModel::GetClassIcon($sClass, true).Dict::Format('UI:IPManagement:Action:DisplayTree:'.$sClass.':Title_Class', $sClassLabel));
			$oP->add("</p>\n");
			
			$oP->add('<div class="display_block">');
			
			// Get number of records
			if ($sOrgId == ALL_ORGS)
			{
				$oSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT $sClass"));
			}
			else
			{
				$oSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT $sClass AS c WHERE c.org_id = $sOrgId"));
			}
			$sObjectsCount = Dict::Format('UI:Pagination:HeaderNoSelection', $oSet->Count());
			
			// Get actions Menu
			$iListId = $oP->GetUniqueId(); 
			$oMenuBlock = new MenuBlock($oSet->GetFilter(), 'list');
			$sActionsMenu = $oMenuBlock->GetRenderContent($oP, array(), $iListId);
			
			// Get toolkit menu
			// Remove "Add To Dashboard" submenu
			$sHtml = '<div class="itop_popup toolkit_menu" id="tk_'.$iListId.'"><ul><li><img src="../images/toolkit_menu.png"><ul>';
			$aActions = array();	
			utils::GetPopupMenuItems($oP, iPopupMenuExtension::MENU_OBJLIST_TOOLKIT, $oSet, $aActions);
			unset($aActions['UI:Menu:AddToDashboard']);
			unset($aActions['UI:Menu:ShortcutList']);
			$sHtml .= $oP->RenderPopupMenuItems($aActions);
			$sToolkitMenu = $sHtml;
			
			// Display menu line
			$sHtml = "<table style=\"width:100%;\">";
			$sHtml .= "<tr><td class=\"pagination_container\">$sObjectsCount</td><td class=\"menucontainer\">$sToolkitMenu $sActionsMenu</td></tr>";
			$sHtml .= "</table>";
			$oP->Add($sHtml);
			
			// Dump Tree(s)
			$oP->add('<table style="width:100%"><tr><td colspan="2">');
			$oP->add('<div style="vertical-align:top;" id="tree">');
			if ($sOrgId == ALL_ORGS)
			{
				$oSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT Organization"));
				while($oOrg = $oSet->Fetch())
				{
					$oP->add("<h2>".Dict::Format('UI:IPManagement:Action:DisplayTree:IPv4Block:OrgName', $oOrg->Get('name'))."</h2>\n");				
					DisplayOrgBlockTree ($oP, $oOrg->GetKey(), $bWithSubnet);
					$oP->add("<br>");
				}
			}
			else
			{
				$oOrg = MetaModel::GetObject('Organization', $sOrgId, false /* MustBeFound */);
				$oP->add("<h2>".Dict::Format('UI:IPManagement:Action:DisplayTree:IPv4Block:OrgName', $oOrg->Get('name'))."</h2>\n");				
				DisplayOrgBlockTree ($oP, $sOrgId, $bWithSubnet);
			}
			$oP->add('</td></tr></table>');
			$oP->add('</div></div>');
			$oP->add_ready_script("\$('#tree ul').treeview();\n");
		break; // End case displaytree
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'listspace': // List occupied and unoccupied space within a block
			$sClass = utils::ReadParam('class', '');
			$id = utils::ReadParam('id', '');
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id) || ($sClass != 'IPv4Block'))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			
			// Check if the object exists
			$oObj = MetaModel::GetObject($sClass, $id, false /* MustBeFound */);
			if (is_null($oObj))
			{
				$oP->set_title(Dict::S('UI:ErrorPageTitle'));
				$oP->P(Dict::S('UI:ObjectDoesNotExist'));
			}
			else
			{
				// The object can be read - Process request now
				$sClassLabel = MetaModel::GetName($sClass);
				
				// Display search bar
				$oSearch = new DBObjectSearch($sClass);
				$oBlock = new DisplayBlock($oSearch, 'search', false);
				$oBlock->Display($oP, 0);
				
				// Display action menu
				$oSingletonFilter = new DBObjectSearch($sClass);
				$oSingletonFilter->AddCondition('id', $oObj->GetKey(), '=');
				$oBlock = new MenuBlock($oSingletonFilter, 'details', false);
				$oBlock->Display($oP, -1);
				
				// Set titles
				SetPageTitles($oP, 'UI:IPManagement:Action:ListSpace:'.$sClass.':', $oObj, $sClassLabel);
				
				// Dump space
				$oP->add('<table style="width:100%"><tr><td colspan="2">');
				$oP->add('<div style="vertical-align:top;" id="tree">');
				ListSpace ($oP, $sClass, $oObj);
				$oP->add('</td></tr></table>');
				$oP->add('</div></div>'); 
				$oP->add_ready_script("\$('#tree ul').treeview();\n");
			}
		break; // End case listspace
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'findspace': // Find space within a block or a subnet
			$sClass = utils::ReadParam('class', '');
			$id = utils::ReadParam('id', '');
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id) || !(($sClass == 'IPv4Block') || ($sClass == 'IPv4Subnet')))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			
			// Check if the object exists
			$oObj = MetaModel::GetObject($sClass, $id, false /* MustBeFound */);
			if (is_null($oObj))
			{
				$oP->set_title(Dict::S('UI:ErrorPageTitle'));
				$oP->P(Dict::S('UI:ObjectDoesNotExist'));
			}
			else
			{
				// The object can be read - Process request now
				DisplayActionForm($oP, $oAppContext, $operation, $sClass, $id, $oObj);
			}
		break; // End case findspace
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'dofindspace': // Apply find space action
			$sClass = utils::ReadParam('class', '');
			$id = utils::ReadParam('id', '');
			$sTransactionId = utils::ReadPostedParam('transaction_id', '');
			
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id) || !(($sClass == 'IPv4Block') || ($sClass == 'IPv4Subnet')))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			
			// Check if the object exists
			$oObj = MetaModel::GetObject($sClass, $id, false /* MustBeFound */);
			if (is_null($oObj))
			{
				$oP->set_title(Dict::S('UI:ErrorPageTitle'));
				$oP->P(Dict::S('UI:ObjectDoesNotExist'));
			}
			else
			{
				// Make sure we don't follow the same path twice in a row.
				$sClassLabel = MetaModel::GetName($sClass);
				if (!utils::IsTransactionValid($sTransactionId, false))
				{
					$oP->set_title(Dict::Format('UI:ModificationPageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
					$oP->p("<strong>".Dict::S('UI:Error:ObjectAlreadyUpdated')."</strong>\n");
				}
				else
				{
					// Display search bar
					$oSearch = new DBObjectSearch($sClass);
					$oBlock = new DisplayBlock($oSearch, 'search', false);
					$oBlock->Display($oP, 0);
					
					// Display action menu
					$oSingletonFilter = new DBObjectSearch($sClass);
					$oSingletonFilter->AddCondition('id', $oObj->GetKey(), '=');
					$oBlock = new MenuBlock($oSingletonFilter, 'details', false);
					$oBlock->Display($oP, -1);
					
					// Set titles
					SetPageTitles($oP, 'UI:IPManagement:Action:DoFindSpace:'.$sClass.':', $oObj, $sClassLabel);
					
					// Dump space
					$iSize = utils::ReadPostedParam('spacesize', '', 'raw_data');
					$iMaxOffer = utils::ReadPostedParam('maxoffer', 'DEFAULT_MAX_FREE_SPACE_OFFERS', 'raw_data');
					$sParameter = array ('spacesize' => $iSize, 'maxoffer' => $iMaxOffer, 'status_subnet' => '', 'type' => '', 'location_id' => '', 'requestor_id' => '');
					
					$oP->add('<table style="width:100%"><tr><td colspan="2">');
					$oP->add('<div style="vertical-align:top;" id="tree">');
					DoFindSpace ($oP, $sClass, $oObj, 0, $sParameter);
					$oP->add('</td></tr></table>');
					$oP->add('</div></div>'); 
					$oP->add_ready_script("\$('#tree ul').treeview();\n");
					$oP->add("<div id=\"dialog_content\"/>\n");
				}
			}
		break; // End case dofindspace
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'picksubnet': // Pick a subnet for creation - not used in 1.0
			$sClass = utils::ReadParam('class', '');
			$id = utils::ReadParam('id', '');
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			
			// Check if the object exists
			$oObj = MetaModel::GetObject($sClass, $id, false /* MustBeFound */);
			if (is_null($oObj))
			{
				$oP->set_title(Dict::S('UI:ErrorPageTitle'));
				$oP->P(Dict::S('UI:ObjectDoesNotExist'));
			}
			else
			{
				// The object can be read - Process request now
				$sClassLabel = MetaModel::GetName($sClass);
				
				// Display search bar
				$oSearch = new DBObjectSearch($sClass);
				$oBlock = new DisplayBlock($oSearch, 'search', false);
				$oBlock->Display($oP, 0);
				
				// Display action menu
				$oSingletonFilter = new DBObjectSearch($sClass);
				$oSingletonFilter->AddCondition('id', $oObj->GetKey(), '=');
				$oBlock = new MenuBlock($oSingletonFilter, 'details', false);
				$oBlock->Display($oP, -1);
				
				// Set titles
				SetPageTitles($oP, 'UI:IPManagement:Action:PickSubnet:'.$sClass.':', $oObj, $sClassLabel);
				
				// Dump space
				$oBlock = MetaModel::GetObject('IPv4Block', $oObj->Get('block_id'), true /* MustBeFound */); 
				$sParameter = array ('spacesize' => MaskToSize($oObj->Get('mask')), 'maxoffer' => DEFAULT_SUBNET_CREATE_MAX_OFFER, 'status_subnet' => $oObj->Get('status_subnet'), 'type' => $oObj->Get('type'), 'location_id' => $oObj->Get('location_id'), 'requestor_id' => $oObj->Get('requestor_id'));
				
				$oP->add('<table style="width:100%"><tr><td colspan="2">');
				$oP->add('<div style="vertical-align:top;" id="tree">');
				DoFindSpace ($oP, 'IPv4Block', $oBlock, $id, $sParameter);
				$oP->add('</td></tr></table>');
				$oP->add('</div></div>'); 
				$oP->add_ready_script("\$('#tree ul').treeview();\n");
			}
		break;
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'listips': // List IPs of a subnet or a IP range
			$sClass = utils::ReadParam('class', '');
			$id = utils::ReadParam('id', '');
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			// Check if the object exists
			$oObj = MetaModel::GetObject($sClass, $id, false /* MustBeFound */);
			if (is_null($oObj))
			{
				$oP->set_title(Dict::S('UI:ErrorPageTitle'));
				$oP->P(Dict::S('UI:ObjectDoesNotExist'));
			}
			else
			{
				// The object can be read - Process request now
				$sClassLabel = MetaModel::GetName($sClass);
				
				// Display search bar
				$oSearch = new DBObjectSearch($sClass);
				$oBlock = new DisplayBlock($oSearch, 'search', false);
				$oBlock->Display($oP, 0);
				
				// Display action menu
				$oSingletonFilter = new DBObjectSearch($sClass);
				$oSingletonFilter->AddCondition('id', $oObj->GetKey(), '=');
				$oBlock = new MenuBlock($oSingletonFilter, 'details', false);
				$oBlock->Display($oP, -1);
				
				// Set titles
				SetPageTitles($oP, 'UI:IPManagement:Action:ListIps:'.$sClass.':', $oObj, $sClassLabel);
				
				// Dump IP Tree
				$sParameter = array ('status_ip' => '', 'short_name' => '', 'domain_name' => '', 'usage_id' => '', 'mac_address' => '', 'requestor_id' => '');
				$oP->add('<table style="width:100%"><tr><td colspan="2">');
				$oP->add('<div style="vertical-align:top;" id="tree">');
				ListIps ($oP, $sClass, $oObj, 0, $sParameter);
				$oP->add('</div></td></tr></table>');
				$oP->add_ready_script("\$('#tree ul').treeview();\n");
				$oP->add("<div id=\"dialog_content\"/>\n");
			}
		break; // End case Listips
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'shrinkblock':	// Shrink a block
		case 'shrinksubnet': // Shrink a subnet
			$sClass = utils::ReadParam('class', '');
			$id = utils::ReadParam('id', '');
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			// Check if the object exists
			$oObj = MetaModel::GetObject($sClass, $id, false /* MustBeFound */);
			if (is_null($oObj))
			{
				$oP->set_title(Dict::S('UI:ErrorPageTitle'));
				$oP->P(Dict::S('UI:ObjectDoesNotExist'));
			}
			else
			{
				// The object can be read - Check now that user is allowed to modify it
				$oSet = CMDBObjectSet::FromObject($oObj);
				if (UserRights::IsActionAllowed($sClass, UR_ACTION_MODIFY, $oSet) == UR_ALLOWED_NO)
				{
					throw new SecurityException('User not allowed to modify this object', array('class' => $sClass, 'id' => $id));
				}
				
				// Process request now
				DisplayActionForm($oP, $oAppContext, $operation, $sClass, $id, $oObj);
			}
		break; // End case shrink
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'applyshrinkblock': // Apply split a block
			$sClass = utils::ReadPostedParam('class', '');
			$id = utils::ReadPostedParam('id', '');
			$sTransactionId = utils::ReadPostedParam('transaction_id', '');
			
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id) || ($sClass != 'IPv4Block'))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			
			// Object does exist. It has already been checked in action 'split' but check anyway.
			// Never know in a multi user environment...
			$oObj = MetaModel::GetObject('IPv4Block', $id, true /* MustBeFound */);
			
			// Make sure we don't follow the same path twice in a row.
			$sClassLabel = MetaModel::GetName($sClass);
			if (!utils::IsTransactionValid($sTransactionId, false))
			{
				$oP->set_title(Dict::Format('UI:ModificationPageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
				$oP->p("<strong>".Dict::S('UI:Error:ObjectAlreadyUpdated')."</strong>\n");
			}
			else
			{
				// Set working variables
				$sOrgId = $oObj->Get('org_id');
				$iParentId = $oObj->Get('parent_id');
				$sFirstIpCurrentBlock = $oObj->Get('firstip');
				$iFirstIpCurrentBlock = myip2long($sFirstIpCurrentBlock);
				$sLastIpCurrentBlock = $oObj->Get('lastip');
				$iLastIpCurrentBlock = myip2long($sLastIpCurrentBlock);
				$sNewFirstIp = utils::ReadPostedParam('attr_firstip', '', 'raw_data');
				$iNewFirstIp = myip2long($sNewFirstIp);
				$sNewLastIp = utils::ReadPostedParam('attr_lastip', '', 'raw_data');
				$iNewLastIp = myip2long($sNewLastIp);
				$sRequestor_id = utils::ReadPostedParam('attr_requestor_id', null);
				
				$ContinueShrink = true;
				$sErrorString = '';
				
				// Make sure new first IPs is smaller than new last IP
				if ($iNewFirstIp >= $iNewLastIp)
				{
					$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Block:Reverted');
					$ContinueShrink = false;
				}
				// Make sure new block is contained in old one
				else if (($iNewFirstIp < $iFirstIpCurrentBlock) || ($iLastIpCurrentBlock < $iNewFirstIp) || ($iNewLastIp < $iFirstIpCurrentBlock) || ($iLastIpCurrentBlock < $iNewLastIp))
				{
					$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Block:IPOutOfBlock');
					$ContinueShrink = false;
				}
				// Make sure block is changing
				else if (($iFirstIpCurrentBlock == $iNewFirstIp) && ($iLastIpCurrentBlock == $iNewLastIp))
				{
					$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Block:NoChange');
					$ContinueShrink = false;
				}
				else
				{
					// Check that new block has minimum size
					$iBlockMinSize = $oObj->GetMinBlockSize();
					if (($iNewLastIp - $iNewFirstIp + 1) < $iBlockMinSize)
					{
						$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Block:SmallerThanMinSize', $iBlockMinSize);
						$ContinueShrink = false;
					}
					else
					{
						// Check that block is CIDR aligned
						if (!IsBlockCIDRAligned($sOrgId, $iNewFirstIp, $iNewLastIp))
						{
							$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Block:NotCIDRAligned');
							$ContinueShrink = false;
						}
						if ($ContinueShrink == true)
						{
							// Make sure that no child block sits accross border
							$oSRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Block AS b WHERE b.parent_id = '$id' AND b.org_id = $sOrgId"));
							while (($oSRange = $oSRangeSet->Fetch()) && ($ContinueShrink == true))
							{
								$iCurrentFirstIp = myip2long($oSRange->Get('firstip'));
								$iCurrentLastIp = myip2long($oSRange->Get('lastip'));
								if ((($iCurrentFirstIp < $iNewFirstIp) && ($iNewFirstIp <= $iCurrentLastIp)) || (($iCurrentFirstIp < $iNewLastIp) && ($iNewLastIp <= $iCurrentLastIp)))
								{
									$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Block:BlockAccrossBorder');
									$ContinueShrink = false;
								}
							}
							
							if ($ContinueShrink == true)
							{
								// Make sure that no child subnet sits accross border
								$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE s.block_id = '$id' AND s.org_id = $sOrgId"));
								while (($oSubnet = $oSubnetSet->Fetch()) && ($ContinueShrink == true))
								{
									$iCurrentFirstIp = myip2long($oSubnet->Get('ip'));
									$iCurrentLastIp = myip2long($oSubnet->Get('broadcastip'));
									if ((($iCurrentFirstIp < $iNewFirstIp) && ($iNewFirstIp <= $iCurrentLastIp)) || (($iCurrentFirstIp < $iNewLastIp) && ($iNewLastIp <= $iCurrentLastIp)))
									{
										$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Block:SubnetAccrossBorder');
										$ContinueShrink = false;
									}
									else if (($iParentId == 0) && (($iCurrentLastIp < $iNewFirstIp)|| ($iNewLastIp < $iCurrentFirstIp)))
									{
										$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Block:SubnetBecomesOrhpean');
										$ContinueShrink = false;
									}
								}
							}
						}
					}
				}
				
				if (!$ContinueShrink)
				{
					// Found issues, explain and give the user another chance
					$aDefault['firstip'] = $sNewFirstIp;
					$aDefault['lastip'] = $sNewLastIp;
					$aDefault['block_min_size'] = utils::ReadPostedParam('attr_block_min_size', BLOCK_MIN_SIZE);
					$aDefault['block_cidr_aligned'] = utils::ReadPostedParam('attr_block_cidr_aligned', 1);
					$aDefault['requestor_id'] = $sRequestor_id;
					DisplayActionForm($oP, $oAppContext, 'shrinkblock', 'IPv4Block', $id, $oObj, $aDefault);
					$sIssueDesc = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Block:CannotBeShrunk', $sErrorString);
					$oP->add_ready_script("alert('".addslashes($sIssueDesc)."');");
				}
				else
				{
					// Set page titles
					$oP->set_title(Dict::Format('UI:IPManagement:Action:Shrink:IPv4Block:PageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
					$oP->add("<div class=\"page_header\">\n");
					$oP->add("<h1>".$oObj->GetIcon()."&nbsp;".Dict::Format('UI:IPManagement:Action:Shrink:IPv4Block:Done', $sClassLabel, $oObj->GetName())."</h1>\n");
					$oP->add("</div>\n");
					
					// Prepare history records
					$oMyChange = MetaModel::NewObject("CMDBChange");
					$oMyChange->Set("date", time());
					$sUserString = CMDBChange::GetCurrentUserName();
					$oMyChange->Set("userinfo", $sUserString);
					$iChangeId = $oMyChange->DBInsert();
					
					// Update initial block and register it.
					if (!is_null($sRequestor_id))
					{
						$oObj->Set('requestor_id', $sRequestor_id);
					}
					$oObj->Set('firstip', $sNewFirstIp);
					$oObj->Set('lastip', $sNewLastIp);
					$oObj->DBUpdateTracked($oMyChange);
					
					//	Attach dropped child blocks to parent block
					$oSRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Block AS b WHERE b.parent_id = '$id' AND b.org_id = $sOrgId"));
					while ($oSRange = $oSRangeSet->Fetch())
					{
						$iCurrentFirstIp = myip2long($oSRange->Get('firstip'));
						$iCurrentLastIp = myip2long($oSRange->Get('lastip'));
						if (($iCurrentLastIp < $iNewFirstIp) || ($iNewLastIp < $iCurrentFirstIp))
						{
							$oSRange->Set('parent_id', $iParentId);
							$oSRange->DBUpdateTracked($oMyChange);
						}
					}
							
					//	Attach child subnets to parent block
					$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE s.block_id = '$id' AND s.org_id = $sOrgId"));
					while ($oSubnet = $oSubnetSet->Fetch())
					{
						$iCurrentFirstIp = myip2long($oSubnet->Get('ip'));
						$iCurrentLastIp = myip2long($oSubnet->Get('broadcastip'));
						if (($iCurrentLastIp < $iNewFirstIp) || ($iNewLastIp < $iCurrentFirstIp))
						{
							$oSubnet->Set('block_id', $iParentId);
							$oSubnet->DBUpdateTracked($oMyChange);
						}
					}
					
					// Display result as array
					$oSet = CMDBobjectSet::FromArray('IPv4Block', array($oObj));
					CMDBAbstractObject::DisplaySet($oP, $oSet, array('display_limit' => false, 'menu' => false));
					
					// Close transaction
					utils::RemoveTransaction($sTransactionId);
				}
			}
		break; // End case applyshrinkblock
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'applyshrinksubnet': // Apply shrink for a subnet
			$sClass = utils::ReadPostedParam('class', '');
			$id = utils::ReadPostedParam('id', '');
			$sTransactionId = utils::ReadPostedParam('transaction_id', '');
			
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id) || ($sClass != 'IPv4Subnet'))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			
			// Object does exist. It has already been checked in action 'split' but check anyway.
			// Never know in a multi user environment...
			$oObj = MetaModel::GetObject('IPv4Subnet', $id, true /* MustBeFound */);
			
			// Make sure we don't follow the same path twice in a row.
			$sClassLabel = MetaModel::GetName($sClass);
			if (!utils::IsTransactionValid($sTransactionId, false))
			{
				$oP->set_title(Dict::Format('UI:ModificationPageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
				$oP->p("<strong>".Dict::S('UI:Error:ObjectAlreadyUpdated')."</strong>\n");
			}
			else
			{
				// Set working variables
				$sOrgId = $oObj->Get('org_id');
				$iSubnetKey = $id;
				$sIpSubnetToShrink = $oObj->Get('ip');
				$iIpSubnetToShrink = myip2long($sIpSubnetToShrink);
				$sMaskSubnetToShrink = $oObj->Get('mask');
				$iMaskSubnetToShrink = myip2long($sMaskSubnetToShrink);
				$sIpBroadcastSubnetToShrink = $oObj->Get('broadcastip');
				$iShrink = utils::ReadPostedParam('scale_id', 1);
				$sRequestor_id = utils::ReadPostedParam('attr_requestor_id', null);
				
				// Confirm that subnet can be shrunk as requested (protection against forged urls)
				$ContinueShrink = true;
				$sErrorString = '';
				switch ($sMaskSubnetToShrink)
				{
					case '255.255.255.255':
					case '255.255.255.254':
						// To small to be shrunk. Minimum size is /30
						$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Subnet:SizeTooSmall');
						$ContinueShrink = false;
					break;
					
					case '255.255.255.252':
						// A /30 can only be shrunk by 2
						if ($iShrink > 2)
						{
							$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Subnet:SizeTooSmallBy', $iShrink);
							$ContinueShrink = false;
						}
					break;
					
					case '255.255.255.248':
						// A /29 can be shrunk by 2 or 4
						if ($iShrink > 4)
						{
							$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Subnet:SizeTooSmallBy', $iShrink);
							$ContinueShrink = false;
						}
					break;
					
					case '255.255.255.240':
						// A /28 can be shrunk by 2, 4 or 8
						if ($iShrink > 8)
						{
							$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Subnet:SizeTooSmallBy', $iShrink);
							$ContinueShrink = false;
						}
					break;
					
					default:
						// All other subnets can be shrunk by 2, 4, 8 or 16
					break;		
				}
				
				if ($ContinueShrink)
				{	 
					switch($iShrink)
					{
						case 2:
						default:
							$iMaskNewSubnet = ip2long(long2ip($iMaskSubnetToShrink)) >> 1; // Shrink by 2 = shift bits by 1 to the right
						break;
						
						case 4:
							$iMaskNewSubnet = ip2long(long2ip($iMaskSubnetToShrink)) >> 2; // Shrink by 4 = shift bits by 2 to the right
						break;
						
						case 8:
							$iMaskNewSubnet = ip2long(long2ip($iMaskSubnetToShrink)) >> 3; // Shrink by 8 = shift bits by 3 to the right
						break;
						
						case 16:
							$iMaskNewSubnet = ip2long(long2ip($iMaskSubnetToShrink)) >> 4; // Shrink by 16 = shift bits by 4 to the right
						break;
					}
					$sMaskNewSubnet = mylong2ip($iMaskNewSubnet);
					$iIpBroadcastNewSubnet = $iIpSubnetToShrink + MaskToSize($sMaskNewSubnet) - 1;
					$sIpBroadcastNewSubnet = mylong2ip($iIpBroadcastNewSubnet);
					
					// Check that no IP range within subnet sits across future border or becomes orphean
					$oIpRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Range AS r WHERE r.subnet_id = $iSubnetKey"));
					while (($oIpRange = $oIpRangeSet->Fetch()) && ($ContinueShrink))
					{
						$iIpRangeFirstIp = myip2long($oIpRange->Get('firstip'));
						$iIpRangeLastIp = myip2long($oIpRange->Get('lastip'));
						if (($iIpRangeFirstIp < $iIpBroadcastNewSubnet) && ($iIpRangeLastIp > $iIpBroadcastNewSubnet))
						{
							// IP range sits accross future border
							$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Subnet:IPRangeInTheMiddle', $oIpRange->Get('range'), $oIpRange->Get('firstip'), $oIpRange->Get('lastip'));
							$ContinueShrink = false;
						}
						else
						if ($iIpBroadcastNewSubnet <= $iIpRangeFirstIp)
						{
							// IP range is becoming orphean
							$sErrorString = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Subnet:IPRangeDropped', $oIpRange->Get('range'), $oIpRange->Get('firstip'), $oIpRange->Get('lastip'));
							$ContinueShrink = false;
						}
					}
				}
				
				if (!$ContinueShrink)
				{
					// Found issues, explain and give the user another chance
					$aDefault['scale_id'] = $iShrink;
					$aDefault['requestor_id'] = $sRequestor_id;
					DisplayActionForm($oP, $oAppContext, 'shrinksubnet', 'IPv4Subnet', $id, $oObj, $aDefault);
					$sIssueDesc = Dict::Format('UI:IPManagement:Action:Shrink:IPv4Subnet:CannotBeShrunk', $sErrorString);
					$oP->add_ready_script("alert('".addslashes($sIssueDesc)."');");
				}
				else
				{
					// Set page titles
					$oP->set_title(Dict::Format('UI:IPManagement:Action:Shrink:IPv4Subnet:PageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
					$oP->add("<div class=\"page_header\">\n");
					$oP->add("<h1>".$oObj->GetIcon()."&nbsp;".Dict::Format('UI:IPManagement:Action:Shrink:IPv4Subnet:Done', $sClassLabel, $oObj->GetName(), $iShrink)."</h1>\n");
					$oP->add("</div>\n");
					
					// Prepare history records
					$oMyChange = MetaModel::NewObject("CMDBChange");
					$oMyChange->Set("date", time());
					$sUserString = CMDBChange::GetCurrentUserName();
					$oMyChange->Set("userinfo", $sUserString);
					$iChangeId = $oMyChange->DBInsert();
					
					// Update initial subnet and register it.
					if (!is_null($sRequestor_id))
					{
						$oObj->Set('requestor_id', $sRequestor_id);
					}
					$oObj->Set('mask', $sMaskNewSubnet);
					$oObj->Set('broadcastip', $sIpBroadcastNewSubnet);
//					$oObj->SetWriteReason(ACTION_SHRINK);
					$oObj->Set('write_reason', 'shrink');
					$oObj->DBUpdateTracked($oMyChange);
					
					// Delete old broadcast IP
					// Creation of missing broadcast IP is done by IPv4Subnet::AfterUpdate
					$oIp = MetaModel::GetObjectFromOQL("SELECT IPv4Address AS i WHERE i.ip = '$sIpBroadcastSubnetToShrink' AND i.org_id = $sOrgId", null, false);
					if (!is_null($oIp))
					{
						$oIp->DBDeleteTracked($oMyChange);	
					}
					
					// Display result as array
					$oSet = CMDBobjectSet::FromArray('IPv4Subnet', array($oObj));
					CMDBAbstractObject::DisplaySet($oP, $oSet, array('display_limit' => false, 'menu' => false));
					
					// Close transaction
					utils::RemoveTransaction($sTransactionId);
				}
			}
		break; // End case applyshrinksubnet
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'splitblock':	// Split a block
		case 'splitsubnet': // Split a subnet
			$sClass = utils::ReadParam('class', '');
			$id = utils::ReadParam('id', '');
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			// Check if the object exists
			$oObj = MetaModel::GetObject($sClass, $id, false /* MustBeFound */);
			if (is_null($oObj))
			{
				$oP->set_title(Dict::S('UI:ErrorPageTitle'));
				$oP->P(Dict::S('UI:ObjectDoesNotExist'));
			}
			else
			{
				// The object can be read - Check now that user is allowed to modify it
				$oSet = CMDBObjectSet::FromObject($oObj);
				if (UserRights::IsActionAllowed($sClass, UR_ACTION_MODIFY, $oSet) == UR_ALLOWED_NO)
				{
					throw new SecurityException('User not allowed to modify this object', array('class' => $sClass, 'id' => $id));
				}
				
				// Process request now
				DisplayActionForm($oP, $oAppContext, $operation, $sClass, $id, $oObj);
			}
		break; // End case split...
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'applysplitblock': // Apply split a block
			$sClass = utils::ReadPostedParam('class', '');
			$id = utils::ReadPostedParam('id', '');
			$sTransactionId = utils::ReadPostedParam('transaction_id', '');
			
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id) || ($sClass != 'IPv4Block'))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			
			// Object does exist. It has already been checked in action 'split' but check anyway.
			// Never know in a multi user environment...
			$oObj = MetaModel::GetObject('IPv4Block', $id, true /* MustBeFound */);
			
			// Make sure we don't follow the same path twice in a row.
			$sClassLabel = MetaModel::GetName($sClass);
			if (!utils::IsTransactionValid($sTransactionId, false))
			{
				$oP->set_title(Dict::Format('UI:ModificationPageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
				$oP->p("<strong>".Dict::S('UI:Error:ObjectAlreadyUpdated')."</strong>\n");
			}
			else
			{
				// Set working variables
				$sOrgId = $oObj->Get('org_id');
				$sFirstIpCurrentBlock = $oObj->Get('firstip');
				$iFirstIpCurrentBlock = myip2long($sFirstIpCurrentBlock);
				$sLastIpCurrentBlock = $oObj->Get('lastip');
				$iLastIpCurrentBlock = myip2long($sLastIpCurrentBlock);
				$sSplitIp = utils::ReadPostedParam('attr_ip', '', 'raw_data');
				$iSplitIp = myip2long($sSplitIp);
				$sNewName = utils::ReadPostedParam('newname', '', 'raw_data');
				$sRequestor_id = utils::ReadPostedParam('attr_requestor_id', null);
				
				$ContinueSplit = true;
				$sErrorString = '';
				
				// Make sure split Ip is in block
				if (($iSplitIp <= $iFirstIpCurrentBlock) || ($iLastIpCurrentBlock <= $iSplitIp))
				{
					$sErrorString = Dict::Format('UI:IPManagement:Action:Split:IPv4Block:IPOutOfBlock');
					$ContinueSplit = false;
				}
				else
				{
					// Check that new blocks have minimum size
					$iBlockMinSize = $oObj->GetMinBlockSize();
					if ((($iSplitIp - $iFirstIpCurrentBlock) < $iBlockMinSize) || (($iLastIpCurrentBlock - $iSplitIp) < $iBlockMinSize))
					{
						$sErrorString = Dict::Format('UI:IPManagement:Action:Split:IPv4Block:SmallerThanMinSize', $iBlockMinSize);
						$ContinueSplit = false;
					}
					else
					{
						// Check that block is CIDR aligned
						if (!IsBlockCIDRAligned($sOrgId, $iFirstIpCurrentBlock, $iSplitIp - 1) || !IsBlockCIDRAligned($sOrgId, $iSplitIp, $iLastIpCurrentBlock))
						{
							$sErrorString = Dict::Format('UI:IPManagement:Action:Split:IPv4Block:NotCIDRAligned');
							$ContinueSplit = false;
						} 
						if ($ContinueSplit == true)
						{
							// Make sure that no child block sits accross border
							$oSRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Block AS b WHERE b.parent_id = '$id' AND b.org_id = $sOrgId"));
							while (($oSRange = $oSRangeSet->Fetch()) && ($ContinueSplit == true))
							{
								$iCurrentFirstIp = myip2long($oSRange->Get('firstip'));
								$iCurrentLastIp = myip2long($oSRange->Get('lastip'));
								if (($iCurrentFirstIp < $iSplitIp) && ($iSplitIp <= $iCurrentLastIp))
								{
									$sErrorString = Dict::Format('UI:IPManagement:Action:Split:IPv4Block:BlockAccrossBorder');
									$ContinueSplit = false;
								}
							}
							
							if ($ContinueSplit == true)
							{
								// Make sure that no child subnet sits accross border
								$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE s.block_id = '$id' AND s.org_id = $sOrgId"));
								while (($oSubnet = $oSubnetSet->Fetch()) && ($ContinueSplit == true))
								{
									$iCurrentFirstIp = myip2long($oSubnet->Get('ip'));
									$iCurrentLastIp = myip2long($oSubnet->Get('broadcastip'));
									if (($iCurrentFirstIp < $iSplitIp) && ($iSplitIp <= $iCurrentLastIp))
									{
										$sErrorString = Dict::Format('UI:IPManagement:Action:Split:IPv4Block:SubnetAccrossBorder');
										$ContinueSplit = false;
									}
								}
							}
							
							if ($ContinueSplit == true)
							{
								// Check new name doesn't already exist
								if ($sNewName == '')
								{
									$sErrorString = Dict::Format('UI:IPManagement:Action:Split:IPv4Block:EmptyNewName');
									$ContinueSplit = false;
								}
								else
								{
									$oSRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Block AS b WHERE b.name = '$sNewName' AND b.org_id = $sOrgId"));
									while ($oSRange = $oSRangeSet->Fetch())
									{
										// Skip check with self (DB copy) if necessary
										if ($oSRange->GetKey() != $id)
										{
										$sErrorString = Dict::Format('UI:IPManagement:Action:Split:IPv4Block:NameExist');
										$ContinueSplit = false;
										}
									}
								}
							}
						}
					}
				}
				
				if (!$ContinueSplit)
				{
					// Found issues, explain and give the user another chance
					$aDefault['ip'] = $sSplitIp;
					$aDefault['newname'] = $sNewName;
					$aDefault['block_min_size'] = utils::ReadPostedParam('attr_block_min_size', BLOCK_MIN_SIZE);
					$aDefault['block_cidr_aligned'] = utils::ReadPostedParam('attr_block_cidr_aligned', 1);
					$aDefault['requestor_id'] = $sRequestor_id;
					DisplayActionForm($oP, $oAppContext, 'splitblock', 'IPv4Block', $id, $oObj, $aDefault);
					$sIssueDesc = Dict::Format('UI:IPManagement:Action:Split:IPv4Block:CannotBeSplit', $sErrorString);
					$oP->add_ready_script("alert('".addslashes($sIssueDesc)."');");
				}
				else
				{
					// Set page titles
					$oP->set_title(Dict::Format('UI:IPManagement:Action:Split:IPv4Block:PageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
					$oP->add("<div class=\"page_header\">\n");
					$oP->add("<h1>".$oObj->GetIcon()."&nbsp;".Dict::Format('UI:IPManagement:Action:Split:IPv4Block:Done', $sClassLabel, $oObj->GetName())."</h1>\n");
					$oP->add("</div>\n");
					
					// Prepare history records
					$oMyChange = MetaModel::NewObject("CMDBChange");
					$oMyChange->Set("date", time());
					$sUserString = CMDBChange::GetCurrentUserName();
					$oMyChange->Set("userinfo", $sUserString);
					$iChangeId = $oMyChange->DBInsert();
					
					// Update initial block and register it.
					if (!is_null($sRequestor_id))
					{
						$oObj->Set('requestor_id', $sRequestor_id);
					}
					$oObj->Set('lastip', mylong2ip($iSplitIp - 1));
//					$oObj->SetWriteReason(ACTION_SPLIT);
					$oObj->Set('write_reason', 'split');
					$oObj->DBUpdateTracked($oMyChange);
					
					//	Create new block
					$oNewBlock = MetaModel::NewObject('IPv4Block');
					$oNewBlock->Set('org_id', $sOrgId);
					$oNewBlock->Set('name', $sNewName);
					$oNewBlock->Set('parent_id', $oObj->Get('parent_id'));
					$oNewBlock->Set('firstip', $sSplitIp);
					$oNewBlock->Set('lastip', $sLastIpCurrentBlock);
					$oNewBlock->Set('type', $oObj->Get('type'));
					$oNewBlock->Set('comment', $oObj->Get('comment'));
					if (!is_null($sRequestor_id))
					{
						$oNewBlock->Set('requestor_id', $sRequestor_id);
					}
//					$oNewBlock->SetWriteReason(ACTION_SPLIT);
					$oNewBlock->Set('write_reason', 'split');
					$oNewBlock->DBInsertTracked($oMyChange);
					$iNewBlockKey = $oNewBlock->GetKey();
					
					// Link new block to same locations as original block
					// Get list of 'lnkIPv4BlockToLocation' objects referencing original subnet (if any))
					// Create as many 'lnkIPv4BlockToLocation' objects for new block and set parameters.
					$oBlockToLocationSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT lnkIPv4BlockToLocation AS lnk WHERE (lnk.block_id) = $id"));
					while ($oBlockToLocation = $oBlockToLocationSet->Fetch())
					{
						$oNewLocationLink = MetaModel::NewObject('lnkIPv4BlockToLocation');
						$oNewLocationLink->Set('block_id', $iNewBlockKey);
						$oNewLocationLink->Set('location_id', $oBlockToLocation->Get('location_id'));
						$oNewLocationLink->DBInsertTracked($oMyChange);
					}
					
					//	Attach child blocks to that new block
					$oSRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Block AS b WHERE b.parent_id = '$id' AND b.org_id = $sOrgId"));
					while ($oSRange = $oSRangeSet->Fetch())
					{
						$iCurrentFirstIp = myip2long($oSRange->Get('firstip'));
						if ($iSplitIp <= $iCurrentFirstIp)
						{
							$oSRange->Set('parent_id', $iNewBlockKey);
//							$oSRange->SetWriteReason(ACTION_SPLIT);
							$oSRange->Set('write_reason', 'split');
							$oSRange->DBUpdateTracked($oMyChange);
						}
					}
							
					//	Attach child subnets to that new block
					$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE s.block_id = '$id' AND s.org_id = $sOrgId"));
					while ($oSubnet = $oSubnetSet->Fetch())
					{
						$iCurrentFirstIp = myip2long($oSubnet->Get('ip'));
						if ($iSplitIp <= $iCurrentFirstIp)
						{
							$oSubnet->Set('block_id', $iNewBlockKey);
							$oSubnet->DBUpdateTracked($oMyChange);
						}
					}
					
					// Display result as array
					$oSet = CMDBobjectSet::FromArray('IPv4Block', array($oObj, $oNewBlock));
					CMDBAbstractObject::DisplaySet($oP, $oSet, array('display_limit' => false, 'menu' => false));
					
					// Close transaction
					utils::RemoveTransaction($sTransactionId);
				}
			}
		break; // End case applysplitblock
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'applysplitsubnet': // Apply split a subnet
			$sClass = utils::ReadPostedParam('class', '');
			$id = utils::ReadPostedParam('id', '');
			$sTransactionId = utils::ReadPostedParam('transaction_id', '');
			
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id) || ($sClass != 'IPv4Subnet'))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			
			// Object does exist. It has already been checked in action 'split' but check anyway.
			// Never know in a multi user environment...
			$oObj = MetaModel::GetObject('IPv4Subnet', $id, true /* MustBeFound */);
			
			// Make sure we don't follow the same path twice in a row.
			$sClassLabel = MetaModel::GetName($sClass);
			if (!utils::IsTransactionValid($sTransactionId, false))
			{
				$oP->set_title(Dict::Format('UI:ModificationPageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
				$oP->p("<strong>".Dict::S('UI:Error:ObjectAlreadyUpdated')."</strong>\n");
			}
			else
			{
				// Set working variables
				$sOrgId = $oObj->Get('org_id');
				$iSubnetKey = $oObj->GetKey();
				$sIpSubnetToSplit = $oObj->Get('ip');
				$iIpSubnetToSplit = myip2long($sIpSubnetToSplit);
				$sMaskSubnetToSplit = $oObj->Get('mask');
				$iMaskSubnetToSplit = myip2long($sMaskSubnetToSplit);
				$iSplit = utils::ReadPostedParam('scale_id', 1);
				$sRequestor_id = utils::ReadPostedParam('attr_requestor_id', null);
			
				// Confirm that subnet can be split as requested (protection against forged urls)
				$ContinueSplit = true;
				$sErrorString = '';
				switch ($sMaskSubnetToSplit)
				{
					case '255.255.255.255':
					case '255.255.255.254':
						// To small to be split. Minimum size is /30
						$sErrorString = Dict::Format('UI:IPManagement:Action:Split:IPv4Subnet:SizeTooSmall');
						$ContinueSplit = false;
					break;
					
					case '255.255.255.252':
						// A /30 can only be split by 2
						if ($iSplit > 2)
						{
							$sErrorString = Dict::Format('UI:IPManagement:Action:Split:IPv4Subnet:SizeTooSmallBy', $iSplit);
							$ContinueSplit = false;
						}
					break;
					
					case '255.255.255.248':
						// A /29 can be split by 2 or 4
						if ($iSplit > 4)
						{
							$sErrorString = Dict::Format('UI:IPManagement:Action:Split:IPv4Subnet:SizeTooSmallBy', $iSplit);
							$ContinueSplit = false;
						}
					break;
					
					case '255.255.255.240':
						// A /28 can be split by 2, 4 or 8
						if ($iSplit > 8)
						{
							$sErrorString = Dict::Format('UI:IPManagement:Action:Split:IPv4Subnet:SizeTooSmallBy', $iSplit);
							$ContinueSplit = false;
						}
					break;
					
					default:
						// All other subnets can be split by 2, 4, 8 or 16
					break;		
				}
				if ($ContinueSplit)
				{ 
					switch($iSplit)
					{
						case 2:
						default:
							$iMaskNewSubnet = ip2long(long2ip($iMaskSubnetToSplit)) >> 1; // Split by 2 = shift bits by 1 to the right
						break;
						
						case 4:
							$iMaskNewSubnet = ip2long(long2ip($iMaskSubnetToSplit)) >> 2; // Split by 4 = shift bits by 2 to the right
						break;
						
						case 8: 
							$iMaskNewSubnet = ip2long(long2ip($iMaskSubnetToSplit)) >> 3; // Split by 8 = shift bits by 3 to the right
						break;
						
						case 16: 
							$iMaskNewSubnet = ip2long(long2ip($iMaskSubnetToSplit)) >> 4; // Split by 16 = shift bits by 4 to the right
						break;
					}
					$sMaskNewSubnet = mylong2ip($iMaskNewSubnet);
					$iSizeNewSubnet = MaskToSize($sMaskNewSubnet);
					
					// Check that no IP range within subnet sits across future borders
					$oIpRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Range AS r WHERE r.subnet_id = $iSubnetKey"));
					while (($oIpRange = $oIpRangeSet->Fetch()) && ($ContinueSplit))
					{
						$iIpRangeFirstIp = myip2long($oIpRange->Get('firstip'));
						$iIpRangeLastIp = myip2long($oIpRange->Get('lastip'));
						$iIpNew = $iIpSubnetToSplit + $iSizeNewSubnet;
						// Find 1st subnet IP after 1st IP of range
						while ($iIpNew <= $iIpRangeFirstIp)
						{
							$iIpNew += $iSizeNewSubnet;
						}
						// If last IP of range not in new subnet boundary, cancel split operation
						if ($iIpNew <= $iIpRangeLastIp) 
						{
							$sErrorString = Dict::Format('UI:IPManagement:Action:Split:IPv4Subnet:IPRangeInTheMiddle', $oIpRange->Get('range'), $oIpRange->Get('firstip'), $oIpRange->Get('lastip'));
							$ContinueSplit = false;
						}
					}
				}
				
				if (!$ContinueSplit)
				{
					// Found issues, explain and give the user another chance
					$aDefault['scale_id'] = $iSplit;
					$aDefault['requestor_id'] = $sRequestor_id;
					DisplayActionForm($oP, $oAppContext, 'splitsubnet', 'IPv4Subnet', $id, $oObj, $aDefault);
					$sIssueDesc = Dict::Format('UI:IPManagement:Action:Split:IPv4Subnet:CannotBeSplit', $sErrorString);
					$oP->add_ready_script("alert('".addslashes($sIssueDesc)."');");
				}
				else
				{
					// Set page titles
					$oP->set_title(Dict::Format('UI:IPManagement:Action:Split:IPv4Subnet:PageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
					$oP->add("<div class=\"page_header\">\n");
					$oP->add("<h1>".$oObj->GetIcon()."&nbsp;".Dict::Format('UI:IPManagement:Action:Split:IPv4Subnet:Done', $sClassLabel, $oObj->GetName(), $iSplit)."</h1>\n");
					$oP->add("</div>\n");
					
					// Prepare history records
					$oMyChange = MetaModel::NewObject("CMDBChange");
					$oMyChange->Set("date", time());
					$sUserString = CMDBChange::GetCurrentUserName();
					$oMyChange->Set("userinfo", $sUserString);
					$iChangeId = $oMyChange->DBInsert();
					
					// Update initial subnet and register it.
					if (!is_null($sRequestor_id))
					{
						$oObj->Set('requestor_id', $sRequestor_id);
					}
					$oObj->Set('mask', $sMaskNewSubnet);
					$oObj->Set('broadcastip', mylong2ip($iIpSubnetToSplit + $iSizeNewSubnet - 1));
//					$oObj->SetWriteReason(ACTION_SPLIT);
					$oObj->Set('write_reason', 'split');
					$oObj->DBUpdateTracked($oMyChange);
					
					// Create ($iSplit - 1) new split subnet in continuity of 1st one
					// Copy all parameters from 1st subnet but IP and mask
					// IP = First IP + (0x0...010...0)*$i - 1 is last bit of new mask
					// Ex 10.1.192.0 = 10.1.0.0 + (0.0.192.0)
					$oNewObj = array();
					$oNewObj[0] = $oObj;
					$iIpNew = $iIpSubnetToSplit + $iSizeNewSubnet;
					$sBlockId = $oObj->Get('block_id');
					$sStatus = $oObj->Get('status');
					$sType = $oObj->Get('type');
					$sComment = $oObj->Get('comment');
					$sRequestor_id = $oObj->Get('requestor_id');
					for ($i = 1; $i < $iSplit; $i++)
					{
						$oNewObj[$i] = MetaModel::NewObject('IPv4Subnet');
						$oNewObj[$i]->Set('org_id', $sOrgId);
						$oNewObj[$i]->Set('ip', mylong2ip($iIpNew));	
						$oNewObj[$i]->Set('mask', $sMaskNewSubnet);
						$oNewObj[$i]->Set('broadcastip', mylong2ip($iIpNew + $iSizeNewSubnet - 1));
						$oNewObj[$i]->Set('block_id', $sBlockId);
						$oNewObj[$i]->Set('status', $sStatus);
						$oNewObj[$i]->Set('type', $sType);
						$oNewObj[$i]->Set('comment', $sComment);
						$oNewObj[$i]->Set('requestor_id', $sRequestor_id);
						$oNewObj[$i]->DBInsertTracked($oMyChange);
						$iIpNew += $iSizeNewSubnet;
					}
					
					// Link subnets to same locations as original subnet
					// Get list of 'lnkIPv4SubnetToLocation' objects referencing original subnet (if any))
					// Create as many 'lnkIPv4SubnetToLocation' objects for each new subnet and set parameters.
					$oSubnetToLocationSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT lnkIPv4SubnetToLocation AS l WHERE l.subnet_id = $iSubnetKey"));
					while ($oSubnetToLocation = $oSubnetToLocationSet->Fetch())
					{
						for ($i = 1; $i < $iSplit; $i++)
						{
							$oNewLocationLink = MetaModel::NewObject('lnkIPv4SubnetToLocation');
							$oNewLocationLink->Set('subnet_id', $oNewObj[$i]->GetKey());
							$oNewLocationLink->Set('location_id', $oSubnetToLocation->Get('location_id'));
							$oNewLocationLink->DBInsertTracked($oMyChange);
						}
					}
					
					// Update ranges (if any) with new subnet parameter
					$oIpRangeSet->Rewind();
					while ($oIpRange = $oIpRangeSet->Fetch())
					{
						$iIpRangeFirstIp = myip2long($oIpRange->Get('firstip'));
						$iIpRangeLastIp = myip2long($oIpRange->Get('lastip'));
						$iIpNew = $iIpSubnetToSplit;
						while ($iIpRangeLastIp >= ($iIpNew + $iSizeNewSubnet))
						{
							$iIpNew += $iSizeNewSubnet;
						}
						// Find subnet in array of newly created subnets
						$sIpNew = mylong2ip($iIpNew);
						for ($i = 0; (($i < $iSplit) && ($oNewObj[$i]->Get('ip') != $sIpNew)); $i++) {}
						$oIpRange->Set('subnet_id', $oNewObj[$i]->GetKey());
						$oIpRange->DBUpdateTracked($oMyChange);
					}
					
					// Creation of missing broadcast and subnet IPs is done by IPv4Subnet::AfterInsert or IPv4Subnet::AfterUpdate
					
					// Display result as array
					$oSet = CMDBobjectSet::FromArray('IPv4Subnet', $oNewObj);
					CMDBAbstractObject::DisplaySet($oP, $oSet, array('display_limit' => false, 'menu' => false));
					
					// Close transaction
					utils::RemoveTransaction($sTransactionId);
				}
			}
		break; // End case applysplitsubnet
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'expandblock':	 // Expand a block	 
		case 'expandsubnet':	// Expand a subnet 
			$sClass = utils::ReadParam('class', '');
			$id = utils::ReadParam('id', '');
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			// Check if the object exists
			$oObj = MetaModel::GetObject($sClass, $id, false /* MustBeFound */);
			if (is_null($oObj))
			{
				$oP->set_title(Dict::S('UI:ErrorPageTitle'));
				$oP->P(Dict::S('UI:ObjectDoesNotExist'));
			}
			else
			{
				// The object can be read - Check now that user is allowed to modify it
				$oSet = CMDBObjectSet::FromObject($oObj);
				if (UserRights::IsActionAllowed($sClass, UR_ACTION_MODIFY, $oSet) == UR_ALLOWED_NO)
				{
					throw new SecurityException('User not allowed to modify this object', array('class' => $sClass, 'id' => $id));
				}
				
				// Process request now
				DisplayActionForm($oP, $oAppContext, $operation, $sClass, $id, $oObj);
			}
		break; // End case expand
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'applyexpandblock': // Apply split a block
			$sClass = utils::ReadPostedParam('class', '');
			$id = utils::ReadPostedParam('id', '');
			$sTransactionId = utils::ReadPostedParam('transaction_id', '');
			
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id) || ($sClass != 'IPv4Block'))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			
			// Object does exist. It has already been checked in action 'expand' but check anyway.
			// Never know in a multi user environment...
			$oObj = MetaModel::GetObject('IPv4Block', $id, true /* MustBeFound */);
			
			// Make sure we don't follow the same path twice in a row.
			$sClassLabel = MetaModel::GetName($sClass);
			if (!utils::IsTransactionValid($sTransactionId, false))
			{
				$oP->set_title(Dict::Format('UI:ModificationPageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
				$oP->p("<strong>".Dict::S('UI:Error:ObjectAlreadyUpdated')."</strong>\n");
			}
			else
			{
				// Set working variables
				$sOrgId = $oObj->Get('org_id');
				$iKey = $oObj->GetKey();
				$iParentId = $oObj->Get('parent_id');
				$sFirstIpCurrentBlock = $oObj->Get('firstip');
				$iFirstIpCurrentBlock = myip2long($sFirstIpCurrentBlock);
				$sLastIpCurrentBlock = $oObj->Get('lastip');
				$iLastIpCurrentBlock = myip2long($sLastIpCurrentBlock);
				$sNewFirstIp = utils::ReadPostedParam('attr_firstip', '', 'raw_data');
				$iNewFirstIp = myip2long($sNewFirstIp);
				$sNewLastIp = utils::ReadPostedParam('attr_lastip', '', 'raw_data');
				$iNewLastIp = myip2long($sNewLastIp);
				$sRequestor_id = utils::ReadPostedParam('attr_requestor_id', null);
				
				$ContinueExpand = true;
				$sErrorString = '';
				
				// Make sure new first IPs is smaller than new last IP
				if ($iNewFirstIp >= $iNewLastIp)
				{
					$sErrorString = Dict::Format('UI:IPManagement:Action:Expand:IPv4Block:Reverted');
					$ContinueExpand = false;
				}
				// Make sure new block contains old one
				elseif (($iFirstIpCurrentBlock < $iNewFirstIp) || ($iNewLastIp < $iLastIpCurrentBlock))
				{
					$sErrorString = Dict::Format('UI:IPManagement:Action:Expand:IPv4Block:IPOutOfBlock');
					$ContinueExpand = false;
				}
				// Make sure block is changing
				elseif (($iFirstIpCurrentBlock == $iNewFirstIp) && ($iLastIpCurrentBlock == $iNewLastIp))
				{
					$sErrorString = Dict::Format('UI:IPManagement:Action:Expand:IPv4Block:NoChange');
					$ContinueExpand = false;
				}
				else
				{
					// Check that new block has minimum size
					$iBlockMinSize = $oObj->GetMinBlockSize();
					if (($iNewLastIp - $iNewFirstIp + 1) < $iBlockMinSize)
					{
						$sErrorString = Dict::Format('UI:IPManagement:Action:Expand:IPv4Block:SmallerThanMinSize', $iBlockMinSize);
						$ContinueExpand = false;
					}
					// Check that block is CIDR aligned
					elseif (!IsBlockCIDRAligned($sOrgId, $iNewFirstIp, $iNewLastIp))
					{
						$sErrorString = Dict::Format('UI:IPManagement:Action:Expand:IPv4Block:NotCIDRAligned');
						$ContinueExpand = false;
					}
					else
					{
						// Make sure that new blocks is still contained in its parent, if any
						if ($iParentId != 0)
						{
							$oParent = MetaModel::GetObject('IPv4Block', $iParentId, false /* MustBeFound */);
							if (!is_null($oParent))
							{
								$iParentFirstIp = myip2long($oParent->Get('firstip'));
								$iParentLastIp = myip2long($oParent->Get('lastip'));
								if (($iNewFirstIp <= $iParentFirstIp) && ($iParentLastIp <= $iNewLastIp))
								{
									$sErrorString = Dict::Format('UI:IPManagement:Action:Expand:IPv4Block:BlockBigerThanParent');
									$ContinueExpand = false;
								}
							}
						}
						
						if ($ContinueExpand)
						{
							// Make sure that no brother block sits accross new borders
							$oSRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Block AS b WHERE b.parent_id = '$iParentId' AND b.id != '$iKey' AND b.org_id = $sOrgId"));
							while (($oSRange = $oSRangeSet->Fetch()) && ($ContinueExpand == true))
							{
								$iCurrentFirstIp = myip2long($oSRange->Get('firstip'));
								$iCurrentLastIp = myip2long($oSRange->Get('lastip'));
								if ((($iCurrentFirstIp < $iNewFirstIp) && ($iNewFirstIp <= $iCurrentLastIp)) || (($iCurrentFirstIp <= $iNewLastIp) && ($iNewLastIp < $iCurrentLastIp)))
								{
									$sErrorString = Dict::Format('UI:IPManagement:Action:Expand:IPv4Block:BlockAccrossBorder');
									$ContinueExpand = false;
								}
							}
						}
							
						if ($ContinueExpand)
						{
							// Make sure that no subnet attached to the same parent block sits accros new borders
							if ($iParentId != 0)
							{
								$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE s.block_id = '$iParentId' AND s.org_id = $sOrgId"));
								while ($oSubnet = $oSubnetSet->Fetch())
								{
									$iCurrentFirstIp = myip2long($oSubnet->Get('ip'));
									$iCurrentLastIp = myip2long($oSubnet->Get('broadcastip'));
									if ((($iCurrentFirstIp < $iNewFirstIp) && ($iNewFirstIp <= $iCurrentLastIp)) || (($iCurrentFirstIp < $iNewLastIp) && ($iNewLastIp <= $iCurrentLastIp)))
									{
										$sErrorString = Dict::Format('UI:IPManagement:Action:Expand:IPv4Block:SubnetAccrossBorder');
										$ContinueExpand = false;
									}
								}
							}
						}
					}
				}
				
				if (!$ContinueExpand)
				{
					// Found issues, explain and give the user another chance
					$aDefault['firstip'] = $sNewFirstIp;
					$aDefault['lastip'] = $sNewLastIp;
					$aDefault['block_min_size'] = utils::ReadPostedParam('attr_block_min_size', BLOCK_MIN_SIZE);
					$aDefault['block_cidr_aligned'] = utils::ReadPostedParam('attr_block_cidr_aligned', 1);
					$aDefault['requestor_id'] = $sRequestor_id;
					DisplayActionForm($oP, $oAppContext, 'expandblock', 'IPv4Block', $id, $oObj, $aDefault);
					$sIssueDesc = Dict::Format('UI:IPManagement:Action:Expand:IPv4Block:CannotBeExpanded', $sErrorString);
					$oP->add_ready_script("alert('".addslashes($sIssueDesc)."');");
				}
				else
				{
					// Set page titles
					$oP->set_title(Dict::Format('UI:IPManagement:Action:Expand:IPv4Block:PageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
					$oP->add("<div class=\"page_header\">\n");
					$oP->add("<h1>".$oObj->GetIcon()."&nbsp;".Dict::Format('UI:IPManagement:Action:Expand:IPv4Block:Done', $sClassLabel, $oObj->GetName())."</h1>\n");
					$oP->add("</div>\n");
					
					// Prepare history records
					$oMyChange = MetaModel::NewObject("CMDBChange");
					$oMyChange->Set("date", time());
					$sUserString = CMDBChange::GetCurrentUserName();
					$oMyChange->Set("userinfo", $sUserString);
					$iChangeId = $oMyChange->DBInsert();
					
					// Update initial block and register it.
					if (!is_null($sRequestor_id))
					{
						$oObj->Set('requestor_id', $sRequestor_id);
					}
					$oObj->Set('firstip', $sNewFirstIp);
					$oObj->Set('lastip', $sNewLastIp);
//					$oObj->SetWriteReason(ACTION_EXPAND);
					$oObj->Set('write_reason', 'expand');
					$oObj->DBUpdateTracked($oMyChange);
					
					// Absorb brother blocks
					$oSRangeSet->Rewind();
					while ($oSRange = $oSRangeSet->Fetch())
					{
						$iCurrentFirstIp = myip2long($oSRange->Get('firstip'));
						$iCurrentLastIp = myip2long($oSRange->Get('lastip'));
						if (($iNewFirstIp <= $iCurrentFirstIp) && ($iCurrentLastIp <= $iNewLastIp))
							{
								$oSRange->Set('parent_id', $iKey);      
								$oSRange->DBUpdateTracked($oMyChange);
							}
					}
					
					//	Attach child subnets to parent block
					if ($iParentId != 0)
					{
						$oSubnetSet->Rewind();
						while ($oSubnet = $oSubnetSet->Fetch())
						{
							$iCurrentFirstIp = myip2long($oSubnet->Get('ip'));
							$iCurrentLastIp = myip2long($oSubnet->Get('broadcastip'));
							if (($iNewFirstIp <= $iCurrentFirstIp) && ($iCurrentLastIp <= $iNewLastIp))
							{
								$oSubnet->Set('block_id', $iKey);
								$oSubnet->DBUpdateTracked($oMyChange);
							}
						}
					}
					
					// Display result as array
					$oSet = CMDBobjectSet::FromArray('IPv4Block', array($oObj));
					CMDBAbstractObject::DisplaySet($oP, $oSet, array('display_limit' => false, 'menu' => false));
					
					// Close transaction
					utils::RemoveTransaction($sTransactionId);
				}
			}
		break; // End case applyexpandblock
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'applyexpandsubnet': // Apply expand a subnet
			$sClass = utils::ReadPostedParam('class', '');
			$id = utils::ReadPostedParam('id', '');
			$sTransactionId = utils::ReadPostedParam('transaction_id', '');
			
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id) || ($sClass != 'IPv4Subnet'))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			
			// Object does exist. It has already been checked in action 'expand' but check anyway.
			// Never know in a multi user environment...
			$oObj = MetaModel::GetObject('IPv4Subnet', $id, true /* MustBeFound */);
			
			$sClassLabel = MetaModel::GetName($sClass);
			if (!utils::IsTransactionValid($sTransactionId, false))
			{
				$oP->set_title(Dict::Format('UI:ModificationPageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
				$oP->p("<strong>".Dict::S('UI:Error:ObjectAlreadyUpdated')."</strong>\n");
			}
			else
			{
				// Set working variables
				$sOrgId = $oObj->Get('org_id');
				$sIpSubnetToExpand = $oObj->Get('ip');
				$iIpSubnetToExpand = myip2long($sIpSubnetToExpand);
				$sMaskSubnetToExpand = $oObj->Get('mask');
				$iMaskSubnetToExpand = myip2long($sMaskSubnetToExpand);
				$iExpand = utils::ReadPostedParam('scale_id', 1);
				$sRequestor_id = utils::ReadPostedParam('attr_requestor_id', null);
				
				// Confirm that subnet can be expanded as requested (protection against forged urls)
				$ContinueExpand = true;
				$sErrorString = '';
				if (($iMaskSubnetToExpand & myip2long('0.127.255.255')) == 0)
				{
					// To big to be expanded. Maximum size is /17 (by choice - bigger doesn't make sense))
					$sErrorString = Dict::Format('UI:IPManagement:Action:Expand:IPv4Subnet:SizeTooBigBy', $iExpand);
					$ContinueExpand = false;
				}
				switch ($sMaskSubnetToExpand)
				{
					case '255.255.128.0.':
						// A /128 can only be expanded by 2
						if ($iExpand > 2)
						{
							$sErrorString = Dict::Format('UI:IPManagement:Action:Expand:IPv4Subnet:SizeTooBigBy', $iExpand);
							$ContinueExpand = false;
						}			
					break;
							
					case '255.255.192.0':
						// A /192 can be expanded by 2 or 4
						if ($iExpand > 4)
						{
							$sErrorString = Dict::Format('UI:IPManagement:Action:Expand:IPv4Subnet:SizeTooBigBy', $iExpand);
							$ContinueExpand = false;
						}			
					break;
							
					case '255.255.224.0':
						// A /192 can be expanded by 2, 4 or 8
						if ($iExpand > 8)
						{
							$sErrorString = Dict::Format('UI:IPManagement:Action:Expand:IPv4Subnet:SizeTooBigBy', $iExpand);
							$ContinueExpand = false;
						}			
					break;
							
					default:
						// All other subnets can be expanded by 2, 4, 8 or 16
					break; 
				}
				if ($ContinueExpand)
				{
					switch($iExpand)
					{
						case 2:
						default:
							$iMaskNewSubnet = ip2long(long2ip($iMaskSubnetToExpand)) << 1; // Expand by 2 = shift bits by 1 to the left
						break;
						
						case 4:
							$iMaskNewSubnet = ip2long(long2ip($iMaskSubnetToExpand)) << 2; // Expand by 4 = shift bits by 2 to the left
						break;
						
						case 8:
							$iMaskNewSubnet = ip2long(long2ip($iMaskSubnetToExpand)) << 3; // Expand by 8 = shift bits by 3 to the left
						break;
						
						case 16:
							$iMaskNewSubnet = ip2long(long2ip($iMaskSubnetToExpand)) << 4; // Expand by 16 = shift bits by 4 to the left
						break;
					}
					$sMaskNewSubnet = mylong2ip($iMaskNewSubnet);
					$iIpNewSubnet = myip2long(long2ip(ip2long(long2ip($iIpSubnetToExpand)) & ip2long(long2ip($iMaskNewSubnet))));
					$sIpNewSubnet = mylong2ip($iIpNewSubnet);
					$iIpBroadcastNewSubnet = $iIpNewSubnet + MaskToSize($sMaskNewSubnet) - 1;
					$sIpBroadcastNewSubnet = mylong2ip($iIpBroadcastNewSubnet);
					
					// Check that new subnet is fully contained within its block. If not, cancell the action
					$oBlock = MetaModel::GetObject('IPv4Block', $oObj->Get('block_id'), true /* MustBeFound */);
					$sBlockLastIp = $oBlock->Get('lastip');
					$iBlockLastIp = myip2long($sBlockLastIp);
					if (($iIpNewSubnet < myip2long($oBlock->Get('firstip'))) || ($iBlockLastIp < $iIpBroadcastNewSubnet))
					{
						$sErrorString = Dict::Format('UI:IPManagement:Action:Expand:IPv4Subnet:NotInIPBlock');
						$ContinueExpand = false;
					}
				}
				
				if (!$ContinueExpand)
				{
					// Found issues, explain and give the user another chance
					$aDefault['scale_id'] = $iExpand;
					$aDefault['requestor_id'] = $sRequestor_id;
					DisplayActionForm($oP, $oAppContext, 'expandsubnet', 'IPv4Subnet', $id, $oObj, $aDefault);
					$sIssueDesc = Dict::Format('UI:IPManagement:Action:Expand:IPv4Subnet:CannotBeExpanded', $sErrorString);
					$oP->add_ready_script("alert('".addslashes($sIssueDesc)."');");
				}
				else
				{
					// Set page titles
					$oP->set_title(Dict::Format('UI:IPManagement:Action:Expand:IPv4Subnet:PageTitle_Object_Class', $oObj->GetName(), $sClassLabel));
					$oP->add("<div class=\"page_header\">\n");
					$oP->add("<h1>".$oObj->GetIcon()."&nbsp;".Dict::Format('UI:IPManagement:Action:Expand:IPv4Subnet:Done', $sClassLabel, $oObj->GetName(), $iExpand)."</h1>\n");
					$oP->add("</div>\n");
					
					// Prepare history records
					$oMyChange = MetaModel::NewObject("CMDBChange");
					$oMyChange->Set("date", time());
					$sUserString = CMDBChange::GetCurrentUserName();
					$oMyChange->Set("userinfo", $sUserString);
					$oMyChange->DBInsert();
					
					// List subnets currently in range of new subnet and delete them all but the one newly updated
					$iNewSubnetKey = $oObj->GetKey();
					$oSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Subnet AS s WHERE INET_ATON(s.ip) >= INET_ATON('$sIpNewSubnet') AND INET_ATON(s.ip) <= INET_ATON('$sIpBroadcastNewSubnet') AND s.org_id = $sOrgId"));
					$CreateNew = true;
					while ($oSubnet = $oSubnetSet->Fetch()) // While there is a subnet in the list
					{
						$iSubnetKey = $oSubnet->GetKey();
						// If current subnet and initial subnet are not the same
						if ($iSubnetKey != $iNewSubnetKey)
						{
							// Find all links to locations and delete them first
							$oSubnetToLocationSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT lnkIPv4SubnetToLocation AS l WHERE l.subnet_id = $iSubnetKey"));
							while ($oSubnetToLocation = $oSubnetToLocationSet->Fetch())
							{
								$oSubnetToLocation->DBDeleteTracked($oMyChange);
							}
							
							// Find all IP Ranges attached to legacy subnet and attach them to new one
							$oSubnetRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Range AS r WHERE r.subnet_id = $iSubnetKey"));
							while ($oRange = $oSubnetRangeSet->fetch())
							{
//								$oRange->SetWriteReason(ACTION_EXPAND);
								$oRange->Set('write_reason', 'expand');
								$oRange->Set('subnet_id', $iNewSubnetKey);
								$oRange->DBUpdateTracked($oMyChange);
							}
							
							// Find all subnet change tickets attached to legacy subnet and remove reference to subnet
							$oSubnetChangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4SubnetChange AS c WHERE c.subnet_id = $iSubnetKey"));
							while ($oSubnetChange = $oSubnetChangeSet->fetch())
							{
								$oSubnetChange->Set('subnet_id', 0);
								$oSubnetChange->DBUpdateTracked($oMyChange);
							}
							
							// Find all IP creation tickets attached to legacy subnet and attach them to the new one
							$oIpChangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4AddressChangeCreate AS c WHERE c.subnet_id = $iSubnetKey"));
							while ($oIpChange = $oIpChangeSet->fetch())
							{
								$oIpChange->Set('subnet_id', $iNewSubnetKey);
								$oIpChange->DBUpdateTracked($oMyChange);
							}
							
							// Delete current subnet
							$oSubnet->DBDeleteTracked($oMyChange);
						}
					}
					
					// Update initial subnet and register it.
					// This action MUST be done after deletion of potential subnets included in order to avoid rejection of change by CheckToWrite !
					if (!is_null($sRequestor_id))
					{
						$oObj->Set('requestor_id', $sRequestor_id);
					}
					$oObj->Set('rip', $sIpNewSubnet);
					$oObj->Set('ip', $sIpNewSubnet);
					$oObj->Set('mask', $sMaskNewSubnet);
					$oObj->Set('broadcastip', $sIpBroadcastNewSubnet);
//					$oObj->SetWriteReason(ACTION_EXPAND);
					$oObj->Set('write_reason', 'expand');
					$oObj->DBUpdateTracked($oMyChange);
					
					// List Subnet IPs in new subnet. Delete them all but the new subnet IP if any
					// Creation of subnet IP is done by IPv4Subnet::AfterUpdate()
					$sUsageNetworkIpId = GetIpUsageId($sOrgId, NETWORK_IP_CODE);
					$oIpSubnetSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE i.usage_id = $sUsageNetworkIpId AND INET_ATON(i.ip) >= INET_ATON('$sIpNewSubnet') AND INET_ATON(i.ip) <= INET_ATON('$sIpBroadcastNewSubnet') AND i.org_id = $sOrgId"));
					while ($oIp = $oIpSubnetSet->Fetch())
					{
						if ($oIp->Get('ip') != $sIpNewSubnet)
						{
							$oIp->DBDeleteTracked($oMyChange);	
						}
					}
					
					// List Broadcast IPs in new subnet. Delete them all but the new broadcast IP if any
					// Creation of broadcast IP is done by IPv4Subnet::AfterUpdate()
					$sUsageBroadcastIpId = GetIpUsageId($sOrgId, BROADCAST_IP_CODE);
					$oBroadcastSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE i.usage_id = $sUsageBroadcastIpId AND INET_ATON(i.ip) >= INET_ATON('$sIpNewSubnet') AND INET_ATON(i.ip) <= INET_ATON('$sIpBroadcastNewSubnet') AND i.org_id = $sOrgId"));
					while ($oIp = $oBroadcastSet->Fetch())
					{
						if ($oIp->Get('ip') != $sIpBroadcastNewSubnet)
						{
							$oIp->DBDeleteTracked($oMyChange);	
						}
					}
					
					// Display result as array
					if ($sIpSubnetToExpand != $sIpNewSubnet)
					{
						// Otherwise wrong subnet IP is displayed in array...
						$oObj = MetaModel::GetObject('IPv4Subnet', $id, true /* MustBeFound */);
					}
					$oSet = CMDBobjectSet::FromArray('IPv4Subnet', array($oObj));
					CMDBAbstractObject::DisplaySet($oP, $oSet, array('display_limit' => false, 'menu' => false));
					
					// Close transaction
					utils::RemoveTransaction($sTransactionId);
				}
			}
		break; // End case applyexpandsubnet
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'csvexportips': // Export IPs of subnet of range in csv window
			$sClass = utils::ReadParam('class', '');
			$id = utils::ReadParam('id', '');
			// Check if right parameters have been given
			if ( empty($sClass) || empty($id))
			{
				throw new ApplicationException(Dict::Format('UI:Error:2ParametersMissing', 'class', 'id'));
			}
			// Check if the object exists
			$oObj = MetaModel::GetObject($sClass, $id, false /* MustBeFound */);
			if (is_null($oObj))
			{
				$oP->set_title(Dict::S('UI:ErrorPageTitle'));
				$oP->P(Dict::S('UI:ObjectDoesNotExist'));
			}
			else
			{
				// The object can be read - Process request now
				$sClassLabel = MetaModel::GetName($sClass);
				
				// Display search bar
				$oSearch = new DBObjectSearch($sClass);
				$oBlock = new DisplayBlock($oSearch, 'search', false);
				$oBlock->Display($oP, 0);
				
				// Display action menu
				$oSingletonFilter = new DBObjectSearch($sClass);
				$oSingletonFilter->AddCondition('id', $oObj->GetKey(), '=');
				$oBlock = new MenuBlock($oSingletonFilter, 'details', false);
				$oBlock->Display($oP, -1);
				
				// Set titles
				SetPageTitles($oP, 'UI:IPManagement:Action:CsvExportIps:'.$sClass.':', $oObj, $sClassLabel);
				
				$oP->add("<div id=\"3\" class=\"display_block\">\n"); 
				$oP->add("<textarea>\n"); 
				switch ($sClass)
				{
					case 'IPv4Subnet':
						// Get list of registered IPs in subnet
						$sOrgId = $oObj->Get('org_id');
						$sFirstIp = $oObj->Get('ip'); 
						$sLastIp = $oObj->Get('broadcastip'); 
						$iFirstIp = myip2long($sFirstIp);
						$iLastIp = myip2long($sLastIp);
						$oIpRegisteredSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE INET_ATON(i.ip) >= INET_ATON('$sFirstIp') AND INET_ATON(i.ip) <= INET_ATON('$sLastIp')  AND i.org_id = $sOrgId"));
						
						// Get list of IP Ranges in subnet
						$oIpRangeSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Range AS r WHERE INET_ATON(r.firstip) >= INET_ATON('$sFirstIp') AND INET_ATON(r.lastip) <= INET_ATON('$sLastIp') AND r.org_id = $sOrgId"));
						$iCountRange = $oIpRangeSet->Count();
						
						// List exported parameters
						$sHtml = "Registered,Id";
						$aParam = array('org_id', 'ip', 'status', 'short_name', 'domain_name', 'usage_id', 'ipinterface_id', 'mac_address', 'comment', 'requestor_id');
						foreach($aParam as $sAttCode)
						{
							$sHtml .= ','.MetaModel::GetLabel('IPv4Address', $sAttCode);
						}
						$sHtml .= ",IP Range\n";
						
						// List all IPs of subnet now in IP order 
						$aIpRegistered = $oIpRegisteredSet->GetColumnAsArray('ip', false);
						$iAnIp = $iFirstIp;
						while ($iAnIp <= $iLastIp)
						{
							$sAnIp = mylong2ip($iAnIp);
							if (!in_array($sAnIp, $aIpRegistered))
							{
								$sHtml .= "no,,,".$sAnIp.",free,,,,,,,,";
							}
							else
							{
								$oIpRegisteredSet->Rewind();
								$oIpRegistered = $oIpRegisteredSet->Fetch();  
								while ($sAnIp != $oIpRegistered->Get('ip'))
								{
									$oIpRegistered = $oIpRegisteredSet->Fetch();
								}
								$sHtml .= "yes,".$oIpRegistered->GetKey().",";
								$sHtml .= $oIpRegistered->Get('org_id').",";
								$sHtml .= $oIpRegistered->Get('ip').",";
								$sHtml .= $oIpRegistered->Get('status').",";
								$sHtml .= $oIpRegistered->Get('short_name').",";
								$sHtml .= $oIpRegistered->Get('domain_name').",";
								$sHtml .= $oIpRegistered->Get('usage_id').",";
								$sHtml .= $oIpRegistered->Get('ipinterface_id').",";
								$sHtml .= $oIpRegistered->Get('mac_address').",";
								$sHtml .= $oIpRegistered->Get('comment').",";
								$sHtml .= $oIpRegistered->Get('requestor_id').",";
							}
							// Check if IP belongs to a range or not
							if ($iCountRange != 0)
							{
								$oIpRangeSet->Rewind();
								$oIpRange = $oIpRangeSet->Fetch();
								$iFoundRange = false;
								while (($oIpRange != null) && ($iFoundRange == false))
								{
									if ((myip2long($oIpRange->Get('firstip')) <= $iAnIp) && ($iAnIp <= myip2long($oIpRange->Get('lastip'))))
									{
										$iFoundRange = true;
									}
									else
									{
										$oIpRange = $oIpRangeSet->Fetch();
									}
								}
								if ($iFoundRange)
								{
									$sHtml .= $oIpRange->Get('range')."\n";
								}
								else
								{
									$sHtml .= "\n";
								}
							}
							else
							{
								$sHtml .= "\n";
							}
							$iAnIp++;
						}
					break;
					
					case 'IPv4Range':
						// Get list of registered IPs in range
						$sOrgId = $oObj->Get('org_id');
						$sFirstIp = $oObj->Get('firstip'); 
						$sLastIp = $oObj->Get('lastip'); 
						$iFirstIp = myip2long($sFirstIp);
						$iLastIp = myip2long($sLastIp);
						$oIpRegisteredSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE INET_ATON(i.ip) >= INET_ATON('$sFirstIp') AND INET_ATON(i.ip) <= INET_ATON('$sLastIp')  AND i.org_id = $sOrgId"));
						
						// List exported parameters
						$sHtml = "Registered,Id";
						$aParam = array('org_id', 'ip', 'status', 'short_name', 'domain_name', 'usage_id', 'ipinterface_id', 'mac_address', 'comment', 'requestor_id');
						foreach($aParam as $sAttCode)
						{
							$sHtml .= ','.MetaModel::GetLabel('IPv4Address', $sAttCode);
						}
						$sHtml .= "\n";
						
						// List all IPs of range now in IP order 
						$aIpRegistered = $oIpRegisteredSet->GetColumnAsArray('ip', false);
						$iAnIp = $iFirstIp;
						while ($iAnIp <= $iLastIp)
						{
							$sAnIp = mylong2ip($iAnIp);
							if (!in_array($sAnIp, $aIpRegistered))
							{
								//$oP->add($sAnIp."\n");
								$sHtml .= "no,,,".$sAnIp.",free,,,,,,,\n";
							}
							else
							{
								$oIpRegisteredSet->Rewind();
								$oIpRegistered = $oIpRegisteredSet->Fetch();  
								while ($sAnIp != $oIpRegistered->Get('ip'))
								{
									$oIpRegistered = $oIpRegisteredSet->Fetch();
								}
								$sHtml .= "yes,".$oIpRegistered->GetKey().",";
								$sHtml .= $oIpRegistered->Get('org_id').",";
								$sHtml .= $oIpRegistered->Get('ip').",";
								$sHtml .= $oIpRegistered->Get('status').",";
								$sHtml .= $oIpRegistered->Get('short_name').",";
								$sHtml .= $oIpRegistered->Get('domain_name').",";
								$sHtml .= $oIpRegistered->Get('usage_id').",";
								$sHtml .= $oIpRegistered->Get('ipinterface_id').",";
								$sHtml .= $oIpRegistered->Get('mac_address').",";
								$sHtml .= $oIpRegistered->Get('comment').",";
								$sHtml .= $oIpRegistered->Get('requestor_id')."\n";
							}
							$iAnIp++;
						}
					break;
					
					default:
					break;
				}
				$oP->add($sHtml);
				
				$oP->add("</textarea>\n");
				$oP->add("</div>\n");
				
				// Adjust the size of the block
				$oP->add_ready_script(" $('#3>textarea').height($('#3').parent().height() - 220).width( $('#3').parent().width() - 30);");
			}
		break; // End case csvexportips
		
		///////////////////////////////////////////////////////////////////////////////////////////
		
		case 'cancel': // An action was cancelled
		case 'displaylist':
		default: // Menu node rendering (templates)
			ApplicationMenu::LoadAdditionalMenus();
			$oMenuNode = ApplicationMenu::GetMenuNode(ApplicationMenu::GetMenuIndexById(ApplicationMenu::GetActiveNodeId()));
			if (is_object($oMenuNode))
			{
				$oMenuNode->RenderContent($oP, $oAppContext->GetAsHash());
				$oP->set_title($oMenuNode->GetLabel());
			}
		break;
		
	}
	$oP->output(); // Display the whole content now !
}

catch(CoreException $e)
{
	require_once(APPROOT.'/setup/setuppage.class.inc.php');
	$oP = new SetupPage(Dict::S('UI:PageTitle:FatalError'));
	if ($e instanceof SecurityException)
	{
		$oP->add("<h1>".Dict::S('UI:SystemIntrusion')."</h1>\n");
	}
	else
	{
		$oP->add("<h1>".Dict::S('UI:FatalErrorMessage')."</h1>\n");
	}	
	$oP->error(Dict::Format('UI:Error_Details', $e->getHtmlDesc()));	
	$oP->output();
	
	if (MetaModel::IsLogEnabledIssue())
	{
		if (MetaModel::IsValidClass('EventIssue'))
		{
			try
			{
				$oLog = new EventIssue();
				
				$oLog->Set('message', $e->getMessage());
				$oLog->Set('userinfo', '');
				$oLog->Set('issue', $e->GetIssue());
				$oLog->Set('impact', 'Page could not be displayed');
				$oLog->Set('callstack', $e->getTrace());
				$oLog->Set('data', $e->getContextData());
				$oLog->DBInsertNoReload();
			}
			catch(Exception $e)
			{
				IssueLog::Error("Failed to log issue into the DB");
			}
		}
		
		IssueLog::Error($e->getMessage());
	}
	
	// For debugging only
	//throw $e;
}
catch(Exception $e)
{
	require_once(APPROOT.'/setup/setuppage.class.inc.php');
	$oP = new SetupPage(Dict::S('UI:PageTitle:FatalError'));
	$oP->add("<h1>".Dict::S('UI:FatalErrorMessage')."</h1>\n");	
	$oP->error(Dict::Format('UI:Error_Details', $e->getMessage()));	
	$oP->output();
	
	if (MetaModel::IsLogEnabledIssue())
	{
		if (MetaModel::IsValidClass('EventIssue'))
		{
			try
			{
				$oLog = new EventIssue();
				
				$oLog->Set('message', $e->getMessage());
				$oLog->Set('userinfo', '');
				$oLog->Set('issue', 'PHP Exception');
				$oLog->Set('impact', 'Page could not be displayed');
				$oLog->Set('callstack', $e->getTrace());
				$oLog->Set('data', array());
				$oLog->DBInsertNoReload();
			}
			catch(Exception $e)
			{
				IssueLog::Error("Failed to log issue into the DB");
			}
		}
		
		IssueLog::Error($e->getMessage());
	}
}
?>